@@ -7,11 +7,11 @@ export SHELLOPTS
 echo "SHELLOPTS=${SHELLOPTS}"
 
 SCRIPTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
-# shellcheck disable=SC1090 # In newer shellcheck than 0.6.0, pass: "-P SCRIPTDIR" (literally)
-source "$SCRIPTDIR"/build.sh
+export ORG_GRADLE_PROJECT_useJdk17Compiler=true
+source "$SCRIPTDIR"/clone-related.sh
 
 
-"$SCRIPTDIR/.plume-scripts/git-clone-related" typetools guava
+"$SCRIPTDIR/.git-scripts/git-clone-related" typetools guava
 cd ../guava
 
 ./typecheck.sh signature
