@@ -45,6 +45,7 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutab
 import org.checkerframework.framework.util.JavaExpressionParseUtil;
 import org.checkerframework.javacutil.AnnotationBuilder;
 import org.checkerframework.javacutil.AnnotationUtils;
+import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.TreeUtils;
 
 /**
@@ -56,13 +57,16 @@ import org.checkerframework.javacutil.TreeUtils;
 public class I18nFormatterTreeUtil {
   /** The checker. */
   public final BaseTypeChecker checker;
+
   /** The processing environment. */
   public final ProcessingEnvironment processingEnv;
 
   /** The value() element/field of an @I18nFormat annotation. */
   protected final ExecutableElement i18nFormatValueElement;
+
   /** The value() element/field of an @I18nFormatFor annotation. */
   protected final ExecutableElement i18nFormatForValueElement;
+
   /** The value() element/field of an @I18nInvalidFormat annotation. */
   protected final ExecutableElement i18nInvalidFormatValueElement;
 
@@ -102,8 +106,7 @@ public class I18nFormatterTreeUtil {
    * @param invalidFormatString an invalid formatter string
    * @return an {@link I18nInvalidFormat} annotation with the given string as its value
    */
-  // package-private
-  AnnotationMirror stringToInvalidFormatAnnotation(String invalidFormatString) {
+  /*package-private*/ AnnotationMirror stringToInvalidFormatAnnotation(String invalidFormatString) {
     AnnotationBuilder builder = new AnnotationBuilder(processingEnv, I18nInvalidFormat.class);
     builder.setValue("value", invalidFormatString);
     return builder.build();
@@ -115,8 +118,7 @@ public class I18nFormatterTreeUtil {
    * @param anno an I18nInvalidFormat annotation
    * @return its value() element/field, or null if it does not have one
    */
-  /*package-visible*/
-  @Nullable String getI18nInvalidFormatValue(AnnotationMirror anno) {
+  /*package-private*/ @Nullable String getI18nInvalidFormatValue(AnnotationMirror anno) {
     return AnnotationUtils.getElementValue(anno, i18nInvalidFormatValueElement, String.class, null);
   }
 
@@ -126,7 +128,7 @@ public class I18nFormatterTreeUtil {
    * @param anno an I18NFormatFor annotation
    * @return its value() element/field
    */
-  /*package-visible*/ String getI18nFormatForValue(AnnotationMirror anno) {
+  /*package-private*/ String getI18nFormatForValue(AnnotationMirror anno) {
     return AnnotationUtils.getElementValue(anno, i18nFormatForValueElement, String.class);
   }
 
@@ -216,7 +218,8 @@ public class I18nFormatterTreeUtil {
     checker.reportWarning(res.location, msgKey, args);
   }
 
-  private I18nConversionCategory[] asFormatCallCategoriesLowLevel(MethodInvocationNode node) {
+  private I18nConversionCategory @Nullable [] asFormatCallCategoriesLowLevel(
+      MethodInvocationNode node) {
     Node vararg = node.getArgument(1);
     if (vararg instanceof ArrayCreationNode) {
       List<Node> convs = ((ArrayCreationNode) vararg).getInitializers();
@@ -273,7 +276,7 @@ public class I18nFormatterTreeUtil {
     AnnotatedExecutableType methodAnno = atypeFactory.getAnnotatedType(method);
     for (AnnotatedTypeMirror paramType : methodAnno.getParameterTypes()) {
       // find @FormatFor
-      if (paramType.getAnnotation(I18nFormatFor.class) != null) {
+      if (paramType.getPrimaryAnnotation(I18nFormatFor.class) != null) {
         return atypeFactory.treeUtil.new I18nFormatCall(tree, atypeFactory);
       }
     }
@@ -289,12 +292,16 @@ public class I18nFormatterTreeUtil {
 
     /** The AST node for the call. */
     private final MethodInvocationTree tree;
+
     /** The format string argument. */
     private ExpressionTree formatArg;
+
     /** The type factory. */
     private final AnnotatedTypeFactory atypeFactory;
+
     /** The arguments to the format string. */
     private List<? extends ExpressionTree> args;
+
     /** Extra description for error messages. */
     private String invalidMessage;
 
@@ -307,6 +314,7 @@ public class I18nFormatterTreeUtil {
      * @param tree method invocation tree
      * @param atypeFactory type factory
      */
+    @SuppressWarnings("nullness:initialization.fields.uninitialized")
     public I18nFormatCall(MethodInvocationTree tree, AnnotatedTypeFactory atypeFactory) {
       this.tree = tree;
       this.atypeFactory = atypeFactory;
@@ -347,7 +355,7 @@ public class I18nFormatterTreeUtil {
       int paramIndex = -1;
       int i = 0;
       for (AnnotatedTypeMirror paramType : methodAnno.getParameterTypes()) {
-        if (paramType.getAnnotation(I18nFormatFor.class) != null) {
+        if (paramType.getPrimaryAnnotation(I18nFormatFor.class) != null) {
           this.formatArg = theargs.get(i);
           this.formatAnno = atypeFactory.getAnnotatedType(formatArg);
 
@@ -356,7 +364,8 @@ public class I18nFormatterTreeUtil {
             return;
           }
 
-          String formatforArg = getI18nFormatForValue(paramType.getAnnotation(I18nFormatFor.class));
+          String formatforArg =
+              getI18nFormatForValue(paramType.getPrimaryAnnotation(I18nFormatFor.class));
 
           paramIndex = JavaExpressionParseUtil.parameterIndex(formatforArg);
           if (paramIndex == -1) {
@@ -383,14 +392,14 @@ public class I18nFormatterTreeUtil {
     public Result<FormatType> getFormatType() {
       FormatType type;
       if (isValidFormatForInvocation()) {
-        if (formatAnno.hasAnnotation(I18nFormat.class)) {
+        if (formatAnno.hasPrimaryAnnotation(I18nFormat.class)) {
           type = FormatType.I18NFORMAT;
-        } else if (formatAnno.hasAnnotation(I18nFormatFor.class)) {
+        } else if (formatAnno.hasPrimaryAnnotation(I18nFormatFor.class)) {
           type = FormatType.I18NFORMATFOR;
         } else {
           type = FormatType.I18NINVALID;
           invalidMessage = "(is a @I18nFormat annotation missing?)";
-          AnnotationMirror inv = formatAnno.getAnnotation(I18nInvalidFormat.class);
+          AnnotationMirror inv = formatAnno.getPrimaryAnnotation(I18nInvalidFormat.class);
           if (inv != null) {
             invalidMessage = getI18nInvalidFormatValue(inv);
           }
@@ -420,7 +429,7 @@ public class I18nFormatterTreeUtil {
       InvocationType type = InvocationType.VARARG;
 
       if (args.size() == 1) {
-        final ExpressionTree first = args.get(0);
+        ExpressionTree first = args.get(0);
         TypeMirror argType = atypeFactory.getAnnotatedType(first).getUnderlyingType();
         // figure out if argType is an array
         type =
@@ -439,16 +448,16 @@ public class I18nFormatterTreeUtil {
                     return first.accept(
                         new SimpleTreeVisitor<InvocationType, Class<Void>>() {
                           @Override
-                          protected InvocationType defaultAction(Tree node, Class<Void> p) {
+                          protected InvocationType defaultAction(Tree tree, Class<Void> p) {
                             // just a normal array
                             return InvocationType.ARRAY;
                           }
 
                           @Override
-                          public InvocationType visitTypeCast(TypeCastTree node, Class<Void> p) {
+                          public InvocationType visitTypeCast(TypeCastTree tree, Class<Void> p) {
                             // it's a (Object[])null
                             return atypeFactory
-                                        .getAnnotatedType(node.getExpression())
+                                        .getAnnotatedType(tree.getExpression())
                                         .getUnderlyingType()
                                         .getKind()
                                     == TypeKind.NULL
@@ -485,7 +494,7 @@ public class I18nFormatterTreeUtil {
      * @see I18nConversionCategory
      */
     public final I18nConversionCategory[] getFormatCategories() {
-      AnnotationMirror anno = formatAnno.getAnnotation(I18nFormat.class);
+      AnnotationMirror anno = formatAnno.getPrimaryAnnotation(I18nFormat.class);
       return formatAnnotationToCategories(anno);
     }
 
@@ -534,7 +543,7 @@ public class I18nFormatterTreeUtil {
         case DOUBLE:
           return Double.class;
         default:
-          return null;
+          throw new BugInCF("unknown primitive type " + t);
       }
     }
 
@@ -544,15 +553,15 @@ public class I18nFormatterTreeUtil {
           .accept(
               new SimpleElementVisitor8<Class<? extends Object>, Class<Void>>() {
                 @Override
-                public Class<? extends Object> visitType(TypeElement e, Class<Void> v) {
+                public Class<? extends Object> visitType(TypeElement te, Class<Void> v) {
                   try {
                     @SuppressWarnings("signature") // https://tinyurl.com/cfissue/658:
                     // Name.toString should be @PolySignature
-                    @BinaryName String cname = e.getQualifiedName().toString();
+                    @BinaryName String cname = te.getQualifiedName().toString();
                     return Class.forName(cname);
-                  } catch (ClassNotFoundException e1) {
-                    return null; // the lookup should work for all
-                    // the classes we care about
+                  } catch (ClassNotFoundException e) {
+                    // The lookup should work for all the classes we care about.
+                    throw new Error(e);
                   }
                 }
               },
@@ -569,7 +578,7 @@ public class I18nFormatterTreeUtil {
    * @param type a TypeMirror
    * @return the class corresponding to the argument
    */
-  private static final Class<? extends Object> typeMirrorToClass(final TypeMirror type) {
+  private static Class<? extends Object> typeMirrorToClass(TypeMirror type) {
     return type.accept(typeMirrorToClassVisitor, Void.TYPE);
   }
 }
