@@ -17,7 +17,6 @@ import org.checkerframework.checker.index.qual.NonNegative;
 import org.checkerframework.checker.index.qual.Positive;
 import org.checkerframework.checker.index.upperbound.OffsetEquation;
 import org.checkerframework.common.value.ValueCheckerUtils;
-import org.checkerframework.dataflow.analysis.RegularTransferResult;
 import org.checkerframework.dataflow.analysis.TransferInput;
 import org.checkerframework.dataflow.analysis.TransferResult;
 import org.checkerframework.dataflow.cfg.UnderlyingAST;
@@ -37,6 +36,7 @@ import org.checkerframework.framework.flow.CFStore;
 import org.checkerframework.framework.flow.CFValue;
 import org.checkerframework.framework.type.AnnotatedTypeFactory;
 import org.checkerframework.javacutil.AnnotationUtils;
+import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.TreeUtils;
 
 /**
@@ -161,24 +161,32 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
 
   /** The canonical {@link GTENegativeOne} annotation. */
   public final AnnotationMirror GTEN1;
+
   /** The canonical {@link NonNegative} annotation. */
   public final AnnotationMirror NN;
+
   /** The canonical {@link Positive} annotation. */
   public final AnnotationMirror POS;
+
   /** The canonical {@link LowerBoundUnknown} annotation. */
   public final AnnotationMirror UNKNOWN;
 
-  // The ATF (Annotated Type Factory).
-  private LowerBoundAnnotatedTypeFactory aTypeFactory;
+  /** The annotated type factory. */
+  private final LowerBoundAnnotatedTypeFactory atypeFactory;
 
+  /**
+   * Create a new LowerBoundTransfer.
+   *
+   * @param analysis the CFAnalysis
+   */
   public LowerBoundTransfer(CFAnalysis analysis) {
     super(analysis);
-    aTypeFactory = (LowerBoundAnnotatedTypeFactory) analysis.getTypeFactory();
+    atypeFactory = (LowerBoundAnnotatedTypeFactory) analysis.getTypeFactory();
     // Initialize qualifiers.
-    GTEN1 = aTypeFactory.GTEN1;
-    NN = aTypeFactory.NN;
-    POS = aTypeFactory.POS;
-    UNKNOWN = aTypeFactory.UNKNOWN;
+    GTEN1 = atypeFactory.GTEN1;
+    NN = atypeFactory.NN;
+    POS = atypeFactory.POS;
+    UNKNOWN = atypeFactory.UNKNOWN;
   }
 
   /**
@@ -194,7 +202,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
 
     Long integerLiteral =
         ValueCheckerUtils.getExactValue(
-            mLiteral.getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            mLiteral.getTree(), atypeFactory.getValueAnnotatedTypeFactory());
 
     if (integerLiteral == null) {
       return;
@@ -202,7 +210,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     long intLiteral = integerLiteral.longValue();
 
     if (intLiteral == 0) {
-      if (aTypeFactory.areSameByClass(otherAnno, NonNegative.class)) {
+      if (atypeFactory.areSameByClass(otherAnno, NonNegative.class)) {
         List<Node> internals = splitAssignments(otherNode);
         for (Node internal : internals) {
           JavaExpression je = JavaExpression.fromNode(internal);
@@ -210,7 +218,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
         }
       }
     } else if (intLiteral == -1) {
-      if (aTypeFactory.areSameByClass(otherAnno, GTENegativeOne.class)) {
+      if (atypeFactory.areSameByClass(otherAnno, GTENegativeOne.class)) {
         List<Node> internals = splitAssignments(otherNode);
         for (Node internal : internals) {
           JavaExpression je = JavaExpression.fromNode(internal);
@@ -240,9 +248,9 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       return result;
     }
 
-    // There is also special processing to look for literals on one side of the equals and a GTEN1
-    // or NN on the other, so that those types can be promoted in the branch where their values are
-    // not equal to certain literals.
+    // There is also special processing to look for literals on one side of the equals and a
+    // GTEN1 or NN on the other, so that those types can be promoted in the branch where their
+    // values are not equal to certain literals.
     CFStore notEqualsStore = notEqualTo ? rfi.thenStore : rfi.elseStore;
     notEqualToValue(rfi.left, rfi.right, rfi.rightAnno, notEqualsStore);
     notEqualToValue(rfi.right, rfi.left, rfi.leftAnno, notEqualsStore);
@@ -264,7 +272,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       return;
     }
     JavaExpression otherJe = JavaExpression.fromNode(otherNode);
-    if (aTypeFactory
+    if (atypeFactory
         .getLessThanAnnotatedTypeFactory()
         .isLessThanOrEqual(leftNode.getTree(), otherJe.toString())) {
       store.insertValue(otherJe, POS);
@@ -333,7 +341,9 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     JavaExpression leftJe = JavaExpression.fromNode(left);
 
     AnnotationMirror newLBType =
-        aTypeFactory.getQualifierHierarchy().greatestLowerBound(rightAnno, leftAnno);
+        atypeFactory
+            .getQualifierHierarchy()
+            .greatestLowerBoundShallow(rightAnno, left.getType(), leftAnno, right.getType());
 
     store.insertValue(leftJe, newLBType);
   }
@@ -415,7 +425,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the right side's value is known at compile time.
     Long valRight =
         ValueCheckerUtils.getExactValue(
-            rightExprNode.getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            rightExprNode.getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valRight != null) {
       return getAnnotationForLiteralPlus(valRight.intValue(), leftAnno);
     }
@@ -425,7 +435,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the left side's value is known at compile time.
     Long valLeft =
         ValueCheckerUtils.getExactValue(
-            leftExprNode.getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            leftExprNode.getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valLeft != null) {
       return getAnnotationForLiteralPlus(valLeft.intValue(), rightAnno);
     }
@@ -435,16 +445,16 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
      *      nn + * -> *
      *      pos + gte-1 -> nn
      */
-    if (aTypeFactory.areSameByClass(leftAnno, Positive.class)
-        && aTypeFactory.areSameByClass(rightAnno, Positive.class)) {
+    if (atypeFactory.areSameByClass(leftAnno, Positive.class)
+        && atypeFactory.areSameByClass(rightAnno, Positive.class)) {
       return POS;
     }
 
-    if (aTypeFactory.areSameByClass(leftAnno, NonNegative.class)) {
+    if (atypeFactory.areSameByClass(leftAnno, NonNegative.class)) {
       return rightAnno;
     }
 
-    if (aTypeFactory.areSameByClass(rightAnno, NonNegative.class)) {
+    if (atypeFactory.areSameByClass(rightAnno, NonNegative.class)) {
       return leftAnno;
     }
 
@@ -475,7 +485,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the right side's value is known at compile time.
     Long valRight =
         ValueCheckerUtils.getExactValue(
-            minusNode.getRightOperand().getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            minusNode.getRightOperand().getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valRight != null) {
       AnnotationMirror leftAnno = getLowerBoundAnnotation(minusNode.getLeftOperand(), p);
       // Instead of a separate method for subtraction, add the negative of a constant.
@@ -484,32 +494,32 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       Tree leftExpr = minusNode.getLeftOperand().getTree();
       Integer minLen = null;
       // Check if the left side is a field access of an array's length, or invocation of
-      // String.length. If so, try to look up the MinLen of the array, and potentially keep this
-      // either NN or POS instead of GTEN1 or LBU.
+      // String.length. If so, try to look up the MinLen of the array, and potentially keep
+      // this either NN or POS instead of GTEN1 or LBU.
       if (leftExpr.getKind() == Tree.Kind.MEMBER_SELECT) {
         MemberSelectTree mstree = (MemberSelectTree) leftExpr;
-        minLen = aTypeFactory.getMinLenFromMemberSelectTree(mstree);
+        minLen = atypeFactory.getMinLenFromMemberSelectTree(mstree);
       } else if (leftExpr.getKind() == Tree.Kind.METHOD_INVOCATION) {
         MethodInvocationTree mitree = (MethodInvocationTree) leftExpr;
-        minLen = aTypeFactory.getMinLenFromMethodInvocationTree(mitree);
+        minLen = atypeFactory.getMinLenFromMethodInvocationTree(mitree);
       }
 
       if (minLen != null) {
-        result = aTypeFactory.anmFromVal(minLen - valRight);
+        result = atypeFactory.anmFromVal(minLen - valRight);
       }
       return result;
     }
 
     OffsetEquation leftExpression =
-        OffsetEquation.createOffsetFromNode(minusNode.getLeftOperand(), aTypeFactory, '+');
+        OffsetEquation.createOffsetFromNode(minusNode.getLeftOperand(), atypeFactory, '+');
     if (leftExpression != null) {
-      if (aTypeFactory
+      if (atypeFactory
           .getLessThanAnnotatedTypeFactory()
           .isLessThan(minusNode.getRightOperand().getTree(), leftExpression.toString())) {
         return POS;
       }
 
-      if (aTypeFactory
+      if (atypeFactory
           .getLessThanAnnotatedTypeFactory()
           .isLessThanOrEqual(minusNode.getRightOperand().getTree(), leftExpression.toString())) {
         return NN;
@@ -559,7 +569,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       NumericalMultiplicationNode node, TransferInput<CFValue, CFStore> p) {
 
     // Special handling for multiplying an array length by a Math.random().
-    AnnotationMirror randomSpecialCaseResult = aTypeFactory.checkForMathRandomSpecialCase(node);
+    AnnotationMirror randomSpecialCaseResult = atypeFactory.checkForMathRandomSpecialCase(node);
     if (randomSpecialCaseResult != null) {
       return randomSpecialCaseResult;
     }
@@ -569,7 +579,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the right side's value is known at compile time.
     Long valRight =
         ValueCheckerUtils.getExactValue(
-            node.getRightOperand().getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            node.getRightOperand().getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valRight != null) {
       return getAnnotationForLiteralMultiply(valRight.intValue(), leftAnno);
     }
@@ -578,7 +588,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the left side's value is known at compile time.
     Long valLeft =
         ValueCheckerUtils.getExactValue(
-            node.getLeftOperand().getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            node.getLeftOperand().getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valLeft != null) {
       return getAnnotationForLiteralMultiply(valLeft.intValue(), rightAnno);
     }
@@ -619,7 +629,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       // division by zero. Division by zero is treated as bottom so that users aren't warned
       // about dead code that's dividing by zero. This code assumes that non-dead code won't
       // include literal divide by zeros...
-      return aTypeFactory.BOTTOM;
+      return atypeFactory.BOTTOM;
     } else if (val == 1) {
       return leftAnno;
     } else if (val >= 2) {
@@ -653,7 +663,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the right side's value is known at compile time.
     Long valRight =
         ValueCheckerUtils.getExactValue(
-            node.getRightOperand().getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            node.getRightOperand().getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valRight != null) {
       return addAnnotationForLiteralDivideRight(valRight.intValue(), leftAnno);
     }
@@ -663,7 +673,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the left side's value is known at compile time.
     Long valLeft =
         ValueCheckerUtils.getExactValue(
-            node.getLeftOperand().getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            node.getLeftOperand().getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valLeft != null) {
       return addAnnotationForLiteralDivideLeft(valLeft.intValue(), leftAnno);
     }
@@ -705,7 +715,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     for (VariableTree variableTree : paramTrees) {
       if (TreeUtils.typeOf(variableTree).getKind() == TypeKind.CHAR) {
         JavaExpression je = JavaExpression.fromVariableTree(variableTree);
-        info.insertValuePermitNondeterministic(je, aTypeFactory.NN);
+        info.insertValuePermitNondeterministic(je, atypeFactory.NN);
       }
     }
   }
@@ -728,7 +738,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
     // Check if the right side's value is known at compile time.
     Long valRight =
         ValueCheckerUtils.getExactValue(
-            node.getRightOperand().getTree(), aTypeFactory.getValueAnnotatedTypeFactory());
+            node.getRightOperand().getTree(), atypeFactory.getValueAnnotatedTypeFactory());
     if (valRight != null) {
       return addAnnotationForLiteralRemainder(valRight.intValue());
     }
@@ -788,7 +798,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
    * @return true if the argument is the @Positive type annotation
    */
   private boolean isPositive(AnnotationMirror anm) {
-    return aTypeFactory.areSameByClass(anm, Positive.class);
+    return atypeFactory.areSameByClass(anm, Positive.class);
   }
 
   /**
@@ -798,7 +808,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
    * @return true if the argument is the @NonNegative type annotation
    */
   private boolean isNonNegative(AnnotationMirror anm) {
-    return aTypeFactory.areSameByClass(anm, NonNegative.class) || isPositive(anm);
+    return atypeFactory.areSameByClass(anm, NonNegative.class) || isPositive(anm);
   }
 
   /**
@@ -808,19 +818,28 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
    * @return true if the argument is the @GTENegativeOne type annotation
    */
   private boolean isGTEN1(AnnotationMirror anm) {
-    return aTypeFactory.areSameByClass(anm, GTENegativeOne.class) || isNonNegative(anm);
+    return atypeFactory.areSameByClass(anm, GTENegativeOne.class) || isNonNegative(anm);
   }
 
   private AnnotationMirror getLowerBoundAnnotation(
       Node subNode, TransferInput<CFValue, CFStore> p) {
     CFValue value = p.getValueOfSubNode(subNode);
+    if (value == null) {
+      throw new BugInCF("value==null for getLowerBoundAnnotation(%s, %s)%n", subNode, p);
+    }
     return getLowerBoundAnnotation(value);
   }
 
+  /**
+   * Returns the lower bound annotation for the given value.
+   *
+   * @param cfValue a value
+   * @return the lower bound annotation for the given value
+   */
   private AnnotationMirror getLowerBoundAnnotation(CFValue cfValue) {
-    return aTypeFactory
+    return atypeFactory
         .getQualifierHierarchy()
-        .findAnnotationInHierarchy(cfValue.getAnnotations(), aTypeFactory.UNKNOWN);
+        .findAnnotationInHierarchy(cfValue.getAnnotations(), atypeFactory.UNKNOWN);
   }
 
   @Override
@@ -828,7 +847,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       NumericalAdditionNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> result = super.visitNumericalAddition(n, p);
     AnnotationMirror newAnno = getAnnotationForPlus(n, p);
-    return createNewResult(result, newAnno);
+    return recreateTransferResult(newAnno, result);
   }
 
   @Override
@@ -836,7 +855,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       NumericalSubtractionNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> result = super.visitNumericalSubtraction(n, p);
     AnnotationMirror newAnno = getAnnotationForMinus(n, p);
-    return createNewResult(result, newAnno);
+    return recreateTransferResult(newAnno, result);
   }
 
   @Override
@@ -844,7 +863,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       NumericalMultiplicationNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> result = super.visitNumericalMultiplication(n, p);
     AnnotationMirror newAnno = getAnnotationForMultiply(n, p);
-    return createNewResult(result, newAnno);
+    return recreateTransferResult(newAnno, result);
   }
 
   @Override
@@ -852,7 +871,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       IntegerDivisionNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> result = super.visitIntegerDivision(n, p);
     AnnotationMirror newAnno = getAnnotationForDivide(n, p);
-    return createNewResult(result, newAnno);
+    return recreateTransferResult(newAnno, result);
   }
 
   @Override
@@ -860,7 +879,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       IntegerRemainderNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> transferResult = super.visitIntegerRemainder(n, p);
     AnnotationMirror resultAnno = getAnnotationForRemainder(n, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -868,7 +887,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       SignedRightShiftNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> transferResult = super.visitSignedRightShift(n, p);
     AnnotationMirror resultAnno = getAnnotationForRightShift(n, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -876,7 +895,7 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       UnsignedRightShiftNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> transferResult = super.visitUnsignedRightShift(n, p);
     AnnotationMirror resultAnno = getAnnotationForRightShift(n, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -884,21 +903,6 @@ public class LowerBoundTransfer extends IndexAbstractTransfer {
       BitwiseAndNode n, TransferInput<CFValue, CFStore> p) {
     TransferResult<CFValue, CFStore> transferResult = super.visitBitwiseAnd(n, p);
     AnnotationMirror resultAnno = getAnnotationForAnd(n, p);
-    return createNewResult(transferResult, resultAnno);
-  }
-
-  /**
-   * Create a new transfer result based on the original result and the new annotation.
-   *
-   * @param result the original result
-   * @param resultAnno the new annotation
-   * @return the new transfer result
-   */
-  private TransferResult<CFValue, CFStore> createNewResult(
-      TransferResult<CFValue, CFStore> result, AnnotationMirror resultAnno) {
-    CFValue newResultValue =
-        analysis.createSingleAnnotationValue(
-            resultAnno, result.getResultValue().getUnderlyingType());
-    return new RegularTransferResult<>(newResultValue, result.getRegularStore());
+    return recreateTransferResult(resultAnno, transferResult);
   }
 }
