@@ -14,15 +14,16 @@ import org.checkerframework.checker.index.qual.LTEqLengthOf;
 import org.checkerframework.checker.index.qual.LTLengthOf;
 import org.checkerframework.checker.index.qual.LTOMLengthOf;
 import org.checkerframework.checker.index.qual.SubstringIndexFor;
+import org.checkerframework.checker.nullness.qual.KeyFor;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.dataflow.cfg.node.Node;
 import org.checkerframework.dataflow.qual.Pure;
 import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.javacutil.AnnotationBuilder;
 import org.checkerframework.javacutil.AnnotationUtils;
-import org.checkerframework.javacutil.Pair;
 import org.checkerframework.javacutil.TypeSystemError;
 import org.plumelib.util.CollectionsPlume;
+import org.plumelib.util.IPair;
 
 /**
  * Abstraction for Upper Bound annotations. This abstract class has 4 subclasses, each of which is a
@@ -56,7 +57,7 @@ public abstract class UBQualifier {
    *     optional offset)
    */
   public static UBQualifier createUBQualifier(
-      AnnotationMirror am, String offset, UpperBoundChecker ubChecker) {
+      AnnotationMirror am, @Nullable String offset, UpperBoundChecker ubChecker) {
     switch (AnnotationUtils.annotationName(am)) {
       case "org.checkerframework.checker.index.qual.UpperBoundUnknown":
         return UpperBoundUnknownQualifier.UNKNOWN;
@@ -83,7 +84,7 @@ public abstract class UBQualifier {
   }
 
   /** A cache for the {@link #nCopiesEmptyStringCache} method. */
-  private static List<List<String>> nCopiesEmptyStringCache = new ArrayList<>(10);
+  private static final List<List<String>> nCopiesEmptyStringCache = new ArrayList<>(10);
 
   static {
     nCopiesEmptyStringCache.add(Collections.emptyList());
@@ -125,6 +126,11 @@ public abstract class UBQualifier {
     List<String> sequences =
         AnnotationUtils.getElementValueArray(
             ltLengthOfAnno, ubChecker.ltLengthOfValueElement, String.class);
+    if (sequences.isEmpty()) {
+      // These annotations can be created by delocalization of an LTLengthOf annotation
+      // that only contains local variables at a call site.
+      return UpperBoundUnknownQualifier.UNKNOWN;
+    }
     List<String> offsets =
         AnnotationUtils.getElementValueArray(
             ltLengthOfAnno,
@@ -147,6 +153,11 @@ public abstract class UBQualifier {
     List<String> sequences =
         AnnotationUtils.getElementValueArray(
             substringIndexForAnno, ubChecker.substringIndexForValueElement, String.class);
+    if (sequences.isEmpty()) {
+      // These annotations can be created by delocalization of a SubstringIndexFor annotation
+      // that only contains local variables at a call site.
+      return UpperBoundUnknownQualifier.UNKNOWN;
+    }
     List<String> offsets =
         AnnotationUtils.getElementValueArray(
             substringIndexForAnno, ubChecker.substringIndexForOffsetElement, String.class);
@@ -169,7 +180,8 @@ public abstract class UBQualifier {
     List<String> sequences =
         AnnotationUtils.getElementValueArray(am, ubChecker.ltEqLengthOfValueElement, String.class);
     if (sequences.isEmpty()) {
-      // How did this AnnotationMirror even get made?  It seems invalid.
+      // These annotations can be created by delocalization of an LTEqLengthOf annotation
+      // that only contains local variables at a call site.
       return UpperBoundUnknownQualifier.UNKNOWN;
     }
     List<String> offset = Collections.nCopies(sequences.size(), "-1");
@@ -185,9 +197,14 @@ public abstract class UBQualifier {
    * @return a UBQualifier created from the @LTOMLengthOf annotation
    */
   private static UBQualifier parseLTOMLengthOf(
-      AnnotationMirror am, String extraOffset, UpperBoundChecker ubChecker) {
+      AnnotationMirror am, @Nullable String extraOffset, UpperBoundChecker ubChecker) {
     List<String> sequences =
         AnnotationUtils.getElementValueArray(am, ubChecker.ltOMLengthOfValueElement, String.class);
+    if (sequences.isEmpty()) {
+      // These annotations can be created by delocalization of an LTOMLengthOf annotation
+      // that only contains local variables at a call site.
+      return UpperBoundUnknownQualifier.UNKNOWN;
+    }
     List<String> offset = Collections.nCopies(sequences.size(), "1");
     return createUBQualifier(sequences, offset, extraOffset);
   }
@@ -235,7 +252,7 @@ public abstract class UBQualifier {
    * @return an {@link UBQualifier} for the sequences with the given offsets
    */
   public static UBQualifier createUBQualifier(
-      List<String> sequences, List<String> offsets, String extraOffset) {
+      List<String> sequences, List<String> offsets, @Nullable String extraOffset) {
     assert !sequences.isEmpty();
 
     OffsetEquation extraEq;
@@ -420,7 +437,7 @@ public abstract class UBQualifier {
      * @param i an integer
      * @return true if the given integer literal is a subtype of this
      */
-    /*package-protected*/ boolean literalIsSubtype(int i) {
+    /*package-private*/ boolean literalIsSubtype(int i) {
       for (Map.Entry<String, Set<OffsetEquation>> entry : map.entrySet()) {
         for (OffsetEquation equation : entry.getValue()) {
           if (!equation.isInt()) {
@@ -444,7 +461,7 @@ public abstract class UBQualifier {
      * @return the map representation of a {@link UBQualifier}, or null if there is an error
      */
     private static @Nullable Map<String, Set<OffsetEquation>> sequencesAndOffsetsToMap(
-        List<String> sequences, List<String> offsets, OffsetEquation extraEq) {
+        List<String> sequences, List<String> offsets, @Nullable OffsetEquation extraEq) {
 
       Map<String, Set<OffsetEquation>> map = new HashMap<>(CollectionsPlume.mapCapacity(sequences));
       if (offsets.isEmpty()) {
@@ -475,8 +492,10 @@ public abstract class UBQualifier {
     private static class SequencesOffsetsAndClass {
       /** List of sequences. */
       public final List<String> sequences;
+
       /** List of offsets. */
       public final List<String> offsets;
+
       /** The class of the annotation to be built. */
       public final Class<? extends Annotation> annoClass;
 
@@ -506,7 +525,7 @@ public abstract class UBQualifier {
      */
     private static SequencesOffsetsAndClass mapToSequencesAndOffsets(
         Map<String, Set<OffsetEquation>> map, boolean buildSubstringIndexAnnotation) {
-      List<String> sortedSequences = new ArrayList<>(map.keySet());
+      List<@KeyFor("map") String> sortedSequences = new ArrayList<>(map.keySet());
       Collections.sort(sortedSequences);
       List<String> sequences = new ArrayList<>();
       List<String> offsets = new ArrayList<>();
@@ -558,7 +577,8 @@ public abstract class UBQualifier {
      * @param offsets list of offset, if empty, an offset of 0 is used
      * @param extraEq offset to add to each element of offsets; may be null
      */
-    private LessThanLengthOf(List<String> sequences, List<String> offsets, OffsetEquation extraEq) {
+    private LessThanLengthOf(
+        List<String> sequences, List<String> offsets, @Nullable OffsetEquation extraEq) {
       this(sequencesAndOffsetsToMap(sequences, offsets, extraEq));
     }
 
@@ -607,6 +627,7 @@ public abstract class UBQualifier {
       }
       return false;
     }
+
     /**
      * Is a value with this type less than the length of the sequence?
      *
@@ -891,7 +912,7 @@ public abstract class UBQualifier {
           || !containsSame(other.map.keySet(), lubMap.keySet())) {
         return;
       }
-      List<Pair<String, OffsetEquation>> remove = new ArrayList<>();
+      List<IPair<String, OffsetEquation>> remove = new ArrayList<>();
       for (Map.Entry<String, Set<OffsetEquation>> entry : lubMap.entrySet()) {
         String sequence = entry.getKey();
         Set<OffsetEquation> lubOffsets = entry.getValue();
@@ -905,7 +926,7 @@ public abstract class UBQualifier {
             int thisInt = OffsetEquation.getIntOffsetEquation(thisOffsets).getInt();
             int otherInt = OffsetEquation.getIntOffsetEquation(otherOffsets).getInt();
             if (thisInt != otherInt) {
-              remove.add(Pair.of(sequence, lubEq));
+              remove.add(IPair.of(sequence, lubEq));
             }
           } else if (thisOffsets.contains(lubEq) && otherOffsets.contains(lubEq)) {
             //  continue;
@@ -914,11 +935,12 @@ public abstract class UBQualifier {
           }
         }
       }
-      for (Pair<String, OffsetEquation> pair : remove) {
-        Set<OffsetEquation> offsets = lubMap.get(pair.first);
+      for (IPair<String, OffsetEquation> pair : remove) {
+        String sequence = pair.first;
+        Set<OffsetEquation> offsets = lubMap.get(sequence);
         offsets.remove(pair.second);
         if (offsets.isEmpty()) {
-          lubMap.remove(pair.first);
+          lubMap.remove(sequence);
         }
       }
     }
@@ -1095,19 +1117,14 @@ public abstract class UBQualifier {
      * @param sequences access of the length of these sequences are removed
      * @return a copy of this qualifier with some offsets removed
      */
-    public UBQualifier removeSequenceLengthAccess(final List<String> sequences) {
+    public UBQualifier removeSequenceLengthAccess(List<String> sequences) {
       if (sequences.isEmpty()) {
         return UpperBoundUnknownQualifier.UNKNOWN;
       }
-      OffsetEquationFunction removeSequenceLengthsFunc =
-          new OffsetEquationFunction() {
-            @Override
-            public OffsetEquation compute(OffsetEquation eq) {
-              return eq.removeSequenceLengths(sequences);
-            }
-          };
+      OffsetEquationFunction removeSequenceLengthsFunc = eq -> eq.removeSequenceLengths(sequences);
       return computeNewOffsets(removeSequenceLengthsFunc);
     }
+
     /**
      * Returns a copy of this qualifier with sequence-offset pairs where in the original the offset
      * contains an access of an sequence length in {@code sequences}. The sequence length access has
@@ -1116,35 +1133,32 @@ public abstract class UBQualifier {
      * @param sequences access of the length of these sequences are removed
      * @return a copy of this qualifier with some offsets removed
      */
-    public UBQualifier removeSequenceLengthAccessAndNeg1(final List<String> sequences) {
+    public UBQualifier removeSequenceLengthAccessAndNeg1(List<String> sequences) {
       if (sequences.isEmpty()) {
         return UpperBoundUnknownQualifier.UNKNOWN;
       }
       OffsetEquationFunction removeSequenceLenFunc =
-          new OffsetEquationFunction() {
-            @Override
-            public OffsetEquation compute(OffsetEquation eq) {
-              OffsetEquation newEq = eq.removeSequenceLengths(sequences);
-              if (newEq == null) {
-                return null;
-              }
-              if (newEq.getInt() == -1) {
-                return newEq.copyAdd('+', OffsetEquation.ONE);
-              }
-              return newEq;
+          eq -> {
+            OffsetEquation newEq = eq.removeSequenceLengths(sequences);
+            if (newEq == null) {
+              return null;
             }
+            if (newEq.getInt() == -1) {
+              return newEq.copyAdd('+', OffsetEquation.ONE);
+            }
+            return newEq;
           };
       return computeNewOffsets(removeSequenceLenFunc);
     }
 
-    private UBQualifier addOffset(final OffsetEquation newOffset) {
-      OffsetEquationFunction addOffsetFunc =
-          new OffsetEquationFunction() {
-            @Override
-            public OffsetEquation compute(OffsetEquation eq) {
-              return eq.copyAdd('+', newOffset);
-            }
-          };
+    /**
+     * Returns a new qualifier, which is this qualifier plus the given offset.
+     *
+     * @param newOffset the offset to add to this
+     * @return a new qualifier, which is this qualifier plus the given offset
+     */
+    private UBQualifier addOffset(OffsetEquation newOffset) {
+      OffsetEquationFunction addOffsetFunc = eq -> eq.copyAdd('+', newOffset);
       return computeNewOffsets(addOffsetFunc);
     }
 
@@ -1163,16 +1177,7 @@ public abstract class UBQualifier {
       if (divisor == 1) {
         return this;
       } else if (divisor > 1) {
-        OffsetEquationFunction divideFunc =
-            new OffsetEquationFunction() {
-              @Override
-              public OffsetEquation compute(OffsetEquation eq) {
-                if (eq.isNegativeOrZero()) {
-                  return eq;
-                }
-                return null;
-              }
-            };
+        OffsetEquationFunction divideFunc = eq -> (eq.isNegativeOrZero() ? eq : null);
         return computeNewOffsets(divideFunc);
       }
       return UpperBoundUnknownQualifier.UNKNOWN;
@@ -1245,14 +1250,14 @@ public abstract class UBQualifier {
     }
 
     /** Functional interface that operates on {@link OffsetEquation}s. */
-    interface OffsetEquationFunction {
+    private interface OffsetEquationFunction {
       /**
        * Returns the result of the computation or null if the passed equation should be removed.
        *
        * @param eq current offset equation
        * @return the result of the computation or null if the passed equation should be removed
        */
-      OffsetEquation compute(OffsetEquation eq);
+      @Nullable OffsetEquation compute(OffsetEquation eq);
     }
 
     /**
@@ -1293,11 +1298,13 @@ public abstract class UBQualifier {
   public static class UpperBoundLiteralQualifier extends UBQualifier {
 
     /** Represents the value -1. */
-    public static UpperBoundLiteralQualifier NEGATIVEONE = new UpperBoundLiteralQualifier(-1);
+    public static final UpperBoundLiteralQualifier NEGATIVEONE = new UpperBoundLiteralQualifier(-1);
+
     /** Represents the value 0. */
-    public static UpperBoundLiteralQualifier ZERO = new UpperBoundLiteralQualifier(0);
+    public static final UpperBoundLiteralQualifier ZERO = new UpperBoundLiteralQualifier(0);
+
     /** Represents the value 1. */
-    public static UpperBoundLiteralQualifier ONE = new UpperBoundLiteralQualifier(1);
+    public static final UpperBoundLiteralQualifier ONE = new UpperBoundLiteralQualifier(1);
 
     /**
      * Creates a new UpperBoundLiteralQualifier, without using cached values.
@@ -1328,14 +1335,14 @@ public abstract class UBQualifier {
     }
 
     /** The integer value. */
-    int value;
+    private final int value;
 
     /**
      * Returns the integer value.
      *
      * @return the integer value
      */
-    int getValue() {
+    public int getValue() {
       return value;
     }
 
@@ -1350,6 +1357,8 @@ public abstract class UBQualifier {
         return true;
       } else if (superType.isBottom()) {
         return false;
+      } else if (superType.isPoly()) {
+        return false;
       } else if (superType.isLiteral()) {
         int otherValue = ((UpperBoundLiteralQualifier) superType).value;
         return value == otherValue;
@@ -1386,7 +1395,7 @@ public abstract class UBQualifier {
   /** The top type qualifier. */
   public static class UpperBoundUnknownQualifier extends UBQualifier {
     /** The canonical representative. */
-    static final UBQualifier UNKNOWN = new UpperBoundUnknownQualifier();
+    public static final UBQualifier UNKNOWN = new UpperBoundUnknownQualifier();
 
     /** This class is a singleton. */
     private UpperBoundUnknownQualifier() {}
@@ -1417,8 +1426,13 @@ public abstract class UBQualifier {
     }
   }
 
+  /** The bottom qualifier for the upperbound type system. */
   private static class UpperBoundBottomQualifier extends UBQualifier {
-    static final UBQualifier BOTTOM = new UpperBoundBottomQualifier();
+    /** The canonical bottom qualifier for the upperbound type system. */
+    public static final UBQualifier BOTTOM = new UpperBoundBottomQualifier();
+
+    /** This class is a singleton. */
+    private UpperBoundBottomQualifier() {}
 
     @Override
     public boolean isBottom() {
@@ -1446,8 +1460,13 @@ public abstract class UBQualifier {
     }
   }
 
+  /** The polymorphic qualifier. */
   private static class PolyQualifier extends UBQualifier {
-    static final UBQualifier POLY = new PolyQualifier();
+    /** The canonical representative. */
+    public static final UBQualifier POLY = new PolyQualifier();
+
+    /** This class is a singleton. */
+    private PolyQualifier() {}
 
     @Override
     @Pure
