@@ -53,11 +53,13 @@ import org.checkerframework.checker.index.substringindex.SubstringIndexChecker;
 import org.checkerframework.checker.index.upperbound.UBQualifier.LessThanLengthOf;
 import org.checkerframework.checker.index.upperbound.UBQualifier.UpperBoundLiteralQualifier;
 import org.checkerframework.checker.index.upperbound.UBQualifier.UpperBoundUnknownQualifier;
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.common.value.ValueAnnotatedTypeFactory;
 import org.checkerframework.common.value.ValueChecker;
 import org.checkerframework.common.value.ValueCheckerUtils;
 import org.checkerframework.common.value.qual.BottomVal;
+import org.checkerframework.common.value.util.Range;
 import org.checkerframework.dataflow.cfg.node.Node;
 import org.checkerframework.dataflow.expression.JavaExpression;
 import org.checkerframework.framework.flow.CFAbstractStore;
@@ -74,10 +76,11 @@ import org.checkerframework.framework.type.typeannotator.TypeAnnotator;
 import org.checkerframework.framework.util.JavaExpressionParseUtil.JavaExpressionParseException;
 import org.checkerframework.framework.util.dependenttypes.DependentTypesHelper;
 import org.checkerframework.javacutil.AnnotationBuilder;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
-import org.checkerframework.javacutil.Pair;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypeSystemError;
+import org.plumelib.util.IPair;
 
 /**
  * Implements the introduction rules for the Upper Bound Checker.
@@ -87,7 +90,7 @@ import org.checkerframework.javacutil.TypeSystemError;
  * <ul>
  *   <li>1. Math.min has unusual semantics that combines annotations for the UBC.
  *   <li>2. The return type of Random.nextInt depends on the argument, but is not equal to it, so a
- *       polymorhpic qualifier is insufficient.
+ *       polymorphic qualifier is insufficient.
  *   <li>3. Unary negation on a NegativeIndexFor (from the SearchIndex Checker) results in a
  *       LTLengthOf for the same arrays.
  *   <li>4. Right shifting by a constant between 0 and 30 preserves the type of the left side
@@ -109,21 +112,26 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
   /** The @{@link UpperBoundUnknown} annotation. */
   public final AnnotationMirror UNKNOWN =
       AnnotationBuilder.fromClass(elements, UpperBoundUnknown.class);
+
   /** The @{@link UpperBoundBottom} annotation. */
   public final AnnotationMirror BOTTOM =
       AnnotationBuilder.fromClass(elements, UpperBoundBottom.class);
+
   /** The @{@link PolyUpperBound} annotation. */
   public final AnnotationMirror POLY = AnnotationBuilder.fromClass(elements, PolyUpperBound.class);
+
   /** The @{@link UpperBoundLiteral}(-1) annotation. */
   public final AnnotationMirror NEGATIVEONE =
       new AnnotationBuilder(getProcessingEnv(), UpperBoundLiteral.class)
           .setValue("value", -1)
           .build();
+
   /** The @{@link UpperBoundLiteral}(0) annotation. */
   public final AnnotationMirror ZERO =
       new AnnotationBuilder(getProcessingEnv(), UpperBoundLiteral.class)
           .setValue("value", 0)
           .build();
+
   /** The @{@link UpperBoundLiteral}(1) annotation. */
   public final AnnotationMirror ONE =
       new AnnotationBuilder(getProcessingEnv(), UpperBoundLiteral.class)
@@ -133,12 +141,15 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
   /** The NegativeIndexFor.value element/field. */
   public final ExecutableElement negativeIndexForValueElement =
       TreeUtils.getMethod(NegativeIndexFor.class, "value", 0, processingEnv);
+
   /** The SameLen.value element/field. */
   public final ExecutableElement sameLenValueElement =
       TreeUtils.getMethod(SameLen.class, "value", 0, processingEnv);
+
   /** The LTLengthOf.value element/field. */
   public final ExecutableElement ltLengthOfValueElement =
       TreeUtils.getMethod(LTLengthOf.class, "value", 0, processingEnv);
+
   /** The LTLengthOf.offset element/field. */
   public final ExecutableElement ltLengthOfOffsetElement =
       TreeUtils.getMethod(LTLengthOf.class, "offset", 0, processingEnv);
@@ -232,7 +243,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
   @Override
   public void addComputedTypeAnnotations(Element element, AnnotatedTypeMirror type) {
     super.addComputedTypeAnnotations(element, type);
-    if (element != null) {
+    if (element != null && !ajavaTypes.isParsing()) {
       AnnotatedTypeMirror valueType = getValueAnnotatedTypeFactory().getAnnotatedType(element);
       addUpperBoundTypeFromValueType(valueType, type);
     }
@@ -244,7 +255,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     // If dataflow shouldn't be used to compute this type, then do not use the result from
     // the Value Checker, because dataflow is used to compute that type.  (Without this,
     // "int i = 1; --i;" fails.)
-    if (iUseFlow && tree != null && TreeUtils.isExpressionTree(tree)) {
+    if (iUseFlow && tree != null && !ajavaTypes.isParsing() && TreeUtils.isExpressionTree(tree)) {
       AnnotatedTypeMirror valueType = getValueAnnotatedTypeFactory().getAnnotatedType(tree);
       addUpperBoundTypeFromValueType(valueType, type);
     }
@@ -259,7 +270,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
    */
   private void addUpperBoundTypeFromValueType(
       AnnotatedTypeMirror valueType, AnnotatedTypeMirror type) {
-    if (containsSameByClass(valueType.getAnnotations(), BottomVal.class)) {
+    if (containsSameByClass(valueType.getPrimaryAnnotations(), BottomVal.class)) {
       type.replaceAnnotation(BOTTOM);
     }
   }
@@ -281,9 +292,9 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
 
     @Override
     protected Void scan(AnnotatedTypeMirror type, Void aVoid) {
-      // If there is an LTLengthOf annotation whose argument lengths don't match, replace it with
-      // bottom.
-      AnnotationMirror anm = type.getAnnotation(LTLengthOf.class);
+      // If there is an LTLengthOf annotation whose argument lengths don't match, replace it
+      // with bottom.
+      AnnotationMirror anm = type.getPrimaryAnnotation(LTLengthOf.class);
       if (anm != null) {
         List<String> sequences =
             AnnotationUtils.getElementValueArray(anm, ltLengthOfValueElement, String.class);
@@ -313,9 +324,9 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
    * Queries the SameLen Checker to return the type that the SameLen Checker associates with the
    * given tree.
    */
-  public AnnotationMirror sameLenAnnotationFromTree(Tree tree) {
+  public @Nullable AnnotationMirror sameLenAnnotationFromTree(Tree tree) {
     AnnotatedTypeMirror sameLenType = getSameLenAnnotatedTypeFactory().getAnnotatedType(tree);
-    return sameLenType.getAnnotation(SameLen.class);
+    return sameLenType.getPrimaryAnnotation(SameLen.class);
   }
 
   // Wrapper methods for accessing the IndexMethodIdentifier.
@@ -378,7 +389,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     return areSameByClass(
         getLowerBoundAnnotatedTypeFactory()
             .getAnnotatedType(node.getTree())
-            .getAnnotationInHierarchy(getLowerBoundAnnotatedTypeFactory().UNKNOWN),
+            .getPrimaryAnnotationInHierarchy(getLowerBoundAnnotatedTypeFactory().UNKNOWN),
         classOfType);
   }
 
@@ -397,11 +408,11 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
      */
     UpperBoundQualifierHierarchy(
         Collection<Class<? extends Annotation>> qualifierClasses, Elements elements) {
-      super(qualifierClasses, elements);
+      super(qualifierClasses, elements, UpperBoundAnnotatedTypeFactory.this);
     }
 
     @Override
-    public AnnotationMirror greatestLowerBound(AnnotationMirror a1, AnnotationMirror a2) {
+    public AnnotationMirror greatestLowerBoundQualifiers(AnnotationMirror a1, AnnotationMirror a2) {
       UBQualifier a1Obj = UBQualifier.createUBQualifier(a1, (IndexChecker) checker);
       UBQualifier a2Obj = UBQualifier.createUBQualifier(a2, (IndexChecker) checker);
       UBQualifier glb = a1Obj.glb(a2Obj);
@@ -416,7 +427,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
      * @return the least upper bound of a1 and a2
      */
     @Override
-    public AnnotationMirror leastUpperBound(AnnotationMirror a1, AnnotationMirror a2) {
+    public AnnotationMirror leastUpperBoundQualifiers(AnnotationMirror a1, AnnotationMirror a2) {
       UBQualifier a1Obj = UBQualifier.createUBQualifier(a1, (IndexChecker) checker);
       UBQualifier a2Obj = UBQualifier.createUBQualifier(a2, (IndexChecker) checker);
       UBQualifier lub = a1Obj.lub(a2Obj);
@@ -438,17 +449,17 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     }
 
     /**
-     * Computes subtyping as per the subtyping in the qualifier hierarchy structure unless both
+     * {@inheritDoc}
+     *
+     * <p>Computes subtyping as per the subtyping in the qualifier hierarchy structure unless both
      * annotations have the same class. In this case, rhs is a subtype of lhs iff rhs contains every
      * element of lhs.
-     *
-     * @return true if rhs is a subtype of lhs, false otherwise
      */
     @Override
-    public boolean isSubtype(AnnotationMirror subAnno, AnnotationMirror superAnno) {
-      UBQualifier subtype = UBQualifier.createUBQualifier(subAnno, (IndexChecker) checker);
-      UBQualifier supertype = UBQualifier.createUBQualifier(superAnno, (IndexChecker) checker);
-      return subtype.isSubtype(supertype);
+    public boolean isSubtypeQualifiers(AnnotationMirror subAnno, AnnotationMirror superAnno) {
+      UBQualifier subtypeQual = UBQualifier.createUBQualifier(subAnno, (IndexChecker) checker);
+      UBQualifier supertypeQual = UBQualifier.createUBQualifier(superAnno, (IndexChecker) checker);
+      return subtypeQual.isSubtype(supertypeQual);
     }
   }
 
@@ -469,7 +480,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
      * <ul>
      *   <li>Math.min has unusual semantics that combines annotations for the UBC.
      *   <li>The return type of Random.nextInt depends on the argument, but is not equal to it, so a
-     *       polymorhpic qualifier is insufficient.
+     *       polymorphic qualifier is insufficient.
      * </ul>
      *
      * Other methods should not be special-cased here unless there is a compelling reason to do so.
@@ -481,40 +492,107 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
         AnnotatedTypeMirror rightType = getAnnotatedType(tree.getArguments().get(1));
 
         type.replaceAnnotation(
-            qualHierarchy.greatestLowerBound(
-                leftType.getAnnotationInHierarchy(UNKNOWN),
-                rightType.getAnnotationInHierarchy(UNKNOWN)));
+            qualHierarchy.greatestLowerBoundShallow(
+                leftType.getPrimaryAnnotationInHierarchy(UNKNOWN),
+                leftType.getUnderlyingType(),
+                rightType.getPrimaryAnnotationInHierarchy(UNKNOWN),
+                rightType.getUnderlyingType()));
       }
       if (isRandomNextInt(tree)) {
         AnnotatedTypeMirror argType = getAnnotatedType(tree.getArguments().get(0));
-        AnnotationMirror anno = argType.getAnnotationInHierarchy(UNKNOWN);
+        AnnotationMirror anno = argType.getPrimaryAnnotationInHierarchy(UNKNOWN);
         UBQualifier qualifier = UBQualifier.createUBQualifier(anno, (IndexChecker) checker);
         qualifier = qualifier.plusOffset(1);
         type.replaceAnnotation(convertUBQualifierToAnnotation(qualifier));
       }
+      if (imf.isIndexOfString(tree)) {
+        // String#indexOf(String) and its variants that also take a String technically
+        // return (and are annotated as) @LTEqLengthOf the receiver. However, the result is
+        // always @LTLengthOf the receiver unless both the receiver and the target string
+        // are the empty string: "".indexOf("") returns 0, which isn't an index into "". So,
+        // this special case modifies the return type of these methods if either the
+        // parameter or the receiver is known (by the Value Checker) to not be the empty
+        // string. There are three ways the Value Checker might have that information:
+        // either string could have a @StringVal annotation whose value doesn't include "",
+        // either could have an @ArrayLen annotation whose value doesn't contain zero, or
+        // either could have an @ArrayLenRange annotation whose from value is any positive
+        // integer.
+        ValueAnnotatedTypeFactory vatf =
+            ((UpperBoundAnnotatedTypeFactory) this.atypeFactory).getValueAnnotatedTypeFactory();
+        AnnotatedTypeMirror argType = vatf.getAnnotatedType(tree.getArguments().get(0));
+        AnnotatedTypeMirror receiverType = vatf.getReceiverType(tree);
+        if (definitelyIsNotTheEmptyString(argType, vatf)
+            || definitelyIsNotTheEmptyString(receiverType, vatf)) {
+          String receiverName = JavaExpression.getReceiver(tree).toString();
+          UBQualifier ltLengthOfReceiver = UBQualifier.createUBQualifier(receiverName, "0");
+          AnnotationMirror currentReturnAnno = type.getPrimaryAnnotationInHierarchy(UNKNOWN);
+          UBQualifier currentUBQualifier =
+              UBQualifier.createUBQualifier(currentReturnAnno, (IndexChecker) checker);
+          UBQualifier result = currentUBQualifier.glb(ltLengthOfReceiver);
+          type.replaceAnnotation(convertUBQualifierToAnnotation(result));
+        }
+      }
       return super.visitMethodInvocation(tree, type);
     }
 
-    @Override
-    public Void visitLiteral(LiteralTree node, AnnotatedTypeMirror type) {
-      // Could also handle long literals, but array indexes are always ints.
-      if (node.getKind() == Tree.Kind.INT_LITERAL) {
-        type.addAnnotation(createLiteral(((Integer) node.getValue()).intValue()));
+    /**
+     * Returns true if the given Value Checker annotations guarantee that the annotated element is
+     * not the empty string.
+     *
+     * @param atm an annotated type from the Value Checker
+     * @param vatf the Value Annotated Type Factory
+     * @return true iff atm contains a {@code StringVal} annotation whose value doesn't contain "",
+     *     an {@code ArrayLen} annotation whose value doesn't contain 0, or an {@code ArrayLenRange}
+     *     annotation whose from value is greater than 0
+     */
+    private boolean definitelyIsNotTheEmptyString(
+        AnnotatedTypeMirror atm, ValueAnnotatedTypeFactory vatf) {
+      AnnotationMirrorSet annos = atm.getPrimaryAnnotations();
+      for (AnnotationMirror anno : annos) {
+        switch (AnnotationUtils.annotationName(anno)) {
+          case ValueAnnotatedTypeFactory.STRINGVAL_NAME:
+            List<String> strings = vatf.getStringValues(anno);
+            if (strings != null && !strings.contains("")) {
+              return true;
+            }
+            break;
+          case ValueAnnotatedTypeFactory.ARRAYLEN_NAME:
+            List<Integer> lengths = vatf.getArrayLength(anno);
+            if (lengths != null && !lengths.contains(0)) {
+              return true;
+            }
+            break;
+          default:
+            Range range = vatf.getRange(anno);
+            if (range != null && range.from > 0) {
+              return true;
+            }
+            break;
+        }
       }
-      return super.visitLiteral(node, type);
+      return false;
+    }
+
+    @Override
+    public Void visitLiteral(LiteralTree tree, AnnotatedTypeMirror type) {
+      // Could also handle long literals, but array indexes are always ints.
+      if (tree.getKind() == Tree.Kind.INT_LITERAL) {
+        type.addAnnotation(createLiteral(((Integer) tree.getValue()).intValue()));
+      }
+      return super.visitLiteral(tree, type);
     }
 
     /* Handles case 3. */
     @Override
-    public Void visitUnary(UnaryTree node, AnnotatedTypeMirror type) {
+    public Void visitUnary(UnaryTree tree, AnnotatedTypeMirror type) {
       // Dataflow refines this type if possible
-      if (node.getKind() == Tree.Kind.BITWISE_COMPLEMENT) {
+      if (tree.getKind() == Tree.Kind.BITWISE_COMPLEMENT) {
         addAnnotationForBitwiseComplement(
-            getSearchIndexAnnotatedTypeFactory().getAnnotatedType(node.getExpression()), type);
+            getSearchIndexAnnotatedTypeFactory().getAnnotatedType(tree.getExpression()), type);
       } else {
         type.addAnnotation(UNKNOWN);
       }
-      return super.visitUnary(node, type);
+      return super.visitUnary(tree, type);
     }
 
     /**
@@ -542,7 +620,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
      */
     private void addAnnotationForBitwiseComplement(
         AnnotatedTypeMirror searchIndexType, AnnotatedTypeMirror typeDst) {
-      AnnotationMirror nif = searchIndexType.getAnnotation(NegativeIndexFor.class);
+      AnnotationMirror nif = searchIndexType.getPrimaryAnnotation(NegativeIndexFor.class);
       if (nif != null) {
         List<String> arrays =
             AnnotationUtils.getElementValueArray(nif, negativeIndexForValueElement, String.class);
@@ -555,14 +633,18 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     }
 
     @Override
-    public Void visitCompoundAssignment(CompoundAssignmentTree node, AnnotatedTypeMirror type) {
+    public Void visitCompoundAssignment(CompoundAssignmentTree tree, AnnotatedTypeMirror type) {
       // Dataflow refines this type if possible
       type.addAnnotation(UNKNOWN);
-      return super.visitCompoundAssignment(node, type);
+      return super.visitCompoundAssignment(tree, type);
     }
 
     @Override
     public Void visitBinary(BinaryTree tree, AnnotatedTypeMirror type) {
+      // This implementation does NOT call getAnnotatedType on the left or right operands.
+      // Doing so would lead to re-examination of subexpressions many times (which is too
+      // slow).
+
       // A few small rules for addition/subtraction by 0/1, etc.
       if (TreeUtils.isStringConcatenation(tree)) {
         type.addAnnotation(UNKNOWN);
@@ -606,8 +688,10 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
         ExpressionTree left, ExpressionTree right, AnnotatedTypeMirror type) {
       LowerBoundAnnotatedTypeFactory lowerBoundATF = getLowerBoundAnnotatedTypeFactory();
       if (lowerBoundATF.isNonNegative(left)) {
-        AnnotationMirror annotation = getAnnotatedType(left).getAnnotationInHierarchy(UNKNOWN);
-        // For non-negative numbers, right shift is equivalent to division by a power of two.
+        AnnotationMirror annotation =
+            getAnnotatedType(left).getPrimaryAnnotationInHierarchy(UNKNOWN);
+        // For non-negative numbers, right shift is equivalent to division by a power of
+        // two.
         // The range of the shift amount is limited to 0..30 to avoid overflows and int/long
         // differences.
         Long shiftAmount = ValueCheckerUtils.getExactValue(right, getValueAnnotatedTypeFactory());
@@ -634,22 +718,27 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
         ExpressionTree left, ExpressionTree right, AnnotatedTypeMirror type) {
       LowerBoundAnnotatedTypeFactory lowerBoundATF = getLowerBoundAnnotatedTypeFactory();
       AnnotatedTypeMirror leftType = getAnnotatedType(left);
-      AnnotationMirror leftResultType = UNKNOWN;
+      AnnotationMirror leftResultAnno = UNKNOWN;
       if (lowerBoundATF.isNonNegative(left)) {
-        leftResultType = leftType.getAnnotationInHierarchy(UNKNOWN);
+        leftResultAnno = leftType.getPrimaryAnnotationInHierarchy(UNKNOWN);
       }
 
       AnnotatedTypeMirror rightType = getAnnotatedType(right);
-      AnnotationMirror rightResultType = UNKNOWN;
+      AnnotationMirror rightResultAnno = UNKNOWN;
       if (lowerBoundATF.isNonNegative(right)) {
-        rightResultType = rightType.getAnnotationInHierarchy(UNKNOWN);
+        rightResultAnno = rightType.getPrimaryAnnotationInHierarchy(UNKNOWN);
       }
 
-      type.addAnnotation(qualHierarchy.greatestLowerBound(leftResultType, rightResultType));
+      type.addAnnotation(
+          qualHierarchy.greatestLowerBoundShallow(
+              leftResultAnno,
+              leftType.getUnderlyingType(),
+              rightResultAnno,
+              rightType.getUnderlyingType()));
     }
 
     /** Gets a sequence tree for a length access tree, or null if it is not a length access. */
-    private ExpressionTree getLengthSequenceTree(ExpressionTree lengthTree) {
+    private @Nullable ExpressionTree getLengthSequenceTree(ExpressionTree lengthTree) {
       return IndexUtil.getLengthSequenceTree(lengthTree, imf, processingEnv);
     }
 
@@ -785,7 +874,8 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
         }
 
         if (imf.isRandomNextDouble(mitree, processingEnv)) {
-          // Okay, so this is Random.nextDouble() * array.length, which must be NonNegative
+          // Okay, so this is Random.nextDouble() * array.length, which must be
+          // NonNegative
           type.addAnnotation(createLTLengthOfAnnotation(seqTree.toString()));
           return true;
         }
@@ -847,7 +937,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     return ltlQualifier.convertToAnnotation(processingEnv);
   }
 
-  UBQualifier fromLessThan(ExpressionTree tree, TreePath treePath) {
+  @Nullable UBQualifier fromLessThan(ExpressionTree tree, TreePath treePath) {
     List<String> lessThanExpressions =
         getLessThanAnnotatedTypeFactory().getLessThanExpressions(tree);
     if (lessThanExpressions == null) {
@@ -860,7 +950,7 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     return null;
   }
 
-  UBQualifier fromLessThanOrEqual(ExpressionTree tree, TreePath treePath) {
+  @Nullable UBQualifier fromLessThanOrEqual(ExpressionTree tree, TreePath treePath) {
     List<String> lessThanExpressions =
         getLessThanAnnotatedTypeFactory().getLessThanExpressions(tree);
     if (lessThanExpressions == null) {
@@ -870,11 +960,11 @@ public class UpperBoundAnnotatedTypeFactory extends BaseAnnotatedTypeFactoryForI
     return ubQualifier;
   }
 
-  private UBQualifier fromLessThanOrEqual(
+  private @Nullable UBQualifier fromLessThanOrEqual(
       Tree tree, TreePath treePath, List<String> lessThanExpressions) {
     UBQualifier ubQualifier = null;
     for (String expression : lessThanExpressions) {
-      Pair<JavaExpression, String> exprAndOffset;
+      IPair<JavaExpression, String> exprAndOffset;
       try {
         exprAndOffset = getExpressionAndOffsetFromJavaExpressionString(expression, treePath);
       } catch (JavaExpressionParseException e) {
