@@ -18,6 +18,7 @@ import org.checkerframework.checker.index.qual.HasSubsequence;
 import org.checkerframework.checker.index.qual.LTLengthOf;
 import org.checkerframework.checker.index.samelen.SameLenAnnotatedTypeFactory;
 import org.checkerframework.checker.index.upperbound.UBQualifier.LessThanLengthOf;
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.common.basetype.BaseTypeVisitor;
 import org.checkerframework.common.value.ValueAnnotatedTypeFactory;
@@ -34,9 +35,9 @@ import org.checkerframework.framework.util.JavaExpressionParseUtil.JavaExpressio
 import org.checkerframework.framework.util.StringToJavaExpression;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.ElementUtils;
-import org.checkerframework.javacutil.Pair;
 import org.checkerframework.javacutil.TreePathUtil;
 import org.checkerframework.javacutil.TreeUtils;
+import org.plumelib.util.IPair;
 
 /** Warns about array accesses that could be too high. */
 public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFactory> {
@@ -73,12 +74,13 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
 
   /** Warns about LTLengthOf annotations with arguments whose lengths do not match. */
   @Override
-  public Void visitAnnotation(AnnotationTree node, Void p) {
-    AnnotationMirror anno = TreeUtils.annotationFromAnnotationTree(node);
+  public Void visitAnnotation(AnnotationTree tree, Void p) {
+    AnnotationMirror anno = TreeUtils.annotationFromAnnotationTree(tree);
     if (atypeFactory.areSameByClass(anno, LTLengthOf.class)) {
-      List<? extends ExpressionTree> args = node.getArguments();
+      List<? extends ExpressionTree> args = tree.getArguments();
       if (args.size() == 2) {
-        // If offsets are provided, there must be the same number of them as there are arrays.
+        // If offsets are provided, there must be the same number of them as there are
+        // arrays.
         List<String> sequences =
             AnnotationUtils.getElementValueArray(
                 anno, atypeFactory.ltLengthOfValueElement, String.class);
@@ -87,7 +89,7 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
                 anno, atypeFactory.ltLengthOfOffsetElement, String.class, Collections.emptyList());
         if (sequences.size() != offsets.size() && !offsets.isEmpty()) {
           checker.reportError(
-              node, "different.length.sequences.offsets", sequences.size(), offsets.size());
+              tree, "different.length.sequences.offsets", sequences.size(), offsets.size());
           return null;
         }
       }
@@ -101,11 +103,11 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
 
       // check that each expression is parsable at the declaration of this class
       ClassTree enclosingClass = TreePathUtil.enclosingClass(getCurrentPath());
-      checkEffectivelyFinalAndParsable(seq, enclosingClass, node);
-      checkEffectivelyFinalAndParsable(from, enclosingClass, node);
-      checkEffectivelyFinalAndParsable(to, enclosingClass, node);
+      checkEffectivelyFinalAndParsable(seq, enclosingClass, tree);
+      checkEffectivelyFinalAndParsable(from, enclosingClass, tree);
+      checkEffectivelyFinalAndParsable(to, enclosingClass, tree);
     }
-    return super.visitAnnotation(node, p);
+    return super.visitAnnotation(tree, p);
   }
 
   /**
@@ -193,12 +195,14 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
   }
 
   @Override
-  protected void commonAssignmentCheck(
+  protected boolean commonAssignmentCheck(
       Tree varTree,
       ExpressionTree valueTree,
       @CompilerMessageKey String errorKey,
       Object... extraArgs) {
 
+    boolean result = true;
+
     // check that when an assignment to a variable b declared as @HasSubsequence(a, from, to)
     // occurs, to <= a.length, i.e. to is @LTEqLengthOf(a).
 
@@ -230,6 +234,7 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
             subSeq.array,
             subSeq.array,
             subSeq.array);
+        result = false;
       } else {
         checker.reportWarning(
             valueTree,
@@ -244,28 +249,33 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
       }
     }
 
-    super.commonAssignmentCheck(varTree, valueTree, errorKey, extraArgs);
+    result = super.commonAssignmentCheck(varTree, valueTree, errorKey, extraArgs) && result;
+    return result;
   }
 
   @Override
-  protected void commonAssignmentCheck(
+  protected boolean commonAssignmentCheck(
       AnnotatedTypeMirror varType,
       ExpressionTree valueTree,
       @CompilerMessageKey String errorKey,
       Object... extraArgs) {
     AnnotatedTypeMirror valueType = atypeFactory.getAnnotatedType(valueTree);
     commonAssignmentCheckStartDiagnostic(varType, valueType, valueTree);
+    boolean result = true;
+    String diagnosticMessage = "";
     if (!relaxedCommonAssignment(varType, valueTree)) {
       commonAssignmentCheckEndDiagnostic(
           "relaxedCommonAssignment did not succeed, now must call super",
           varType,
           valueType,
           valueTree);
-      super.commonAssignmentCheck(varType, valueTree, errorKey, extraArgs);
-    } else if (checker.hasOption("showchecks")) {
-      commonAssignmentCheckEndDiagnostic(
-          true, "relaxedCommonAssignment", varType, valueType, valueTree);
+      result = super.commonAssignmentCheck(varType, valueTree, errorKey, extraArgs);
+      if (!result && showchecks) {
+        diagnosticMessage = "relaxedCommonAssignment()=>false and super()=>false";
+      }
     }
+    commonAssignmentCheckEndDiagnostic(result, diagnosticMessage, varType, valueType, valueTree);
+    return result;
   }
 
   /**
@@ -331,16 +341,16 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
    * <p>This is useful for expressions like "n+1", for which {@link #parseJavaExpressionString}
    * returns null because the whole expression is not a receiver.
    */
-  static Pair<JavaExpression, String> getExpressionAndOffsetFromJavaExpressionString(
+  static @Nullable IPair<JavaExpression, String> getExpressionAndOffsetFromJavaExpressionString(
       String s, UpperBoundAnnotatedTypeFactory atypeFactory, TreePath currentPath) {
 
-    Pair<String, String> p = AnnotatedTypeFactory.getExpressionAndOffset(s);
+    IPair<String, String> p = AnnotatedTypeFactory.getExpressionAndOffset(s);
 
     JavaExpression je = parseJavaExpressionString(p.first, atypeFactory, currentPath);
     if (je == null) {
       return null;
     }
-    return Pair.of(je, p.second);
+    return IPair.of(je, p.second);
   }
 
   /**
@@ -350,7 +360,7 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
    *
    * <p>This wraps GenericAnnotatedTypeFactory#parseJavaExpressionString.
    */
-  static JavaExpression parseJavaExpressionString(
+  static @Nullable JavaExpression parseJavaExpressionString(
       String s, UpperBoundAnnotatedTypeFactory atypeFactory, TreePath currentPath) {
     JavaExpression result;
     try {
@@ -402,8 +412,8 @@ public class UpperBoundVisitor extends BaseTypeVisitor<UpperBoundAnnotatedTypeFa
 
     // Take advantage of information available on a HasSubsequence(a, from, to) annotation
     // on the lhs qualifier (varLtlQual):
-    // this allows us to show that iff varLtlQual includes LTL(b), b has HSS, and expQual includes
-    // LTL(a, -from), then the LTL(b) can be removed from varLtlQual.
+    // this allows us to show that iff varLtlQual includes LTL(b), b has HSS, and expQual
+    // includes LTL(a, -from), then the LTL(b) can be removed from varLtlQual.
 
     UBQualifier newLHS = processSubsequenceForLHS(varLtlQual, expQual);
     if (newLHS.isUnknown()) {
