@@ -3,10 +3,13 @@ package org.checkerframework.checker.initialization;
 import com.sun.source.tree.ClassTree;
 import com.sun.source.tree.ExpressionTree;
 import com.sun.source.tree.LiteralTree;
+import com.sun.source.tree.MemberReferenceTree;
+import com.sun.source.tree.MemberReferenceTree.ReferenceMode;
 import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.NewClassTree;
 import com.sun.source.tree.Tree;
+import com.sun.source.tree.Tree.Kind;
 import com.sun.source.tree.VariableTree;
 import com.sun.source.util.TreePath;
 import com.sun.tools.javac.code.Type;
@@ -38,6 +41,7 @@ import org.checkerframework.checker.initialization.qual.UnderInitialization;
 import org.checkerframework.checker.initialization.qual.UnknownInitialization;
 import org.checkerframework.checker.nullness.NullnessAnnotatedTypeFactory;
 import org.checkerframework.checker.nullness.NullnessChecker;
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.framework.flow.CFAbstractAnalysis;
 import org.checkerframework.framework.flow.CFAbstractValue;
@@ -56,10 +60,10 @@ import org.checkerframework.framework.util.QualifierKind;
 import org.checkerframework.javacutil.AnnotationBuilder;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.ElementUtils;
-import org.checkerframework.javacutil.Pair;
 import org.checkerframework.javacutil.TreePathUtil;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypesUtils;
+import org.plumelib.util.IPair;
 
 /**
  * The annotated type factory for the freedom-before-commitment type-system. The
@@ -94,8 +98,10 @@ public abstract class InitializationAnnotatedTypeFactory<
 
   /** The Unused.when field/element. */
   protected final ExecutableElement unusedWhenElement;
+
   /** The UnderInitialization.value field/element. */
   protected final ExecutableElement underInitializationValueElement;
+
   /** The UnknownInitialization.value field/element. */
   protected final ExecutableElement unknownInitializationValueElement;
 
@@ -436,7 +442,7 @@ public abstract class InitializationAnnotatedTypeFactory<
   }
 
   @Override
-  public AnnotatedDeclaredType getSelfType(Tree tree) {
+  public @Nullable AnnotatedDeclaredType getSelfType(Tree tree) {
     AnnotatedDeclaredType selfType = super.getSelfType(tree);
 
     TreePath path = getPath(tree);
@@ -466,7 +472,7 @@ public abstract class InitializationAnnotatedTypeFactory<
    * @return path to a top-level member containing the leaf of {@code path}
    */
   @SuppressWarnings("interning:not.interned") // AST node comparison
-  private TreePath findTopLevelClassMemberForTree(TreePath path) {
+  private @Nullable TreePath findTopLevelClassMemberForTree(TreePath path) {
     if (TreeUtils.isClassTree(path.getLeaf())) {
       path = path.getParentPath();
       if (path == null) {
@@ -570,13 +576,13 @@ public abstract class InitializationAnnotatedTypeFactory<
    * @param receiverAnnotations the annotations on the receiver
    * @return the fields that are not yet initialized in a given store (a pair of lists)
    */
-  public Pair<List<VariableTree>, List<VariableTree>> getUninitializedFields(
+  public IPair<List<VariableTree>, List<VariableTree>> getUninitializedFields(
       Store store,
       TreePath path,
       boolean isStatic,
       Collection<? extends AnnotationMirror> receiverAnnotations) {
     ClassTree currentClass = TreePathUtil.enclosingClass(path);
-    List<VariableTree> fields = InitializationChecker.getAllFields(currentClass);
+    List<VariableTree> fields = TreeUtils.fieldsFromClassTree(currentClass);
     List<VariableTree> uninitWithInvariantAnno = new ArrayList<>();
     List<VariableTree> uninitWithoutInvariantAnno = new ArrayList<>();
     for (VariableTree field : fields) {
@@ -596,7 +602,7 @@ public abstract class InitializationAnnotatedTypeFactory<
         }
       }
     }
-    return Pair.of(uninitWithInvariantAnno, uninitWithoutInvariantAnno);
+    return IPair.of(uninitWithInvariantAnno, uninitWithoutInvariantAnno);
   }
 
   /**
@@ -626,10 +632,8 @@ public abstract class InitializationAnnotatedTypeFactory<
    * @return the fields that have the invariant annotation and are initialized in a given store
    */
   public List<VariableTree> getInitializedInvariantFields(Store store, TreePath path) {
-    // TODO: Instead of passing the TreePath around, can we use
-    // getCurrentClassTree?
     ClassTree currentClass = TreePathUtil.enclosingClass(path);
-    List<VariableTree> fields = InitializationChecker.getAllFields(currentClass);
+    List<VariableTree> fields = TreeUtils.fieldsFromClassTree(currentClass);
     List<VariableTree> initializedFields = new ArrayList<>();
     for (VariableTree field : fields) {
       VariableElement fieldElem = TreeUtils.elementFromDeclaration(field);
@@ -707,7 +711,7 @@ public abstract class InitializationAnnotatedTypeFactory<
     // not necessary if there is an explicit UnknownInitialization
     // annotation on the field
     if (AnnotationUtils.containsSameByName(
-        fieldAnnotations.getAnnotations(), UNKNOWN_INITIALIZATION)) {
+        fieldAnnotations.getPrimaryAnnotations(), UNKNOWN_INITIALIZATION)) {
       return;
     }
     if (isUnknownInitialization(receiverType) || isUnderInitialization(receiverType)) {
@@ -720,12 +724,14 @@ public abstract class InitializationAnnotatedTypeFactory<
         // anything can be assigned to this field.
         type.replaceAnnotation(UNKNOWN_INITIALIZATION);
       } else if (computingAnnotatedTypeMirrorOfLHS) {
-        // The receiver is not initialized for this frame, but the type of a lhs is being computed.
+        // The receiver is not initialized for this frame, but the type of a lhs is being
+        // computed.
         // Change the type of the field to @UnknownInitialization so that
         // anything can be assigned to this field.
         type.replaceAnnotation(UNKNOWN_INITIALIZATION);
       } else {
-        // The receiver is not initialized for this frame and the type being computed is not a LHS.
+        // The receiver is not initialized for this frame and the type being computed is not
+        // a LHS.
         // Replace all annotations with the top annotation for that hierarchy.
         type.clearPrimaryAnnotations();
         type.addAnnotations(qualHierarchy.getTopAnnotations());
@@ -774,9 +780,9 @@ public abstract class InitializationAnnotatedTypeFactory<
     }
 
     @Override
-    public Void visitMethod(MethodTree node, AnnotatedTypeMirror p) {
-      Void result = super.visitMethod(node, p);
-      if (TreeUtils.isConstructor(node)) {
+    public Void visitMethod(MethodTree tree, AnnotatedTypeMirror p) {
+      Void result = super.visitMethod(tree, p);
+      if (TreeUtils.isConstructor(tree)) {
         assert p instanceof AnnotatedExecutableType;
         AnnotatedExecutableType exeType = (AnnotatedExecutableType) p;
         DeclaredType underlyingType = (DeclaredType) exeType.getReturnType().getUnderlyingType();
@@ -787,12 +793,18 @@ public abstract class InitializationAnnotatedTypeFactory<
     }
 
     @Override
-    public Void visitNewClass(NewClassTree node, AnnotatedTypeMirror p) {
-      super.visitNewClass(node, p);
+    public Void visitNewClass(NewClassTree tree, AnnotatedTypeMirror p) {
+      super.visitNewClass(tree, p);
       boolean allInitialized = true;
-      Type type = ((JCTree) node).type;
-      for (ExpressionTree a : node.getArguments()) {
-        final AnnotatedTypeMirror t = getAnnotatedType(a);
+      Type type = ((JCTree) tree).type;
+      for (ExpressionTree a : tree.getArguments()) {
+        if (!TreeUtils.isStandaloneExpression(a)) {
+          continue;
+        }
+        boolean oldShouldCache = shouldCache;
+        shouldCache = false;
+        AnnotatedTypeMirror t = getAnnotatedType(a);
+        shouldCache = oldShouldCache;
         allInitialized &= (isInitialized(t) || isFbcBottom(t));
       }
       if (!allInitialized) {
@@ -812,11 +824,11 @@ public abstract class InitializationAnnotatedTypeFactory<
     }
 
     @Override
-    public Void visitMemberSelect(MemberSelectTree node, AnnotatedTypeMirror annotatedTypeMirror) {
-      if (TreeUtils.isArrayLengthAccess(node)) {
+    public Void visitMemberSelect(MemberSelectTree tree, AnnotatedTypeMirror annotatedTypeMirror) {
+      if (TreeUtils.isArrayLengthAccess(tree)) {
         annotatedTypeMirror.replaceAnnotation(INITIALIZED);
       }
-      return super.visitMemberSelect(node, annotatedTypeMirror);
+      return super.visitMemberSelect(tree, annotatedTypeMirror);
     }
   }
 
@@ -834,12 +846,16 @@ public abstract class InitializationAnnotatedTypeFactory<
 
     /** Qualifier kind for the @{@link UnknownInitialization} annotation. */
     private final QualifierKind UNKNOWN_INIT;
+
     /** Qualifier kind for the @{@link UnderInitialization} annotation. */
     private final QualifierKind UNDER_INIT;
 
     /** Create an InitializationQualifierHierarchy. */
     protected InitializationQualifierHierarchy() {
-      super(InitializationAnnotatedTypeFactory.this.getSupportedTypeQualifiers(), elements);
+      super(
+          InitializationAnnotatedTypeFactory.this.getSupportedTypeQualifiers(),
+          elements,
+          InitializationAnnotatedTypeFactory.this);
       UNKNOWN_INIT = getQualifierKind(UNKNOWN_INITIALIZATION);
       UNDER_INIT = getQualifierKind(UNDER_INITALIZATION);
     }
@@ -887,9 +903,8 @@ public abstract class InitializationAnnotatedTypeFactory<
      */
     protected AnnotationMirror leastUpperBoundInitialization(
         AnnotationMirror anno1, QualifierKind qual1, AnnotationMirror anno2, QualifierKind qual2) {
-      if (!isInitializationAnnotation(anno1) || !isInitializationAnnotation(anno2)) {
-        return null;
-      }
+      assert isInitializationAnnotation(anno1);
+      assert isInitializationAnnotation(anno2);
 
       // Handle the case where one is a subtype of the other.
       if (isSubtypeInitialization(anno1, qual1, anno2, qual2)) {
@@ -922,7 +937,7 @@ public abstract class InitializationAnnotatedTypeFactory<
     }
 
     /**
-     * Returns the least upper bound of two types.
+     * Returns the least upper bound of two Java basetypes (without annotations).
      *
      * @param a the first argument
      * @param b the second argument
@@ -949,7 +964,7 @@ public abstract class InitializationAnnotatedTypeFactory<
      * @param qual2 a qualifier kind
      * @return the glb of anno1 and anno2
      */
-    protected AnnotationMirror greatestLowerBoundInitialization(
+    protected @Nullable AnnotationMirror greatestLowerBoundInitialization(
         AnnotationMirror anno1, QualifierKind qual1, AnnotationMirror anno2, QualifierKind qual2) {
       if (!isInitializationAnnotation(anno1) || !isInitializationAnnotation(anno2)) {
         return null;
@@ -990,4 +1005,19 @@ public abstract class InitializationAnnotatedTypeFactory<
       return createUnderInitializationAnnotation(typeFrame);
     }
   }
+
+  @Override
+  protected ParameterizedExecutableType methodFromUse(
+      ExpressionTree tree,
+      ExecutableElement methodElt,
+      AnnotatedTypeMirror receiverType,
+      boolean inferTypeArgs) {
+    ParameterizedExecutableType x =
+        super.methodFromUse(tree, methodElt, receiverType, inferTypeArgs);
+    if (tree.getKind() == Kind.MEMBER_REFERENCE
+        && ((MemberReferenceTree) tree).getMode() == ReferenceMode.NEW) {
+      x.executableType.getReturnType().replaceAnnotation(INITIALIZED);
+    }
+    return x;
+  }
 }
