@@ -8,18 +8,17 @@ import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.NewClassTree;
 import com.sun.source.tree.Tree;
-import com.sun.source.tree.TypeCastTree;
 import com.sun.source.tree.VariableTree;
 import java.lang.annotation.Annotation;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
-import java.util.Set;
 import java.util.StringJoiner;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
+import javax.lang.model.element.VariableElement;
 import org.checkerframework.checker.compilermsgs.qual.CompilerMessageKey;
 import org.checkerframework.checker.nullness.NullnessChecker;
 import org.checkerframework.common.basetype.BaseTypeChecker;
@@ -36,12 +35,13 @@ import org.checkerframework.framework.flow.CFAbstractValue;
 import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedDeclaredType;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutableType;
-import org.checkerframework.framework.util.AnnotationFormatter;
-import org.checkerframework.framework.util.DefaultAnnotationFormatter;
+import org.checkerframework.javacutil.AnnotationFormatter;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
+import org.checkerframework.javacutil.DefaultAnnotationFormatter;
 import org.checkerframework.javacutil.ElementUtils;
-import org.checkerframework.javacutil.Pair;
 import org.checkerframework.javacutil.TreeUtils;
+import org.plumelib.util.IPair;
 
 /**
  * The visitor for the freedom-before-commitment type-system. The freedom-before-commitment
@@ -57,6 +57,9 @@ public class InitializationVisitor<
   /** The annotation formatter. */
   protected final AnnotationFormatter annoFormatter;
 
+  /** List of fields in the current compilation unit that have been initialized. */
+  protected final List<VariableTree> initializedFields;
+
   /**
    * Creates a new InitializationVisitor.
    *
@@ -69,11 +72,11 @@ public class InitializationVisitor<
   }
 
   @Override
-  public void setRoot(CompilationUnitTree root) {
+  public void setRoot(CompilationUnitTree newRoot) {
     // Clean up the cache of initialized fields once per compilation unit.
     // Alternatively, but harder to determine, this could be done once per top-level class.
     initializedFields.clear();
-    super.setRoot(root);
+    super.setRoot(newRoot);
   }
 
   @Override
@@ -96,7 +99,7 @@ public class InitializationVisitor<
   }
 
   @Override
-  protected void commonAssignmentCheck(
+  protected boolean commonAssignmentCheck(
       Tree varTree,
       ExpressionTree valueExp,
       @CompilerMessageKey String errorKey,
@@ -106,13 +109,13 @@ public class InitializationVisitor<
       // cast is safe: a field access can only be an IdentifierTree or MemberSelectTree
       ExpressionTree lhs = (ExpressionTree) varTree;
       ExpressionTree y = valueExp;
-      Element el = TreeUtils.elementFromUse(lhs);
+      VariableElement el = TreeUtils.variableElementFromUse(lhs);
       AnnotatedTypeMirror xType = atypeFactory.getReceiverType(lhs);
       AnnotatedTypeMirror yType = atypeFactory.getAnnotatedType(y);
       // the special FBC rules do not apply if there is an explicit
       // UnknownInitialization annotation
-      Set<AnnotationMirror> fieldAnnotations =
-          atypeFactory.getAnnotatedType(TreeUtils.elementFromUse(lhs)).getAnnotations();
+      AnnotationMirrorSet fieldAnnotations =
+          atypeFactory.getAnnotatedType(el).getPrimaryAnnotations();
       if (!AnnotationUtils.containsSameByName(
           fieldAnnotations, atypeFactory.UNKNOWN_INITIALIZATION)) {
         if (!ElementUtils.isStatic(el)
@@ -126,19 +129,19 @@ public class InitializationVisitor<
             err = "initialization.field.write.unknown";
           }
           checker.reportError(varTree, err, varTree);
-          return; // prevent issuing another errow about subtyping
+          return false; // prevent issuing another error about subtyping
         }
       }
     }
-    super.commonAssignmentCheck(varTree, valueExp, errorKey, extraArgs);
+    return super.commonAssignmentCheck(varTree, valueExp, errorKey, extraArgs);
   }
 
   @Override
-  public Void visitVariable(VariableTree node, Void p) {
+  public Void visitVariable(VariableTree tree, Void p) {
     // is this a field (and not a local variable)?
-    if (TreeUtils.elementFromDeclaration(node).getKind().isField()) {
-      Set<AnnotationMirror> annotationMirrors =
-          atypeFactory.getAnnotatedType(node).getExplicitAnnotations();
+    if (TreeUtils.elementFromDeclaration(tree).getKind().isField()) {
+      AnnotationMirrorSet annotationMirrors =
+          atypeFactory.getAnnotatedType(tree).getExplicitAnnotations();
       // Fields cannot have commitment annotations.
       for (Class<? extends Annotation> c : atypeFactory.getInitializationAnnotations()) {
         for (AnnotationMirror a : annotationMirrors) {
@@ -146,13 +149,13 @@ public class InitializationVisitor<
             continue; // unknown initialization is allowed
           }
           if (atypeFactory.areSameByClass(a, c)) {
-            checker.reportError(node, "initialization.field.type", node);
+            checker.reportError(tree, "initialization.field.type", tree);
             break;
           }
         }
       }
     }
-    return super.visitVariable(node, p);
+    return super.visitVariable(tree, p);
   }
 
   @Override
@@ -162,9 +165,9 @@ public class InitializationVisitor<
       AnnotationMirror inferredAnnotation,
       CFAbstractStore<?, ?> store) {
     // also use the information about initialized fields to check contracts
-    final AnnotationMirror invariantAnno = atypeFactory.getFieldInvariantAnnotation();
+    AnnotationMirror invariantAnno = atypeFactory.getFieldInvariantAnnotation();
 
-    if (!atypeFactory.getQualifierHierarchy().isSubtype(invariantAnno, necessaryAnnotation)
+    if (!qualHierarchy.isSubtypeShallow(invariantAnno, necessaryAnnotation, expr.getType())
         || !(expr instanceof FieldAccess)) {
       return super.checkContract(expr, necessaryAnnotation, inferredAnnotation, store);
     }
@@ -179,7 +182,7 @@ public class InitializationVisitor<
       if (s.isFieldInitialized(fa.getField())) {
         AnnotatedTypeMirror fieldType = atypeFactory.getAnnotatedType(fa.getField());
         // is this an invariant-field?
-        if (AnnotationUtils.containsSame(fieldType.getAnnotations(), invariantAnno)) {
+        if (AnnotationUtils.containsSame(fieldType.getPrimaryAnnotations(), invariantAnno)) {
           return true;
         }
       }
@@ -187,13 +190,13 @@ public class InitializationVisitor<
       @SuppressWarnings("unchecked")
       Value value = (Value) store.getValue(fa.getReceiver());
 
-      Set<AnnotationMirror> receiverAnnoSet;
+      AnnotationMirrorSet receiverAnnoSet;
       if (value != null) {
         receiverAnnoSet = value.getAnnotations();
       } else if (fa.getReceiver() instanceof LocalVariable) {
         Element elem = ((LocalVariable) fa.getReceiver()).getElement();
         AnnotatedTypeMirror receiverType = atypeFactory.getAnnotatedType(elem);
-        receiverAnnoSet = receiverType.getAnnotations();
+        receiverAnnoSet = receiverType.getPrimaryAnnotations();
       } else {
         // Is there anything better we could do?
         return false;
@@ -210,7 +213,7 @@ public class InitializationVisitor<
       // The receiver is fully initialized and the field type
       // has the invariant type.
       if (isReceiverInitialized
-          && AnnotationUtils.containsSame(fieldType.getAnnotations(), invariantAnno)) {
+          && AnnotationUtils.containsSame(fieldType.getPrimaryAnnotations(), invariantAnno)) {
         return true;
       }
     }
@@ -218,56 +221,11 @@ public class InitializationVisitor<
   }
 
   @Override
-  public Void visitTypeCast(TypeCastTree node, Void p) {
-    AnnotatedTypeMirror exprType = atypeFactory.getAnnotatedType(node.getExpression());
-    AnnotatedTypeMirror castType = atypeFactory.getAnnotatedType(node);
-    AnnotationMirror exprAnno = null, castAnno = null;
-
-    // find commitment annotation
-    for (Class<? extends Annotation> a : atypeFactory.getInitializationAnnotations()) {
-      if (castType.hasAnnotation(a)) {
-        assert castAnno == null;
-        castAnno = castType.getAnnotation(a);
-      }
-      if (exprType.hasAnnotation(a)) {
-        assert exprAnno == null;
-        exprAnno = exprType.getAnnotation(a);
-      }
-    }
-
-    // TODO: this is most certainly unsafe!! (and may be hiding some problems)
-    // If we don't find a commitment annotation, then we just assume that
-    // the subtyping is alright.
-    // The case that has come up is with wildcards not getting a type for
-    // some reason, even though the default is @Initialized.
-    boolean isSubtype;
-    if (exprAnno == null || castAnno == null) {
-      isSubtype = true;
-    } else {
-      assert exprAnno != null && castAnno != null;
-      isSubtype = atypeFactory.getQualifierHierarchy().isSubtype(exprAnno, castAnno);
-    }
-
-    if (!isSubtype) {
-      checker.reportError(
-          node,
-          "initialization.cast",
-          annoFormatter.formatAnnotationMirror(exprAnno),
-          annoFormatter.formatAnnotationMirror(castAnno));
-      return p; // suppress cast.unsafe warning
-    }
-
-    return super.visitTypeCast(node, p);
-  }
-
-  protected final List<VariableTree> initializedFields;
-
-  @Override
-  public void processClassTree(ClassTree node) {
+  public void processClassTree(ClassTree tree) {
     // go through all members and look for initializers.
     // save all fields that are initialized and do not report errors about
     // them later when checking constructors.
-    for (Tree member : node.getMembers()) {
+    for (Tree member : tree.getMembers()) {
       if (member.getKind() == Tree.Kind.BLOCK && !((BlockTree) member).isStatic()) {
         BlockTree block = (BlockTree) member;
         Store store = atypeFactory.getRegularExitStore(block);
@@ -279,23 +237,24 @@ public class InitializationVisitor<
             store.addInitializedField(fieldInitialValue.fieldDecl.getField());
           }
         }
-        final List<VariableTree> init =
+        List<VariableTree> init =
             atypeFactory.getInitializedInvariantFields(store, getCurrentPath());
         initializedFields.addAll(init);
       }
     }
 
-    super.processClassTree(node);
+    super.processClassTree(tree);
 
     // Warn about uninitialized static fields.
-    Tree.Kind nodeKind = node.getKind();
-    // Skip interfaces (and annotations, which are interfaces).  In an interface, every static field
-    // must be initialized.  Java forbids uninitialized variables and static initalizer blocks.
+    Tree.Kind nodeKind = tree.getKind();
+    // Skip interfaces (and annotations, which are interfaces).  In an interface, every static
+    // field must be initialized.  Java forbids uninitialized variables and static initializer
+    // blocks.
     if (nodeKind != Tree.Kind.INTERFACE && nodeKind != Tree.Kind.ANNOTATION_TYPE) {
       boolean isStatic = true;
       // See GenericAnnotatedTypeFactory.performFlowAnalysis for why we use
       // the regular exit store of the class here.
-      Store store = atypeFactory.getRegularExitStore(node);
+      Store store = atypeFactory.getRegularExitStore(tree);
       // Add field values for fields with an initializer.
       for (FieldInitialValue<Value> fieldInitialValue :
           store.getAnalysis().getFieldInitialValues()) {
@@ -305,21 +264,21 @@ public class InitializationVisitor<
       }
 
       List<AnnotationMirror> receiverAnnotations = Collections.emptyList();
-      checkFieldsInitialized(node, isStatic, store, receiverAnnotations);
+      checkFieldsInitialized(tree, isStatic, store, receiverAnnotations);
     }
   }
 
   @Override
-  public Void visitMethod(MethodTree node, Void p) {
-    if (TreeUtils.isConstructor(node)) {
+  public void processMethodTree(String className, MethodTree tree) {
+    if (TreeUtils.isConstructor(tree)) {
       Collection<? extends AnnotationMirror> returnTypeAnnotations =
-          AnnotationUtils.getExplicitAnnotationsOnConstructorResult(node);
+          AnnotationUtils.getExplicitAnnotationsOnConstructorResult(tree);
       // check for invalid constructor return type
       for (Class<? extends Annotation> c :
           atypeFactory.getInvalidConstructorReturnTypeAnnotations()) {
         for (AnnotationMirror a : returnTypeAnnotations) {
           if (atypeFactory.areSameByClass(a, c)) {
-            checker.reportError(node, "initialization.constructor.return.type", node);
+            checker.reportError(tree, "initialization.constructor.return.type", tree);
             break;
           }
         }
@@ -327,36 +286,38 @@ public class InitializationVisitor<
 
       // Check that all fields have been initialized at the end of the constructor.
       boolean isStatic = false;
-      Store store = atypeFactory.getRegularExitStore(node);
-      List<? extends AnnotationMirror> receiverAnnotations = getAllReceiverAnnotations(node);
-      checkFieldsInitialized(node, isStatic, store, receiverAnnotations);
+      Store store = atypeFactory.getRegularExitStore(tree);
+      List<? extends AnnotationMirror> receiverAnnotations = getAllReceiverAnnotations(tree);
+      checkFieldsInitialized(tree, isStatic, store, receiverAnnotations);
     }
-    return super.visitMethod(node, p);
+    super.processMethodTree(className, tree);
   }
 
-  /** Returns the full list of annotations on the receiver. */
-  private List<? extends AnnotationMirror> getAllReceiverAnnotations(MethodTree node) {
+  /**
+   * Returns the full list of annotations on the receiver.
+   *
+   * @param tree a method declaration
+   * @return all the annotations on the method's receiver
+   */
+  private List<? extends AnnotationMirror> getAllReceiverAnnotations(MethodTree tree) {
     // TODO: get access to a Types instance and use it to get receiver type
     // Or, extend ExecutableElement with such a method.
-    // Note that we cannot use the receiver type from AnnotatedExecutableType, because that would
-    // only have the nullness annotations; here we want to see all annotations on the receiver.
-    List<? extends AnnotationMirror> rcvannos = null;
-    if (TreeUtils.isConstructor(node)) {
+    // Note that we cannot use the receiver type from AnnotatedExecutableType, because that
+    // would only have the nullness annotations; here we want to see all annotations on the
+    // receiver.
+    if (TreeUtils.isConstructor(tree)) {
       com.sun.tools.javac.code.Symbol meth =
-          (com.sun.tools.javac.code.Symbol) TreeUtils.elementFromDeclaration(node);
-      rcvannos = meth.getRawTypeAttributes();
-      if (rcvannos == null) {
-        rcvannos = Collections.emptyList();
-      }
+          (com.sun.tools.javac.code.Symbol) TreeUtils.elementFromDeclaration(tree);
+      return meth.getRawTypeAttributes();
     }
-    return rcvannos;
+    return Collections.emptyList();
   }
 
   /**
    * Checks that all fields (all static fields if {@code staticFields} is true) are initialized in
    * the given store.
    *
-   * @param node a {@link ClassTree} if {@code staticFields} is true; a {@link MethodTree} for a
+   * @param tree a {@link ClassTree} if {@code staticFields} is true; a {@link MethodTree} for a
    *     constructor if {@code staticFields} is false. This is where errors are reported, if they
    *     are not reported at the fields themselves
    * @param staticFields whether to check static fields or instance fields
@@ -370,7 +331,7 @@ public class InitializationVisitor<
   // GenericAnnotatedTypeFactory.initializationStaticStore and
   // GenericAnnotatedTypeFactory.initializationStore.
   protected void checkFieldsInitialized(
-      Tree node,
+      Tree tree,
       boolean staticFields,
       Store store,
       List<? extends AnnotationMirror> receiverAnnotations) {
@@ -382,31 +343,26 @@ public class InitializationVisitor<
     // Compact canonical record constructors do not generate visible assignments in the source,
     // but by definition they assign to all the record's fields so we don't need to
     // check for uninitialized fields in them:
-    if (node.getKind() == Tree.Kind.METHOD
-        && TreeUtils.isCompactCanonicalRecordConstructor((MethodTree) node)) {
+    if (tree.getKind() == Tree.Kind.METHOD
+        && TreeUtils.isCompactCanonicalRecordConstructor((MethodTree) tree)) {
       return;
     }
 
-    Pair<List<VariableTree>, List<VariableTree>> uninitializedFields =
+    IPair<List<VariableTree>, List<VariableTree>> uninitializedFields =
         atypeFactory.getUninitializedFields(
             store, getCurrentPath(), staticFields, receiverAnnotations);
     List<VariableTree> violatingFields = uninitializedFields.first;
     List<VariableTree> nonviolatingFields = uninitializedFields.second;
 
     // Remove fields that have already been initialized by an initializer block.
-    if (staticFields) {
-      violatingFields.removeAll(initializedFields);
-      nonviolatingFields.removeAll(initializedFields);
-    } else {
-      violatingFields.removeAll(initializedFields);
-      nonviolatingFields.removeAll(initializedFields);
-    }
+    violatingFields.removeAll(initializedFields);
+    nonviolatingFields.removeAll(initializedFields);
 
     // Errors are issued at the field declaration if the field is static or if the constructor
     // is the default constructor.
     // Errors are issued at the constructor declaration if the field is non-static and the
     // constructor is non-default.
-    boolean errorAtField = staticFields || TreeUtils.isSynthetic((MethodTree) node);
+    boolean errorAtField = staticFields || TreeUtils.isSynthetic((MethodTree) tree);
 
     String FIELDS_UNINITIALIZED_KEY =
         (staticFields
@@ -418,10 +374,12 @@ public class InitializationVisitor<
     // Remove fields with a relevant @SuppressWarnings annotation.
     violatingFields.removeIf(
         f ->
-            checker.shouldSuppressWarnings(TreeUtils.elementFromTree(f), FIELDS_UNINITIALIZED_KEY));
+            checker.shouldSuppressWarnings(
+                TreeUtils.elementFromDeclaration(f), FIELDS_UNINITIALIZED_KEY));
     nonviolatingFields.removeIf(
         f ->
-            checker.shouldSuppressWarnings(TreeUtils.elementFromTree(f), FIELDS_UNINITIALIZED_KEY));
+            checker.shouldSuppressWarnings(
+                TreeUtils.elementFromDeclaration(f), FIELDS_UNINITIALIZED_KEY));
 
     if (!violatingFields.isEmpty()) {
       if (errorAtField) {
@@ -435,7 +393,7 @@ public class InitializationVisitor<
         for (VariableTree f : violatingFields) {
           fieldsString.add(f.getName());
         }
-        checker.reportError(node, FIELDS_UNINITIALIZED_KEY, fieldsString);
+        checker.reportError(tree, FIELDS_UNINITIALIZED_KEY, fieldsString);
       }
     }
 
@@ -446,7 +404,7 @@ public class InitializationVisitor<
       List<VariableTree> uninitFields = new ArrayList<>(violatingFields);
       uninitFields.addAll(nonviolatingFields);
       for (VariableTree fieldTree : uninitFields) {
-        Element elt = TreeUtils.elementFromTree(fieldTree);
+        Element elt = TreeUtils.elementFromDeclaration(fieldTree);
         wpi.updateFieldFromType(
             fieldTree,
             elt,
