@@ -19,7 +19,6 @@ import com.sun.source.tree.Tree;
 import com.sun.source.util.TreePath;
 import java.util.Comparator;
 import java.util.List;
-import java.util.Set;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ElementKind;
@@ -30,13 +29,15 @@ import javax.lang.model.type.DeclaredType;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.type.TypeMirror;
 import javax.lang.model.util.ElementFilter;
-import javax.tools.Diagnostic.Kind;
+import javax.tools.Diagnostic;
+import org.checkerframework.checker.initialization.qual.UnknownInitialization;
 import org.checkerframework.checker.interning.qual.CompareToMethod;
 import org.checkerframework.checker.interning.qual.EqualsMethod;
 import org.checkerframework.checker.interning.qual.InternMethod;
 import org.checkerframework.checker.interning.qual.Interned;
 import org.checkerframework.checker.interning.qual.InternedDistinct;
 import org.checkerframework.checker.interning.qual.UsesObjectEquals;
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.signature.qual.CanonicalName;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.common.basetype.BaseTypeVisitor;
@@ -45,6 +46,7 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutableType;
 import org.checkerframework.framework.util.Heuristics;
 import org.checkerframework.javacutil.AnnotationBuilder;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.ElementUtils;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypesUtils;
@@ -56,7 +58,7 @@ import org.checkerframework.javacutil.TypesUtils;
  *
  * <ol>
  *   <li value="1">either argument to a "==" or "!=" comparison is not Interned (error
- *       "not.interned"). As a special case, the comparison is permitted if either arugment is
+ *       "not.interned"). As a special case, the comparison is permitted if either argument is
  *       InternedDistinct.
  *   <li value="2">the receiver and argument for a call to an equals method are both Interned
  *       (optional warning "unnecessary.equals")
@@ -68,15 +70,17 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
 
   /** The @Interned annotation. */
   private final AnnotationMirror INTERNED = AnnotationBuilder.fromClass(elements, Interned.class);
+
   /** The @InternedDistinct annotation. */
   private final AnnotationMirror INTERNED_DISTINCT =
       AnnotationBuilder.fromClass(elements, InternedDistinct.class);
+
   /**
    * The declared type of which the equality tests should be tested, if the user explicitly passed
    * one. The user can pass the class name via the {@code -Acheckclass=...} option. Null if no class
    * is specified, or the class specified isn't in the classpath.
    */
-  private final DeclaredType typeToCheck = typeToCheck();
+  private final @Nullable DeclaredType typeToCheck = typeToCheck();
 
   /** The Comparable.compareTo method. */
   private final ExecutableElement comparableCompareTo =
@@ -107,19 +111,19 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
 
   /** Checks comparison operators, == and !=, for INTERNING violations. */
   @Override
-  public Void visitBinary(BinaryTree node, Void p) {
+  public Void visitBinary(BinaryTree tree, Void p) {
 
     // No checking unless the operator is "==" or "!=".
-    if (!(node.getKind() == Tree.Kind.EQUAL_TO || node.getKind() == Tree.Kind.NOT_EQUAL_TO)) {
-      return super.visitBinary(node, p);
+    if (!(tree.getKind() == Tree.Kind.EQUAL_TO || tree.getKind() == Tree.Kind.NOT_EQUAL_TO)) {
+      return super.visitBinary(tree, p);
     }
 
-    ExpressionTree leftOp = node.getLeftOperand();
-    ExpressionTree rightOp = node.getRightOperand();
+    ExpressionTree leftOp = tree.getLeftOperand();
+    ExpressionTree rightOp = tree.getRightOperand();
 
     // Check passes if either arg is null.
     if (leftOp.getKind() == Tree.Kind.NULL_LITERAL || rightOp.getKind() == Tree.Kind.NULL_LITERAL) {
-      return super.visitBinary(node, p);
+      return super.visitBinary(tree, p);
     }
 
     AnnotatedTypeMirror left = atypeFactory.getAnnotatedType(leftOp);
@@ -127,12 +131,12 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
 
     // If either argument is a primitive, check passes due to auto-unboxing
     if (left.getKind().isPrimitive() || right.getKind().isPrimitive()) {
-      return super.visitBinary(node, p);
+      return super.visitBinary(tree, p);
     }
 
     if (left.hasEffectiveAnnotation(INTERNED_DISTINCT)
         || right.hasEffectiveAnnotation(INTERNED_DISTINCT)) {
-      return super.visitBinary(node, p);
+      return super.visitBinary(tree, p);
     }
 
     // If shouldCheckExpression returns true for either the LHS or RHS,
@@ -155,28 +159,29 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     // in order to assume this).
 
     // Now suppose the user passes -AcheckClass=A on the command-line.
-    // I is not a subtype or supertype of A, so shouldCheckExpression will not return true for I.
+    // I is not a subtype or supertype of A, so shouldCheckExpression will not return true for
+    // I.
     // But the interning check must be performed, given the argument above.  Therefore if
     // shouldCheckExpression returns true for either the LHS or the RHS, this method proceeds
     // with the interning check.
 
     if (!shouldCheckExpression(leftOp) && !shouldCheckExpression(rightOp)) {
-      return super.visitBinary(node, p);
+      return super.visitBinary(tree, p);
     }
 
     // Syntactic checks for legal uses of ==
-    if (suppressInsideComparison(node)) {
-      return super.visitBinary(node, p);
+    if (suppressInsideComparison(tree)) {
+      return super.visitBinary(tree, p);
     }
-    if (suppressEarlyEquals(node)) {
-      return super.visitBinary(node, p);
+    if (suppressEarlyEquals(tree)) {
+      return super.visitBinary(tree, p);
     }
-    if (suppressEarlyCompareTo(node)) {
-      return super.visitBinary(node, p);
+    if (suppressEarlyCompareTo(tree)) {
+      return super.visitBinary(tree, p);
     }
 
     if (suppressEqualsIfClassIsAnnotated(left, right)) {
-      return super.visitBinary(node, p);
+      return super.visitBinary(tree, p);
     }
 
     Element leftElt = TypesUtils.getTypeElement(left.getUnderlyingType());
@@ -193,7 +198,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
             && atypeFactory.getDeclAnnotation(rightElt, UsesObjectEquals.class) != null))) {
       checker.reportError(rightOp, "not.interned");
     }
-    return super.visitBinary(node, p);
+    return super.visitBinary(tree, p);
   }
 
   /**
@@ -201,25 +206,26 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    * equality is safe.
    */
   @Override
-  public Void visitMethodInvocation(MethodInvocationTree node, Void p) {
-    if (isInvocationOfEquals(node)) {
-      AnnotatedTypeMirror receiverType = atypeFactory.getReceiverType(node);
-      AnnotatedTypeMirror comp = atypeFactory.getAnnotatedType(node.getArguments().get(0));
+  public Void visitMethodInvocation(MethodInvocationTree tree, Void p) {
+    if (isInvocationOfEquals(tree)) {
+      AnnotatedTypeMirror receiverType = atypeFactory.getReceiverType(tree);
+      assert receiverType != null : "@AssumeAssertion(nullness)";
+      AnnotatedTypeMirror comp = atypeFactory.getAnnotatedType(tree.getArguments().get(0));
 
       if (this.checker.getLintOption("dotequals", true)
           && receiverType.hasEffectiveAnnotation(INTERNED)
           && comp.hasEffectiveAnnotation(INTERNED)) {
-        checker.reportWarning(node, "unnecessary.equals");
+        checker.reportWarning(tree, "unnecessary.equals");
       }
     }
 
-    return super.visitMethodInvocation(node, p);
+    return super.visitMethodInvocation(tree, p);
   }
 
   // Ensure that method annotations are not written on methods they don't apply to.
   @Override
-  public Void visitMethod(MethodTree node, Void p) {
-    ExecutableElement methElt = TreeUtils.elementFromDeclaration(node);
+  public void processMethodTree(String className, MethodTree tree) {
+    ExecutableElement methElt = TreeUtils.elementFromDeclaration(tree);
     boolean hasCompareToMethodAnno =
         atypeFactory.getDeclAnnotation(methElt, CompareToMethod.class) != null;
     boolean hasEqualsMethodAnno =
@@ -229,15 +235,15 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     int params = methElt.getParameters().size();
     if (hasCompareToMethodAnno && !(params == 1 || params == 2)) {
       checker.reportError(
-          node, "invalid.method.annotation", "@CompareToMethod", "1 or 2", methElt, params);
+          tree, "invalid.method.annotation", "@CompareToMethod", "1 or 2", methElt, params);
     } else if (hasEqualsMethodAnno && !(params == 1 || params == 2)) {
       checker.reportError(
-          node, "invalid.method.annotation", "@EqualsMethod", "1 or 2", methElt, params);
+          tree, "invalid.method.annotation", "@EqualsMethod", "1 or 2", methElt, params);
     } else if (hasInternMethodAnno && !(params == 0)) {
-      checker.reportError(node, "invalid.method.annotation", "@InternMethod", "0", methElt, params);
+      checker.reportError(tree, "invalid.method.annotation", "@InternMethod", "0", methElt, params);
     }
 
-    return super.visitMethod(node, p);
+    super.processMethodTree(className, tree);
   }
 
   /**
@@ -346,7 +352,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     } else if (tree.getKind() == Tree.Kind.NEW_CLASS) {
       NewClassTree newClassTree = (NewClassTree) tree;
       TypeMirror typeMirror = TreeUtils.typeOf(newClassTree);
-      Set<AnnotationMirror> bounds = atypeFactory.getTypeDeclarationBounds(typeMirror);
+      AnnotationMirrorSet bounds = atypeFactory.getTypeDeclarationBounds(typeMirror);
       // Don't issue an invalid type warning for creations of objects of interned classes;
       // instead, issue an interned.object.creation if required.
       if (atypeFactory.containsSameByClass(bounds, Interned.class)) {
@@ -369,7 +375,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    */
   private boolean checkCreationOfInternedObject(
       NewClassTree newInternedObject, AnnotatedExecutableType constructor) {
-    if (constructor.getReturnType().hasAnnotation(Interned.class)) {
+    if (constructor.getReturnType().hasPrimaryAnnotation(Interned.class)) {
       return true;
     }
     TreePath path = getCurrentPath();
@@ -381,7 +387,8 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
       if (parentPath != null && parentPath.getParentPath() != null) {
         Tree parent = parentPath.getParentPath().getLeaf();
         if (parent.getKind() == Tree.Kind.METHOD_INVOCATION) {
-          // Allow new MyInternType().intern(), where "intern" is any method marked @InternMethod.
+          // Allow new MyInternType().intern(), where "intern" is any method marked
+          // @InternMethod.
           ExecutableElement elt = TreeUtils.elementFromUse((MethodInvocationTree) parent);
           if (atypeFactory.getDeclAnnotation(elt, InternMethod.class) != null) {
             return true;
@@ -401,11 +408,11 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
   /**
    * Returns the method that overrides Object.equals, or null.
    *
-   * @param node a class
+   * @param tree a class
    * @return the class's implementation of equals, or null
    */
-  private MethodTree equalsImplementation(ClassTree node) {
-    List<? extends Tree> members = node.getMembers();
+  private @Nullable MethodTree equalsImplementation(ClassTree tree) {
+    List<? extends Tree> members = tree.getMembers();
     for (Tree member : members) {
       if (member instanceof MethodTree) {
         MethodTree mTree = (MethodTree) member;
@@ -425,11 +432,11 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    * idiom of writing an equals method with a non-Object parameter, in addition to the equals method
    * that overrides {@link Object#equals(Object)}.
    *
-   * @param node a method invocation node
-   * @return true iff {@code node} is a invocation of {@code equals()}
+   * @param tree a method invocation tree
+   * @return true iff {@code tree} is a invocation of {@code equals()}
    */
-  public static boolean isInvocationOfEquals(MethodInvocationTree node) {
-    ExecutableElement method = TreeUtils.elementFromUse(node);
+  public static boolean isInvocationOfEquals(MethodInvocationTree tree) {
+    ExecutableElement method = TreeUtils.elementFromUse(tree);
     return (method.getParameters().size() == 1
         && method.getReturnType().getKind() == TypeKind.BOOLEAN
         // method symbols only have simple names
@@ -452,19 +459,19 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    *       statement returns zero, and the comparison tests "this" against the method's parameter
    * </ol>
    *
-   * @param node the comparison to check
+   * @param binaryTree the comparison to check
    * @return true if one of the supported heuristics is matched, false otherwise
    */
   // TODO: handle != comparisons too!
   // TODO: handle more methods, such as early return from addAll when this == arg
-  private boolean suppressInsideComparison(final BinaryTree node) {
+  private boolean suppressInsideComparison(BinaryTree binaryTree) {
     // Only handle == binary trees
-    if (node.getKind() != Tree.Kind.EQUAL_TO) {
+    if (binaryTree.getKind() != Tree.Kind.EQUAL_TO) {
       return false;
     }
 
-    Tree left = node.getLeftOperand();
-    Tree right = node.getRightOperand();
+    ExpressionTree left = binaryTree.getLeftOperand();
+    ExpressionTree right = binaryTree.getRightOperand();
 
     // Only valid if we're comparing identifiers.
     if (!(left.getKind() == Tree.Kind.IDENTIFIER && right.getKind() == Tree.Kind.IDENTIFIER)) {
@@ -493,12 +500,12 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
       // Set ifStatementTree and methodTree
       {
         TreePath ppath = parentPath;
-        Tree tree;
-        while ((tree = ppath.getLeaf()) != null) {
-          if (tree.getKind() == Tree.Kind.IF) {
-            ifStatementTree = tree;
-          } else if (tree.getKind() == Tree.Kind.METHOD) {
-            methodTree = (MethodTree) tree;
+        Tree candidateTree;
+        while ((candidateTree = ppath.getLeaf()) != null) {
+          if (candidateTree.getKind() == Tree.Kind.IF) {
+            ifStatementTree = candidateTree;
+          } else if (candidateTree.getKind() == Tree.Kind.METHOD) {
+            methodTree = (MethodTree) candidateTree;
             break;
           }
           ppath = ppath.getParentPath();
@@ -520,8 +527,8 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     ExecutableElement enclosingMethod = TreeUtils.elementFromDeclaration(methodTree);
     assert enclosingMethod != null;
 
-    final Element lhs = TreeUtils.elementFromUse((IdentifierTree) left);
-    final Element rhs = TreeUtils.elementFromUse((IdentifierTree) right);
+    Element lhs = TreeUtils.elementFromUse((IdentifierTree) left);
+    Element rhs = TreeUtils.elementFromUse((IdentifierTree) right);
 
     // Matcher to check for if statement that returns zero
     Heuristics.Matcher matcherIfReturnsZero =
@@ -559,7 +566,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     // "return 0" statement (for the Comparator.compare heuristic).
     if (overrides(enclosingMethod, Comparator.class, "compare")
         || (hasCompareToMethodAnno && params == 2)) {
-      final boolean returnsZero =
+      boolean returnsZero =
           new Heuristics.Within(new Heuristics.OfKind(Tree.Kind.IF, matcherIfReturnsZero))
               .match(getCurrentPath());
 
@@ -589,7 +596,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     } else if (overrides(enclosingMethod, Comparable.class, "compareTo")
         || (hasCompareToMethodAnno && params == 1)) {
 
-      final boolean returnsZero =
+      boolean returnsZero =
           new Heuristics.Within(new Heuristics.OfKind(Tree.Kind.IF, matcherIfReturnsZero))
               .match(getCurrentPath());
 
@@ -617,19 +624,20 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    * (a == b) || (a != null && a.equals(b))
    * }</pre>
    *
-   * Returns true iff the given node fits this pattern.
+   * Returns true iff the given tree fits this pattern.
    *
-   * @return true iff the node fits a pattern such as (a == b || a.equals(b))
+   * @param topBinaryTree the binary operation to check
+   * @return true iff the tree fits a pattern such as (a == b || a.equals(b))
    */
-  private boolean suppressEarlyEquals(final BinaryTree node) {
+  private boolean suppressEarlyEquals(BinaryTree topBinaryTree) {
     // Only handle == binary trees
-    if (node.getKind() != Tree.Kind.EQUAL_TO) {
+    if (topBinaryTree.getKind() != Tree.Kind.EQUAL_TO) {
       return false;
     }
 
     // should strip parens
-    final ExpressionTree left = TreeUtils.withoutParens(node.getLeftOperand());
-    final ExpressionTree right = TreeUtils.withoutParens(node.getRightOperand());
+    ExpressionTree left = TreeUtils.withoutParens(topBinaryTree.getLeftOperand());
+    ExpressionTree right = TreeUtils.withoutParens(topBinaryTree.getRightOperand());
 
     // looking for ((a == b || a.equals(b))
     Heuristics.Matcher matcherEqOrEquals =
@@ -656,9 +664,10 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
             ExpressionTree rightTree = tree.getRightOperand();
 
             if (tree.getKind() == Tree.Kind.CONDITIONAL_OR) {
-              if (TreeUtils.sameTree(leftTree, node)) {
+              if (TreeUtils.sameTree(leftTree, topBinaryTree)) {
                 // left is "a==b"
-                // check right, which should be a.equals(b) or b.equals(a) or similar
+                // check right, which should be a.equals(b) or b.equals(a) or
+                // similar
                 return visit(rightTree, p);
               } else {
                 return false;
@@ -738,26 +747,27 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
 
   /**
    * Pattern matches to prevent false positives of the form {@code (a == b || a.compareTo(b) == 0)}.
-   * Returns true iff the given node fits this pattern.
+   * Returns true iff the given tree fits this pattern.
    *
-   * @return true iff the node fits the pattern (a == b || a.compareTo(b) == 0)
+   * @param topBinaryTree the binary operation to check
+   * @return true iff the tree fits the pattern (a == b || a.compareTo(b) == 0)
    */
-  private boolean suppressEarlyCompareTo(final BinaryTree node) {
+  private boolean suppressEarlyCompareTo(BinaryTree topBinaryTree) {
     // Only handle == binary trees
-    if (node.getKind() != Tree.Kind.EQUAL_TO) {
+    if (topBinaryTree.getKind() != Tree.Kind.EQUAL_TO) {
       return false;
     }
 
-    Tree left = TreeUtils.withoutParens(node.getLeftOperand());
-    Tree right = TreeUtils.withoutParens(node.getRightOperand());
+    ExpressionTree left = TreeUtils.withoutParens(topBinaryTree.getLeftOperand());
+    ExpressionTree right = TreeUtils.withoutParens(topBinaryTree.getRightOperand());
 
     // Only valid if we're comparing identifiers.
     if (!(left.getKind() == Tree.Kind.IDENTIFIER && right.getKind() == Tree.Kind.IDENTIFIER)) {
       return false;
     }
 
-    final Element lhs = TreeUtils.elementFromUse((IdentifierTree) left);
-    final Element rhs = TreeUtils.elementFromUse((IdentifierTree) right);
+    Element lhs = TreeUtils.elementFromUse((IdentifierTree) left);
+    Element rhs = TreeUtils.elementFromUse((IdentifierTree) right);
 
     // looking for ((a == b || a.compareTo(b) == 0)
     Heuristics.Matcher matcherEqOrCompareTo =
@@ -786,7 +796,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
               @InternedDistinct ExpressionTree leftTree = tree.getLeftOperand(); // looking for a==b
               ExpressionTree rightTree = tree.getRightOperand(); // looking for a.compareTo(b) == 0
               // or b.compareTo(a) == 0
-              if (leftTree != node) {
+              if (leftTree != topBinaryTree) {
                 return false;
               }
               if (rightTree.getKind() != Tree.Kind.EQUAL_TO) {
@@ -867,18 +877,21 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
 
     if (tm.getKind() != TypeKind.DECLARED) {
       checker.message(
-          Kind.WARNING, "InterningVisitor.classIsAnnotated: tm = %s (%s)", tm, tm.getClass());
+          Diagnostic.Kind.WARNING,
+          "InterningVisitor.classIsAnnotated: tm = %s (%s)",
+          tm,
+          tm.getClass());
     }
     Element classElt = ((DeclaredType) tm).asElement();
     if (classElt == null) {
       checker.message(
-          Kind.WARNING,
+          Diagnostic.Kind.WARNING,
           "InterningVisitor.classIsAnnotated: classElt = null for tm = %s (%s)",
           tm,
           tm.getClass());
     }
     if (classElt != null) {
-      Set<AnnotationMirror> bound = atypeFactory.getTypeDeclarationBounds(tm);
+      AnnotationMirrorSet bound = atypeFactory.getTypeDeclarationBounds(tm);
       return atypeFactory.containsSameByClass(bound, Interned.class);
     }
     return false;
@@ -891,7 +904,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    * @param scope the scope to search for the element corresponding to "this" in
    * @return the element corresponding to "this" in the given scope, or null if not found
    */
-  private Element getThis(Scope scope) {
+  private @Nullable Element getThis(Scope scope) {
     for (Element e : scope.getLocalElements()) {
       if (e.getSimpleName().contentEquals("this")) {
         return e;
@@ -901,7 +914,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
   }
 
   /**
-   * Determines whether or not the given element overrides the named method in the named class.
+   * Returns true if the given element overrides the named method in the named class.
    *
    * @param e an element for a method
    * @param clazz the class
@@ -930,7 +943,8 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
    *
    * @return the type to check
    */
-  DeclaredType typeToCheck() {
+  @Nullable DeclaredType typeToCheck(
+      @UnknownInitialization(BaseTypeVisitor.class) InterningVisitor this) {
     @SuppressWarnings("signature:assignment") // user input
     @CanonicalName String className = checker.getOption("checkclass");
     if (className == null) {
