@@ -5,7 +5,6 @@ import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.VariableElement;
@@ -31,6 +30,7 @@ import org.checkerframework.dataflow.cfg.node.ReturnNode;
 import org.checkerframework.dataflow.cfg.node.ThrowNode;
 import org.checkerframework.dataflow.expression.JavaExpression;
 import org.checkerframework.dataflow.expression.LocalVariable;
+import org.checkerframework.dataflow.util.PurityUtils;
 import org.checkerframework.framework.flow.CFAbstractStore;
 import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedDeclaredType;
@@ -38,6 +38,7 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutab
 import org.checkerframework.framework.type.visitor.SimpleAnnotatedTypeScanner;
 import org.checkerframework.framework.util.AnnotatedTypes;
 import org.checkerframework.javacutil.AnnotationBuilder;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypeSystemError;
@@ -61,8 +62,10 @@ public class NullnessTransfer
 
   /** The @{@link NonNull} annotation. */
   protected final AnnotationMirror NONNULL;
+
   /** The @{@link Nullable} annotation. */
   protected final AnnotationMirror NULLABLE;
+
   /** The @{@link PolyNull} annotation. */
   protected final AnnotationMirror POLYNULL;
 
@@ -82,6 +85,16 @@ public class NullnessTransfer
    */
   protected final @Nullable KeyForAnnotatedTypeFactory keyForTypeFactory;
 
+  /** True if -AassumeKeyFor was provided on the command line. */
+  private final boolean assumeKeyFor;
+
+  /**
+   * True if invocationPreservesArgumentNullness flag is turned on, meaning that after a method call
+   * or constructor invocation, non-null arguments of the invocation (including the receiver) are
+   * assumed to still be non-null.
+   */
+  private final boolean nonNullAssumptionAfterInvocation;
+
   /**
    * Create a new NullnessTransfer for the given analysis.
    *
@@ -92,7 +105,8 @@ public class NullnessTransfer
     this.nullnessTypeFactory = (NullnessAnnotatedTypeFactory) analysis.getTypeFactory();
     Elements elements = nullnessTypeFactory.getElementUtils();
     BaseTypeChecker checker = nullnessTypeFactory.getChecker();
-    if (checker.hasOption("assumeKeyFor")) {
+    assumeKeyFor = checker.hasOption("assumeKeyFor");
+    if (assumeKeyFor) {
       this.keyForTypeFactory = null;
     } else {
       // It is error-prone to put a type factory in a field.  It is OK here because
@@ -111,6 +125,12 @@ public class NullnessTransfer
                 TypesUtils.typeFromClass(Map.class, analysis.getTypes(), elements),
                 nullnessTypeFactory,
                 false);
+
+    nonNullAssumptionAfterInvocation =
+        !analysis
+            .getTypeFactory()
+            .getChecker()
+            .getBooleanOption("invocationPreservesArgumentNullness", true);
   }
 
   /**
@@ -202,15 +222,13 @@ public class NullnessTransfer
         }
       }
 
-      Set<AnnotationMirror> secondAnnos =
-          secondValue != null
-              ? secondValue.getAnnotations()
-              : AnnotationUtils.createAnnotationSet();
-      if (nullnessTypeFactory.containsSameByClass(secondAnnos, PolyNull.class)) {
+      if (secondValue != null
+          && nullnessTypeFactory.containsSameByClass(
+              secondValue.getAnnotations(), PolyNull.class)) {
         thenStore = thenStore == null ? res.getThenStore() : thenStore;
         elseStore = elseStore == null ? res.getElseStore() : elseStore;
         // TODO: methodTree is null for lambdas.  Handle that case.  See Issue3850.java.
-        MethodTree methodTree = analysis.getContainingMethod(secondNode.getTree());
+        MethodTree methodTree = analysis.getEnclosingMethod(secondNode.getTree());
         ExecutableElement methodElem =
             methodTree == null ? null : TreeUtils.elementFromDeclaration(methodTree);
         if (notEqualTo) {
@@ -250,7 +268,7 @@ public class NullnessTransfer
         return false;
       }
 
-      if (varType.hasAnnotation(POLYNULL)) {
+      if (varType.hasPrimaryAnnotation(POLYNULL)) {
         NullnessValue v = s.getValue(new LocalVariable(var));
         if (!AnnotationUtils.containsSameByName(v.getAnnotations(), NONNULL)) {
           return false;
@@ -283,7 +301,7 @@ public class NullnessTransfer
         isTopLevel = false;
         return false;
       } else {
-        return type.hasAnnotation(POLYNULL);
+        return type.hasPrimaryAnnotation(POLYNULL);
       }
     }
   }
@@ -320,7 +338,11 @@ public class NullnessTransfer
   public TransferResult<NullnessValue, NullnessStore> visitMethodAccess(
       MethodAccessNode n, TransferInput<NullnessValue, NullnessStore> p) {
     TransferResult<NullnessValue, NullnessStore> result = super.visitMethodAccess(n, p);
-    makeNonNull(result, n.getReceiver());
+    // The receiver of an instance method access is non-null. A static method access does not
+    // ensure that the receiver is non-null.
+    if (!n.isStatic()) {
+      makeNonNull(result, n.getReceiver());
+    }
     return result;
   }
 
@@ -328,7 +350,11 @@ public class NullnessTransfer
   public TransferResult<NullnessValue, NullnessStore> visitFieldAccess(
       FieldAccessNode n, TransferInput<NullnessValue, NullnessStore> p) {
     TransferResult<NullnessValue, NullnessStore> result = super.visitFieldAccess(n, p);
-    makeNonNull(result, n.getReceiver());
+    // The receiver of an instance field access is non-null. A static field access does not
+    // ensure that the receiver is non-null.
+    if (!n.isStatic()) {
+      makeNonNull(result, n.getReceiver());
+    }
     return result;
   }
 
@@ -340,11 +366,22 @@ public class NullnessTransfer
     return result;
   }
 
-  /*
-   * Provided that m is of a type that implements interface java.util.Map:
+  /**
+   * {@inheritDoc}
+   *
+   * <p>When the invocationPreservesArgumentNullness flag is turned on, the receiver and arguments
+   * that are passed to non-null parameters in a method call or constructor invocation are unsoundly
+   * assumed to be non-null after the invocation.
+   *
+   * <p>When the flag is turned off, the analysis is sound: it checks that the method is
+   * SideEffectFree or the receiver is unassignable. Only if either one of the two is true, is the
+   * receiver made non-null. Similar logic is applied to the arguments of the invocation.
+   *
+   * <p>Provided that m is of a type that implements interface java.util.Map:
+   *
    * <ul>
-   * <li>Given a call m.get(k), if k is @KeyFor("m") and m's value type is @NonNull,
-   *     then the result is @NonNull in the thenStore and elseStore of the transfer result.
+   *   <li>Given a call m.get(k), if k is @KeyFor("m") and m's value type is @NonNull, then the
+   *       result is @NonNull in the thenStore and elseStore of the transfer result.
    * </ul>
    */
   @Override
@@ -352,21 +389,31 @@ public class NullnessTransfer
       MethodInvocationNode n, TransferInput<NullnessValue, NullnessStore> in) {
     TransferResult<NullnessValue, NullnessStore> result = super.visitMethodInvocation(n, in);
 
-    // Make receiver non-null.
+    MethodInvocationTree tree = n.getTree();
+    ExecutableElement method = TreeUtils.elementFromUse(tree);
+
+    boolean isMethodSideEffectFree =
+        atypeFactory.isSideEffectFree(method) || PurityUtils.isSideEffectFree(atypeFactory, method);
     Node receiver = n.getTarget().getReceiver();
-    makeNonNull(result, receiver);
+    if (nonNullAssumptionAfterInvocation
+        || isMethodSideEffectFree
+        || !JavaExpression.fromNode(receiver).isAssignableByOtherCode()) {
+      // Make receiver non-null.
+      makeNonNull(result, receiver);
+    }
 
     // For all formal parameters with a non-null annotation, make the actual argument non-null.
     // The point of this is to prevent cascaded errors -- the Nullness Checker will issue a
     // warning for the method invocation, but not for subsequent uses of the argument.  See test
     // case FlowNullness.java.
-    MethodInvocationTree tree = n.getTree();
-    ExecutableElement method = TreeUtils.elementFromUse(tree);
     AnnotatedExecutableType methodType = nullnessTypeFactory.getAnnotatedType(method);
     List<AnnotatedTypeMirror> methodParams = methodType.getParameterTypes();
     List<? extends ExpressionTree> methodArgs = tree.getArguments();
     for (int i = 0; i < methodParams.size() && i < methodArgs.size(); ++i) {
-      if (methodParams.get(i).hasAnnotation(NONNULL)) {
+      if (methodParams.get(i).hasPrimaryAnnotation(NONNULL)
+          && (nonNullAssumptionAfterInvocation
+              || isMethodSideEffectFree
+              || !JavaExpression.fromTree(methodArgs.get(i)).isAssignableByOtherCode())) {
         makeNonNull(result, n.getArgument(i));
       }
     }
@@ -379,7 +426,7 @@ public class NullnessTransfer
         String mapName = JavaExpression.fromNode(receiver).toString();
         isKeyFor = keyForTypeFactory.isKeyForMap(mapName, methodArgs.get(0));
       } else {
-        isKeyFor = analysis.getTypeFactory().getChecker().hasOption("assumeKeyFor");
+        isKeyFor = assumeKeyFor;
       }
       if (isKeyFor) {
         AnnotatedTypeMirror receiverType = nullnessTypeFactory.getReceiverType(n.getTree());
@@ -407,7 +454,7 @@ public class NullnessTransfer
       throw new TypeSystemError("Wrong number %d of type arguments: %s", numTypeArguments, mapType);
     }
     AnnotatedTypeMirror valueType = mapType.getTypeArguments().get(1);
-    return valueType.hasAnnotation(NULLABLE);
+    return valueType.hasPrimaryAnnotation(NULLABLE);
   }
 
   @Override
@@ -424,10 +471,14 @@ public class NullnessTransfer
     }
   }
 
-  /** Creates a dummy abstract value (whose type is not supposed to be looked at). */
+  /**
+   * Creates a dummy abstract value (whose type is not supposed to be looked at).
+   *
+   * @return a dummy abstract value
+   */
   private NullnessValue createDummyValue() {
     TypeMirror dummy = analysis.getEnv().getTypeUtils().getPrimitiveType(TypeKind.BOOLEAN);
-    Set<AnnotationMirror> annos = AnnotationUtils.createAnnotationSet();
+    AnnotationMirrorSet annos = new AnnotationMirrorSet();
     annos.addAll(nullnessTypeFactory.getQualifierHierarchy().getBottomAnnotations());
     return new NullnessValue(analysis, annos, dummy);
   }
