@@ -18,7 +18,7 @@ public class OptionalTest extends CheckerFrameworkPerDirectoryTest {
         testFiles,
         org.checkerframework.checker.optional.OptionalChecker.class,
         "optional",
-        "-Anomsgtext");
+        "-AoptionalMapAssumeNonNull");
   }
 
   @Parameters
