@@ -11,8 +11,11 @@ public class NullnessReflectionExampleTest {
   }
 
   String getCurrentCity()
-      throws NoSuchMethodException, SecurityException, IllegalAccessException,
-          IllegalArgumentException, InvocationTargetException {
+      throws NoSuchMethodException,
+          SecurityException,
+          IllegalAccessException,
+          IllegalArgumentException,
+          InvocationTargetException {
     @MethodVal(
         className = "NullnessReflectionExampleTest",
         methodName = "getCurrentLocation",
