@@ -3,15 +3,17 @@
 import org.checkerframework.checker.mustcall.qual.*;
 
 class Enclosing {
-  @MustCall("a") static class Foo {
+  @InheritableMustCall("a")
+  static class Foo {
     void a() {}
   }
 
   static class Nested {
-    // :: error: required.method.not.called
+    // :: error: (required.method.not.called)
     @Owning Foo foo;
 
     @CreatesMustCallFor("this")
+    // :: error: (creates.mustcall.for.invalid.target)
     void initFoo() {
       if (this.foo == null) {
         this.foo = new Foo();
