@@ -5,12 +5,12 @@ import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.Tree;
 import java.util.Comparator;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.IdentityHashMap;
-import java.util.Map;
 import java.util.Objects;
 import java.util.PriorityQueue;
 import java.util.Set;
-import javax.lang.model.element.Element;
+import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeMirror;
 import org.checkerframework.checker.interning.qual.FindDistinct;
 import org.checkerframework.checker.interning.qual.InternedDistinct;
@@ -35,7 +35,7 @@ import org.checkerframework.javacutil.ElementUtils;
  *
  * @param <V> the abstract value type to be tracked by the analysis
  * @param <S> the store type used in the analysis
- * @param <T> the transfer function type that is used to approximated runtime behavior
+ * @param <T> the transfer function type that is used to approximate run-time behavior
  */
 public abstract class AbstractAnalysis<
         V extends AbstractValue<V>, S extends Store<S>, T extends TransferFunction<V, S>>
@@ -50,14 +50,14 @@ public abstract class AbstractAnalysis<
   /** The transfer function for regular nodes. */
   // TODO: make final. Currently, the transferFunction has a reference to the analysis, so it
   //  can't be created until the Analysis is initialized.
-  protected @Nullable T transferFunction;
+  protected @MonotonicNonNull T transferFunction;
 
   /** The current control flow graph to perform the analysis on. */
   protected @MonotonicNonNull ControlFlowGraph cfg;
 
   /**
-   * The transfer inputs of every basic block (assumed to be 'no information' if not present, inputs
-   * before blocks in forward analysis, after blocks in backward analysis).
+   * The transfer inputs of every basic block; assumed to be 'no information' if not present. The
+   * inputs are before blocks in forward analysis, and are after blocks in backward analysis.
    */
   protected final IdentityHashMap<Block, TransferInput<V, S>> inputs = new IdentityHashMap<>();
 
@@ -68,7 +68,7 @@ public abstract class AbstractAnalysis<
   protected final IdentityHashMap<Node, V> nodeValues = new IdentityHashMap<>();
 
   /** Map from (effectively final) local variable elements to their abstract value. */
-  protected final HashMap<Element, V> finalLocalValues = new HashMap<>();
+  protected final HashMap<VariableElement, V> finalLocalValues = new HashMap<>();
 
   /**
    * The node that is currently handled in the analysis (if it is running). The following invariant
@@ -161,6 +161,9 @@ public abstract class AbstractAnalysis<
     return this.direction;
   }
 
+  /** A cache for {@link #getResult()}. */
+  private @Nullable AnalysisResult<V, S> getResultCache;
+
   @Override
   @SuppressWarnings("nullness:contracts.precondition.override") // implementation field
   @RequiresNonNull("cfg")
@@ -169,8 +172,16 @@ public abstract class AbstractAnalysis<
       throw new BugInCF(
           "AbstractAnalysis::getResult() shouldn't be called when the analysis is running.");
     }
-    return new AnalysisResult<>(
-        nodeValues, inputs, cfg.getTreeLookup(), cfg.getPostfixNodeLookup(), finalLocalValues);
+    if (getResultCache == null) {
+      getResultCache =
+          new AnalysisResult<>(
+              nodeValues,
+              inputs,
+              cfg.getTreeLookup(),
+              cfg.getPostfixNodeLookup(),
+              finalLocalValues);
+    }
+    return getResultCache;
   }
 
   @Override
@@ -187,12 +198,11 @@ public abstract class AbstractAnalysis<
           || (currentTree != null && currentTree == n.getTree())) {
         return null;
       }
-      // check that 'n' is a subnode of 'node'. Check immediate operands
+      // check that 'n' is a subnode of 'currentNode'. Check immediate operands
       // first for efficiency.
       assert !n.isLValue() : "Did not expect an lvalue, but got " + n;
-      if (currentNode == n
-          || (!currentNode.getOperands().contains(n)
-              && !currentNode.getTransitiveOperands().contains(n))) {
+      if (!currentNode.getOperands().contains(n)
+          && !currentNode.getTransitiveOperands().contains(n)) {
         return null;
       }
       // fall through when the current node is not 'n', and 'n' is not a subnode.
@@ -305,12 +315,38 @@ public abstract class AbstractAnalysis<
    *
    * @param t the given tree
    * @return the contained method tree of the given tree
+   * @deprecated use {@link #getEnclosingMethod}
    */
+  @Deprecated // 2024-05-01
   public @Nullable MethodTree getContainingMethod(Tree t) {
+    return getEnclosingMethod(t);
+  }
+
+  /**
+   * Get the {@link MethodTree} of the current CFG if the argument {@link Tree} maps to a {@link
+   * Node} in the CFG or {@code null} otherwise.
+   *
+   * @param t the given tree
+   * @return the contained method tree of the given tree
+   */
+  public @Nullable MethodTree getEnclosingMethod(Tree t) {
     if (cfg == null) {
       return null;
     }
-    return cfg.getContainingMethod(t);
+    return cfg.getEnclosingMethod(t);
+  }
+
+  /**
+   * Get the {@link ClassTree} of the current CFG if the argument {@link Tree} maps to a {@link
+   * Node} in the CFG or {@code null} otherwise.
+   *
+   * @param t the given tree
+   * @return the contained class tree of the given tree
+   * @deprecated use {@link #getEnclosingClass}
+   */
+  @Deprecated // 2024-05-01
+  public @Nullable ClassTree getContainingClass(Tree t) {
+    return getEnclosingClass(t);
   }
 
   /**
@@ -320,11 +356,11 @@ public abstract class AbstractAnalysis<
    * @param t the given tree
    * @return the contained class tree of the given tree
    */
-  public @Nullable ClassTree getContainingClass(Tree t) {
+  public @Nullable ClassTree getEnclosingClass(Tree t) {
     if (cfg == null) {
       return null;
     }
-    return cfg.getContainingClass(t);
+    return cfg.getEnclosingClass(t);
   }
 
   /**
@@ -339,8 +375,9 @@ public abstract class AbstractAnalysis<
       Node node, TransferInput<V, S> transferInput) {
     assert transferFunction != null : "@AssumeAssertion(nullness): invariant";
     if (node.isLValue()) {
-      // TODO: should the default behavior return a regular transfer result, a conditional transfer
-      //  result (depending on store.containsTwoStores()), or is the following correct?
+      // TODO: should the default behavior return a regular transfer result, a conditional
+      // transfer result (depending on store.containsTwoStores()), or is the following
+      // correct?
       return new RegularTransferResult<>(null, transferInput.getRegularStore());
     }
     transferInput.node = node;
@@ -354,7 +391,7 @@ public abstract class AbstractAnalysis<
       Node lhst = assignment.getTarget();
       if (lhst instanceof LocalVariableNode) {
         LocalVariableNode lhs = (LocalVariableNode) lhst;
-        Element elem = lhs.getElement();
+        VariableElement elem = lhs.getElement();
         if (ElementUtils.isEffectivelyFinal(elem)) {
           V resval = transferResult.getResultValue();
           if (resval != null) {
@@ -402,11 +439,12 @@ public abstract class AbstractAnalysis<
     nodeValues.clear();
     finalLocalValues.clear();
     this.cfg = cfg;
+    getResultCache = null;
   }
 
   /**
-   * Updates the value of node {@code node} to the value of the {@code transferResult}. Returns true
-   * if the node's value changed, or a store was updated.
+   * Updates the value of node {@code node} in {@link #nodeValues} to the value of the {@code
+   * transferResult}. Returns true if the node's value changed, or a store was updated.
    *
    * @param node the node to update
    * @param transferResult the transfer result being updated
@@ -423,19 +461,6 @@ public abstract class AbstractAnalysis<
     return nodeValueChanged || transferResult.storeChanged();
   }
 
-  /**
-   * Read the store for a particular basic block from a map of stores (or {@code null} if none
-   * exists yet).
-   *
-   * @param stores a map of stores
-   * @param b the target block
-   * @param <S> method return type should be a subtype of {@link Store}
-   * @return the store for the target block
-   */
-  protected static <S> @Nullable S readFromStore(Map<Block, S> stores, Block b) {
-    return stores.get(b);
-  }
-
   /**
    * Add a basic block to {@link #worklist}. If {@code b} is already present, the method does
    * nothing.
@@ -443,7 +468,11 @@ public abstract class AbstractAnalysis<
    * @param b the block to add to {@link #worklist}
    */
   protected void addToWorklist(Block b) {
-    // TODO: use a more efficient way to check if b is already present
+    // TODO: This costs linear (!) time.  Use a more efficient way to check if b is already
+    // present.
+    // Two possibilities:
+    //  * add unconditionally, and detect duplicates when removing from the queue.
+    //  * maintain a HashSet of the elements that are already in the queue.
     if (!worklist.contains(b)) {
       worklist.add(b);
     }
@@ -462,7 +491,10 @@ public abstract class AbstractAnalysis<
      * Comparators to allow priority queue to order blocks by their depth-first order, using by
      * forward analysis.
      */
-    public class ForwardDFOComparator implements Comparator<Block> {
+    public class ForwardDfoComparator implements Comparator<Block> {
+      /** Creates a new ForwardDfoComparator. */
+      public ForwardDfoComparator() {}
+
       @SuppressWarnings("nullness:unboxing.of.nullable")
       @Override
       public int compare(Block b1, Block b2) {
@@ -474,7 +506,10 @@ public abstract class AbstractAnalysis<
      * Comparators to allow priority queue to order blocks by their depth-first order, using by
      * backward analysis.
      */
-    public class BackwardDFOComparator implements Comparator<Block> {
+    public class BackwardDfoComparator implements Comparator<Block> {
+      /** Creates a new BackwardDfoComparator. */
+      public BackwardDfoComparator() {}
+
       @SuppressWarnings("nullness:unboxing.of.nullable")
       @Override
       public int compare(Block b1, Block b2) {
@@ -485,6 +520,9 @@ public abstract class AbstractAnalysis<
     /** The backing priority queue. */
     protected final PriorityQueue<Block> queue;
 
+    /** Contains the same elements as {@link #queue}, for faster lookup. */
+    protected final Set<Block> queueSet;
+
     /**
      * Create a Worklist.
      *
@@ -492,16 +530,21 @@ public abstract class AbstractAnalysis<
      */
     public Worklist(Direction direction) {
       if (direction == Direction.FORWARD) {
-        queue = new PriorityQueue<>(new ForwardDFOComparator());
+        queue = new PriorityQueue<>(new ForwardDfoComparator());
+        queueSet = new HashSet<>();
       } else if (direction == Direction.BACKWARD) {
-        queue = new PriorityQueue<>(new BackwardDFOComparator());
+        queue = new PriorityQueue<>(new BackwardDfoComparator());
+        queueSet = new HashSet<>();
       } else {
-        throw new BugInCF("Unexpected Direction meet: " + direction.name());
+        throw new BugInCF("Unexpected Direction: " + direction.name());
       }
     }
 
     /**
-     * Process the control flow graph, add the blocks to {@link #depthFirstOrder}.
+     * Process the control flow graph.
+     *
+     * <p>This implementation sets the depth-first order for each block, by adding the blocks to
+     * {@link #depthFirstOrder}.
      *
      * @param cfg the control flow graph to process
      */
@@ -513,6 +556,7 @@ public abstract class AbstractAnalysis<
       }
 
       queue.clear();
+      queueSet.clear();
     }
 
     /**
@@ -525,6 +569,7 @@ public abstract class AbstractAnalysis<
     @EnsuresNonNullIf(result = false, expression = "poll()")
     @SuppressWarnings("nullness:contracts.conditional.postcondition") // forwarded
     public boolean isEmpty() {
+      assert queue.isEmpty() == queueSet.isEmpty();
       return queue.isEmpty();
     }
 
@@ -535,7 +580,7 @@ public abstract class AbstractAnalysis<
      * @return true if {@link #queue} contains the given block
      */
     public boolean contains(Block block) {
-      return queue.contains(block);
+      return queueSet.contains(block);
     }
 
     /**
@@ -546,6 +591,7 @@ public abstract class AbstractAnalysis<
      */
     public void add(Block block) {
       queue.add(block);
+      queueSet.add(block);
     }
 
     /**
@@ -556,7 +602,11 @@ public abstract class AbstractAnalysis<
      */
     @Pure
     public @Nullable Block poll() {
-      return queue.poll();
+      Block result = queue.poll();
+      if (result != null) {
+        queueSet.remove(result);
+      }
+      return result;
     }
 
     @Override
