@@ -10,6 +10,7 @@ import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.NewArrayTree;
 import com.sun.source.tree.NewClassTree;
+import com.sun.source.tree.Tree;
 import com.sun.source.tree.UnaryTree;
 import com.sun.source.tree.VariableTree;
 import com.sun.source.util.TreePath;
@@ -43,6 +44,7 @@ import org.checkerframework.dataflow.cfg.node.ThisNode;
 import org.checkerframework.dataflow.cfg.node.UnaryOperationNode;
 import org.checkerframework.dataflow.cfg.node.ValueLiteralNode;
 import org.checkerframework.dataflow.cfg.node.WideningConversionNode;
+import org.checkerframework.dataflow.qual.Pure;
 import org.checkerframework.javacutil.AnnotationProvider;
 import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.ElementUtils;
@@ -88,18 +90,64 @@ public abstract class JavaExpression {
     return type;
   }
 
-  public abstract boolean containsOfClass(Class<? extends JavaExpression> clazz);
+  /**
+   * Returns true if some subexpression is of given class.
+   *
+   * <p>If you want to debug and determine <em>which</em> subexpression is of the given class, use
+   * {@link #containedOfClass}.
+   *
+   * @param clazz the JavaExpression subclass to search for
+   * @return true if some subexpression's class is the given class
+   */
+  @Pure
+  public final boolean containsOfClass(Class<? extends JavaExpression> clazz) {
+    return containedOfClass(clazz) != null;
+  }
 
-  public boolean containsUnknown() {
+  /**
+   * Returns the first subexpression whose class is the given class, or null.
+   *
+   * <p>This is intended as a diagnostic aid; most clients will use {@link #containsOfClass}.
+   *
+   * @param <T> the type corresponding to {@code clazz}
+   * @param clazz the JavaExpression subclass to search for
+   * @return true if some subexpression whose class is the given class
+   */
+  @Pure
+  public abstract <T extends JavaExpression> @Nullable T containedOfClass(Class<T> clazz);
+
+  /**
+   * Returns true if some subexpression is {@link Unknown}.
+   *
+   * <p>If you want to debug and determine <em>which</em> subexpression is of the given class, use
+   * {@link #containedUnknown}.
+   *
+   * @return true if some subexpression is {@link Unknown}
+   */
+  @Pure
+  public final boolean containsUnknown() {
     return containsOfClass(Unknown.class);
   }
 
+  /**
+   * Returns the first subexpression whose class is {@link Unknown}, or null.
+   *
+   * <p>This is intended as a diagnostic aid; most clients will use {@link #containsUnknown}.
+   *
+   * @return the first subexpression whose class is {@link Unknown}, or null
+   */
+  @Pure
+  public final @Nullable Unknown containedUnknown() {
+    return containedOfClass(Unknown.class);
+  }
+
   /**
    * Returns true if the expression is deterministic.
    *
    * @param provider an annotation provider (a type factory)
    * @return true if this expression is deterministic
    */
+  @Pure
   public abstract boolean isDeterministic(AnnotationProvider provider);
 
   /**
@@ -109,6 +157,7 @@ public abstract class JavaExpression {
    * @param provider an annotation provider (a type factory)
    * @return true if all the expressions in the list are deterministic
    */
+  @Pure
   public static boolean listIsDeterministic(
       List<? extends @Nullable JavaExpression> list, AnnotationProvider provider) {
     return list.stream().allMatch(je -> je == null || je.isDeterministic(provider));
@@ -120,9 +169,34 @@ public abstract class JavaExpression {
    * final field accesses whose receiver is {@link #isUnassignableByOtherCode}, and operations whose
    * operands are all {@link #isUnmodifiableByOtherCode}.
    *
+   * @return true if no subexpression of this can be assigned to from outside the current method
+   *     body
    * @see #isUnmodifiableByOtherCode
+   * @deprecated use {@link #isAssignableByOtherCode}
    */
-  public abstract boolean isUnassignableByOtherCode();
+  @Deprecated // 2024-04-30
+  @Pure
+  public boolean isUnassignableByOtherCode() {
+    return !isAssignableByOtherCode();
+  }
+
+  /**
+   * Returns true if some subexpression of this can be assigned to from outside the current method
+   * body.
+   *
+   * <p>This is false for local variables, the self reference, final field accesses whose receiver
+   * is {@link #isUnassignableByOtherCode}, and operations whose operands are all not {@link
+   * #isModifiableByOtherCode}.
+   *
+   * @return true if some subexpression of this can be assigned to from outside the current method
+   *     body
+   * @see #isModifiableByOtherCode
+   */
+  // TODO: Make abstract when isUnassignableByOtherCode is removed.
+  @Pure
+  public boolean isAssignableByOtherCode() {
+    return !isUnassignableByOtherCode();
+  }
 
   /**
    * Returns true if and only if the value this expression stands for cannot be changed by a method
@@ -131,9 +205,34 @@ public abstract class JavaExpression {
    * <p>Approximately, this returns true if the expression is {@link #isUnassignableByOtherCode} and
    * its type is immutable.
    *
+   * @return true if the value of this expression cannot be changed from outside the current method
+   *     body
+   * @see #isUnassignableByOtherCode
+   * @deprecated use {@link #isModifiableByOtherCode}
+   */
+  @Deprecated // 2024-04-30
+  @Pure
+  public boolean isUnmodifiableByOtherCode() {
+    return !isModifiableByOtherCode();
+  }
+
+  /**
+   * Returns true if the value this expression stands for can be changed by a method call;
+   * equivalently, if the value this expression evaluates to can be changed by a side effect from
+   * outside the containing method.
+   *
+   * <p>Approximately, this returns true if the expression is {@link #isAssignableByOtherCode} or
+   * its type is mutable. ({@code String} is an immutable type.)
+   *
+   * @return true if the value of this expression can be changed from outside the current method
+   *     body
    * @see #isUnassignableByOtherCode
    */
-  public abstract boolean isUnmodifiableByOtherCode();
+  // TODO: Make abstract when isUnmodifiableByOtherCode is removed.
+  @Pure
+  public boolean isModifiableByOtherCode() {
+    return !isUnmodifiableByOtherCode();
+  }
 
   /**
    * Returns true if and only if the two Java expressions are syntactically identical.
@@ -144,6 +243,7 @@ public abstract class JavaExpression {
    * @return true if and only if the two Java expressions are syntactically identical
    */
   @EqualsMethod
+  @Pure
   public abstract boolean syntacticEquals(JavaExpression je);
 
   /**
@@ -153,7 +253,9 @@ public abstract class JavaExpression {
    * @param lst2 the second list to compare
    * @return true if the corresponding list elements satisfy {@link #syntacticEquals}
    */
-  static boolean syntacticEqualsList(
+  @SuppressWarnings("RedundantControlFlow")
+  @Pure
+  public static boolean syntacticEqualsList(
       List<? extends @Nullable JavaExpression> lst1,
       List<? extends @Nullable JavaExpression> lst2) {
     if (lst1.size() != lst2.size()) {
@@ -183,6 +285,7 @@ public abstract class JavaExpression {
    * @return true if and only if this contains a JavaExpression that is syntactically equal to
    *     {@code other}
    */
+  @Pure
   public abstract boolean containsSyntacticEqualJavaExpression(JavaExpression other);
 
   /**
@@ -194,6 +297,7 @@ public abstract class JavaExpression {
    * @return true if and only if the list contains a JavaExpression that is syntactically equal to
    *     {@code other}
    */
+  @Pure
   public static boolean listContainsSyntacticEqualJavaExpression(
       List<? extends @Nullable JavaExpression> list, JavaExpression other) {
     return list.stream()
@@ -207,6 +311,7 @@ public abstract class JavaExpression {
    * <p>This is always true, except for cases where the Java type information prevents aliasing and
    * none of the subexpressions can alias 'other'.
    */
+  @Pure
   public boolean containsModifiableAliasOf(Store<?> store, JavaExpression other) {
     return this.equals(other) || store.canAlias(this, other);
   }
@@ -216,13 +321,14 @@ public abstract class JavaExpression {
    *
    * @return a verbose string representation of this
    */
+  @Pure
   public String toStringDebug() {
     return String.format("%s(%s): %s", getClass().getSimpleName(), type, toString());
   }
 
-  ///
-  /// Static methods
-  ///
+  //
+  // Static methods
+  //
 
   /**
    * Returns the Java expression for a {@link FieldAccessNode}. The result may contain {@link
@@ -235,7 +341,8 @@ public abstract class JavaExpression {
     Node receiverNode = node.getReceiver();
     String fieldName = node.getFieldName();
     if (fieldName.equals("this")) {
-      // The CFG represents "className.this" as a FieldAccessNode, but it isn't a field access.
+      // The CFG represents "className.this" as a FieldAccessNode, but it isn't a field
+      // access.
       return new ThisReference(receiverNode.getType());
     } else if (fieldName.equals("class")) {
       // The CFG represents "className.class" as a FieldAccessNode; bit it is a class literal.
@@ -279,7 +386,7 @@ public abstract class JavaExpression {
     } else if (receiverNode instanceof ThisNode) {
       result = new ThisReference(receiverNode.getType());
     } else if (receiverNode instanceof SuperNode) {
-      result = new ThisReference(receiverNode.getType());
+      result = new SuperReference(receiverNode.getType());
     } else if (receiverNode instanceof LocalVariableNode) {
       LocalVariableNode lv = (LocalVariableNode) receiverNode;
       result = new LocalVariable(lv);
@@ -407,8 +514,8 @@ public abstract class JavaExpression {
             CollectionsPlume.mapList(JavaExpression::fromTree, mn.getArguments());
         JavaExpression methodReceiver;
         if (ElementUtils.isStatic(invokedMethod)) {
-          @SuppressWarnings(
-              "nullness:assignment" // enclosingTypeElement(ExecutableElement): @NonNull
+          @SuppressWarnings("nullness:assignment" // enclosingTypeElement(ExecutableElement):
+          // @NonNull
           )
           @NonNull TypeElement methodType = ElementUtils.enclosingTypeElement(invokedMethod);
           methodReceiver = new ClassName(methodType.asType());
@@ -427,17 +534,22 @@ public abstract class JavaExpression {
         IdentifierTree identifierTree = (IdentifierTree) tree;
         TypeMirror typeOfId = TreeUtils.typeOf(identifierTree);
         Name identifierName = identifierTree.getName();
-        if (identifierName.contentEquals("this") || identifierName.contentEquals("super")) {
+        if (identifierName.contentEquals("this")) {
           result = new ThisReference(typeOfId);
           break;
+        } else if (identifierName.contentEquals("super")) {
+          result = new SuperReference(typeOfId);
+          break;
         }
         assert TreeUtils.isUseOfElement(identifierTree) : "@AssumeAssertion(nullness): tree kind";
         Element ele = TreeUtils.elementFromUse(identifierTree);
-        if (ElementUtils.isTypeElement(ele)) {
+        if (ele == null) {
+          result = null;
+        } else if (ElementUtils.isTypeElement(ele)) {
           result = new ClassName(ele.asType());
-          break;
+        } else {
+          result = fromVariableElement(typeOfId, (VariableElement) ele, identifierTree);
         }
-        result = fromVariableElement(typeOfId, ele);
         break;
 
       case UNARY_PLUS:
@@ -492,7 +604,8 @@ public abstract class JavaExpression {
    * @return a JavaExpression for {@code tree}
    */
   public static JavaExpression fromVariableTree(VariableTree tree) {
-    return fromVariableElement(TreeUtils.typeOf(tree), TreeUtils.elementFromDeclaration(tree));
+    return fromVariableElement(
+        TreeUtils.typeOf(tree), TreeUtils.elementFromDeclaration(tree), tree);
   }
 
   /**
@@ -500,9 +613,14 @@ public abstract class JavaExpression {
    *
    * @param typeOfEle the type of {@code ele}
    * @param ele element whose JavaExpression is returned
+   * @param tree the tree for the variable
    * @return the Java expression corresponding to the given variable element {@code ele}
    */
-  private static JavaExpression fromVariableElement(TypeMirror typeOfEle, Element ele) {
+  private static JavaExpression fromVariableElement(
+      TypeMirror typeOfEle, @Nullable VariableElement ele, Tree tree) {
+    if (ele == null) {
+      return new Unknown(tree);
+    }
     switch (ele.getKind()) {
       case LOCAL_VARIABLE:
       case RESOURCE_VARIABLE:
@@ -520,7 +638,7 @@ public abstract class JavaExpression {
         } else {
           fieldAccessExpression = new ThisReference(enclosingTypeElement);
         }
-        return new FieldAccess(fieldAccessExpression, typeOfEle, (VariableElement) ele);
+        return new FieldAccess(fieldAccessExpression, typeOfEle, ele);
       default:
         if (ElementUtils.isBindingVariable(ele)) {
           return new LocalVariable(ele);
@@ -595,14 +713,14 @@ public abstract class JavaExpression {
     return parameters;
   }
 
-  ///
-  /// Obtaining the receiver
-  ///
+  //
+  // Obtaining the receiver
+  //
 
   /**
-   * Returns the receiver of the given invocation
+   * Returns the receiver of the given invocation.
    *
-   * @param accessTree method or constructor invocation
+   * @param accessTree a method or constructor invocation
    * @return the receiver of the given invocation
    */
   public static JavaExpression getReceiver(ExpressionTree accessTree) {
@@ -626,6 +744,9 @@ public abstract class JavaExpression {
    * <p>Returns either a new ClassName or a new ThisReference depending on whether ele is static or
    * not. The passed element must be a field, method, or class.
    *
+   * <p>When this returns a ThisReference, its type is the class that declares {@code ele}, which is
+   * not necessarily the type of {@code this} at the invocation site.
+   *
    * @param ele a field, method, or class
    * @return either a new ClassName or a new ThisReference depending on whether ele is static or not
    */
@@ -648,9 +769,10 @@ public abstract class JavaExpression {
    * <p>The Tree should be an expression or a statement that does not have a receiver or an implicit
    * receiver. For example, a local variable declaration.
    *
-   * @param path TreePath to tree
+   * @param path a tree path
    * @param enclosingType type of the enclosing type
-   * @return a new ClassName or ThisReference that is a JavaExpression object for the enclosingType
+   * @return a new {@link ClassName} or {@link ThisReference} that is a JavaExpression object for
+   *     the enclosingType
    */
   public static JavaExpression getPseudoReceiver(TreePath path, TypeMirror enclosingType) {
     if (TreePathUtil.isTreeInStaticScope(path)) {
@@ -689,6 +811,7 @@ public abstract class JavaExpression {
    * @return viewpoint-adapted version of this
    */
   public final JavaExpression atMethodBody(MethodTree methodTree) {
+    @SuppressWarnings("nullness:argument") // elementFromDeclaration is non-null for a parameter
     List<JavaExpression> parametersJe =
         CollectionsPlume.mapList(
             (VariableTree param) -> new LocalVariable(TreeUtils.elementFromDeclaration(param)),
@@ -746,7 +869,7 @@ public abstract class JavaExpression {
    */
   private static List<JavaExpression> argumentTreesToJavaExpressions(
       ExecutableElement method, List<? extends ExpressionTree> argTrees) {
-    if (isVarArgsInvocation(method, argTrees)) {
+    if (isVarargsInvocation(method, argTrees)) {
       List<JavaExpression> result = new ArrayList<>(method.getParameters().size());
       for (int i = 0; i < method.getParameters().size() - 1; i++) {
         result.add(JavaExpression.fromTree(argTrees.get(i)));
@@ -775,7 +898,7 @@ public abstract class JavaExpression {
    * @param args the arguments at the call site
    * @return true if method is a varargs method and its varargs arguments are not passed in an array
    */
-  private static boolean isVarArgsInvocation(
+  private static boolean isVarargsInvocation(
       ExecutableElement method, List<? extends ExpressionTree> args) {
     if (!method.isVarArgs()) {
       return false;
