@@ -14,7 +14,6 @@ import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.nullness.qual.RequiresNonNull;
 import org.checkerframework.checker.signature.qual.BinaryName;
 import org.checkerframework.javacutil.BugInCF;
-import org.checkerframework.javacutil.SystemUtil;
 import org.plumelib.util.StringsPlume;
 
 /**
@@ -28,9 +27,10 @@ import org.plumelib.util.StringsPlume;
  */
 public class TestConfigurationBuilder {
 
-  // Presented first are static helper methods that reduce configuration building to a method call
+  // Presented first are static helper methods that reduce configuration building to a method
+  // call.
   // However, if you need more complex configuration or custom configuration, use the
-  // constructors provided below
+  // constructors provided below.
 
   /**
    * This creates a builder for the default configuration used by Checker Framework JUnit tests.
@@ -61,17 +61,12 @@ public class TestConfigurationBuilder {
             .addOption("-Xmaxerrs", "9999")
             .addOption("-g")
             .addOption("-Xlint:unchecked")
-            .addOption("-XDrawDiagnostics") // use short javac diagnostics
             .addSourceFiles(testSourceFiles);
 
     if (outputClassDirectory != null) {
       configBuilder.addOption("-d", outputClassDirectory.getAbsolutePath());
     }
 
-    if (SystemUtil.getJreVersion() == 8) {
-      configBuilder.addOption("-source", "8").addOption("-target", "8");
-    }
-
     configBuilder
         .addOptionIfValueNonEmpty("-sourcepath", testSourcePath)
         .addOption("-implicit:class")
@@ -94,7 +89,7 @@ public class TestConfigurationBuilder {
    *     compiler, and file manager used by Checker Framework tests
    */
   @SuppressWarnings(
-      "signature:argument" // for non-array non-primitive class, getName(): @BinaryName
+      "signature:cast.unsafe" // for non-array non-primitive class, getName(): @BinaryName
   )
   public static TestConfiguration buildDefaultConfiguration(
       String testSourcePath,
@@ -106,7 +101,7 @@ public class TestConfigurationBuilder {
         testSourcePath,
         Arrays.asList(testFile),
         Collections.emptyList(),
-        Arrays.asList(processor.getName()),
+        Arrays.asList((@BinaryName String) processor.getName()),
         options,
         shouldEmitDebugInfo);
   }
@@ -187,10 +182,10 @@ public class TestConfigurationBuilder {
   private List<File> testSourceFiles;
 
   /** The set of Checker Framework processors to test with. */
-  private Set<@BinaryName String> processors;
+  private final Set<@BinaryName String> processors;
 
   /** The set of options to the Javac command line used to run the test. */
-  private SimpleOptionMap options;
+  private final SimpleOptionMap options;
 
   /** Should the Javac options be output before running the test. */
   private boolean shouldEmitDebugInfo;
@@ -246,7 +241,7 @@ public class TestConfigurationBuilder {
       errors.add("No processors were specified!");
     }
 
-    final Map<String, @Nullable String> optionMap = options.getOptions();
+    Map<String, @Nullable String> optionMap = options.getOptions();
     if (!optionMap.containsKey("-d") || optionMap.get("-d") == null) {
       errors.add("No output directory was specified.");
     }
@@ -258,56 +253,125 @@ public class TestConfigurationBuilder {
     return errors;
   }
 
+  /**
+   * Adds the given path option to {@code this}.
+   *
+   * @param key the key to add
+   * @param toAppend the path to append
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder adddToPathOption(String key, String toAppend) {
-    options.addToPathOption(key, toAppend);
+    this.options.addToPathOption(key, toAppend);
     return this;
   }
 
+  /**
+   * Adds the given diagnostics file to {@code this}.
+   *
+   * @param diagnostics the diagnostics file to add to {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addDiagnosticFile(File diagnostics) {
     this.diagnosticFiles.add(diagnostics);
     return this;
   }
 
+  /**
+   * Adds the given diagnostics files to {@code this}.
+   *
+   * @param diagnostics diagnostics files to add to {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addDiagnosticFiles(Iterable<File> diagnostics) {
     this.diagnosticFiles = catListAndIterable(diagnosticFiles, diagnostics);
     return this;
   }
 
+  /**
+   * Sets the diagnostics files of {@code this}.
+   *
+   * @param diagnosticFiles diagnostics files to set on {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder setDiagnosticFiles(List<File> diagnosticFiles) {
     this.diagnosticFiles = new ArrayList<>(diagnosticFiles);
     return this;
   }
 
+  /**
+   * Adds the given source file to {@code this}.
+   *
+   * @param sourceFile source file to add to {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addSourceFile(File sourceFile) {
-    testSourceFiles.add(sourceFile);
+    this.testSourceFiles.add(sourceFile);
     return this;
   }
 
+  /**
+   * Adds the given source files to {@code this}.
+   *
+   * @param sourceFiles source files to add to {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addSourceFiles(Iterable<File> sourceFiles) {
-    testSourceFiles = catListAndIterable(testSourceFiles, sourceFiles);
+    this.testSourceFiles = catListAndIterable(testSourceFiles, sourceFiles);
     return this;
   }
 
+  /**
+   * Sets the source files of {@code this}.
+   *
+   * @param sourceFiles source files to set on {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder setSourceFiles(List<File> sourceFiles) {
     this.testSourceFiles = new ArrayList<>(sourceFiles);
     return this;
   }
 
+  /**
+   * Sets the given options on {@code this}.
+   *
+   * @param options options to set on {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder setOptions(Map<String, @Nullable String> options) {
     this.options.setOptions(options);
     return this;
   }
 
+  /**
+   * Adds the given option to {@code this}.
+   *
+   * @param option option to add to {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addOption(String option) {
     this.options.addOption(option);
     return this;
   }
 
+  /**
+   * Adds the given option and value to {@code this}.
+   *
+   * @param option option to add to {@code this}
+   * @param value value to add
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addOption(String option, String value) {
     this.options.addOption(option, value);
     return this;
   }
 
+  /**
+   * Adds the given option to {@code this} if the value is non-empty.
+   *
+   * @param option option to add to {@code this}
+   * @param value value to add, iff it is non-empty
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addOptionIfValueNonEmpty(String option, String value) {
     if (value != null && !value.isEmpty()) {
       return addOption(option, value);
@@ -331,6 +395,12 @@ public class TestConfigurationBuilder {
     return this;
   }
 
+  /**
+   * Adds the given options to {@code this}.
+   *
+   * @param newOptions options to add to {@code this}
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder addOptions(Iterable<String> newOptions) {
     this.options.addOptions(newOptions);
     return this;
@@ -375,16 +445,32 @@ public class TestConfigurationBuilder {
     return this;
   }
 
+  /**
+   * Sets {@code this} to output debug info.
+   *
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder emitDebugInfo() {
     this.shouldEmitDebugInfo = true;
     return this;
   }
 
+  /**
+   * Sets {@code this} to not output debug info.
+   *
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder dontEmitDebugInfo() {
     this.shouldEmitDebugInfo = false;
     return this;
   }
 
+  /**
+   * Sets {@code this} to output debug info depending on the parameter.
+   *
+   * @param shouldEmitDebugInfo whether to emit debug info
+   * @return the current object {@code this}
+   */
   public TestConfigurationBuilder setShouldEmitDebugInfo(boolean shouldEmitDebugInfo) {
     this.shouldEmitDebugInfo = shouldEmitDebugInfo;
     return this;
@@ -452,8 +538,8 @@ public class TestConfigurationBuilder {
    * @return a list that first has the items from parameter list then the items from iterable
    */
   private static <T> List<T> catListAndIterable(
-      final List<? extends T> list, final Iterable<? extends T> iterable) {
-    final List<T> newList = new ArrayList<>(list);
+      List<? extends T> list, Iterable<? extends T> iterable) {
+    List<T> newList = new ArrayList<>(list);
 
     for (T iterObject : iterable) {
       newList.add(iterObject);
@@ -462,8 +548,14 @@ public class TestConfigurationBuilder {
     return newList;
   }
 
+  /** The output directory for tests. */
   public static final String TESTS_OUTPUTDIR = "tests.outputDir";
 
+  /**
+   * Determine the output directory from the {@code tests.outputDir} property.
+   *
+   * @return the output directory
+   */
   public static File getOutputDirFromProperty() {
     return new File(
         System.getProperty(
@@ -471,6 +563,11 @@ public class TestConfigurationBuilder {
             "tests" + File.separator + "build" + File.separator + "testclasses"));
   }
 
+  /**
+   * Determine the default classpath from the {@code tests.classpath} property.
+   *
+   * @return the default classpath
+   */
   public static String getDefaultClassPath() {
     String classpath = System.getProperty("tests.classpath", "tests" + File.separator + "build");
     String globalclasspath = System.getProperty("java.class.path", "");
