@@ -7,11 +7,13 @@ import com.github.javaparser.ast.expr.Expression;
 import com.github.javaparser.ast.expr.UnaryExpr;
 import com.sun.source.tree.ExpressionTree;
 import com.sun.source.tree.MethodInvocationTree;
+import com.sun.source.tree.Tree;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Method;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
+import java.util.Set;
 import java.util.StringJoiner;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.ExecutableElement;
@@ -19,7 +21,6 @@ import javax.lang.model.util.Elements;
 import org.checkerframework.checker.nullness.qual.MonotonicNonNull;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.common.accumulation.AccumulationChecker.AliasAnalysis;
-import org.checkerframework.common.basetype.BaseAnnotatedTypeFactory;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.common.returnsreceiver.ReturnsReceiverAnnotatedTypeFactory;
 import org.checkerframework.common.returnsreceiver.ReturnsReceiverChecker;
@@ -27,6 +28,7 @@ import org.checkerframework.common.returnsreceiver.qual.This;
 import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutableType;
 import org.checkerframework.framework.type.ElementQualifierHierarchy;
+import org.checkerframework.framework.type.GenericAnnotatedTypeFactory;
 import org.checkerframework.framework.type.QualifierHierarchy;
 import org.checkerframework.framework.type.treeannotator.ListTreeAnnotator;
 import org.checkerframework.framework.type.treeannotator.TreeAnnotator;
@@ -45,7 +47,9 @@ import org.plumelib.util.CollectionsPlume;
  * take a {@link BaseTypeChecker} and call both the constructor defined in this class and {@link
  * #postInit()}.
  */
-public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedTypeFactory {
+public abstract class AccumulationAnnotatedTypeFactory
+    extends GenericAnnotatedTypeFactory<
+        AccumulationValue, AccumulationStore, AccumulationTransfer, AccumulationAnalysis> {
 
   /** The typechecker associated with this factory. */
   public final AccumulationChecker accumulationChecker;
@@ -198,7 +202,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
    */
   public AnnotationMirror createAccumulatorAnnotation(List<String> values) {
     AnnotationBuilder builder = new AnnotationBuilder(processingEnv, accumulator);
-    builder.setValue("value", CollectionsPlume.withoutDuplicates(values));
+    builder.setValue("value", CollectionsPlume.withoutDuplicatesSorted(values));
     return builder.build();
   }
 
@@ -222,7 +226,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
    * @param tree a method invocation tree
    * @return true if the method being invoked returns its receiver
    */
-  public boolean returnsThis(final MethodInvocationTree tree) {
+  public boolean returnsThis(MethodInvocationTree tree) {
     if (!accumulationChecker.isEnabled(AliasAnalysis.RETURNS_RECEIVER)) {
       return false;
     }
@@ -232,7 +236,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
     ExecutableElement methodEle = TreeUtils.elementFromUse(tree);
     AnnotatedExecutableType methodAtm = rrATF.getAnnotatedType(methodEle);
     AnnotatedTypeMirror rrType = methodAtm.getReturnType();
-    return rrType != null && rrType.hasAnnotation(This.class);
+    return rrType != null && rrType.hasPrimaryAnnotation(This.class);
   }
 
   /**
@@ -288,14 +292,23 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
       if (returnsThis(tree)) {
         // There is a @This annotation on the return type of the invoked method.
         ExpressionTree receiverTree = TreeUtils.getReceiverTree(tree.getMethodSelect());
-        AnnotatedTypeMirror receiverType =
-            receiverTree == null ? null : getAnnotatedType(receiverTree);
-        // The current type of the receiver, or top if none exists.
-        AnnotationMirror receiverAnno =
-            receiverType == null ? top : receiverType.getAnnotationInHierarchy(top);
+        AnnotationMirror returnAnno = type.getPrimaryAnnotationInHierarchy(top);
+        AnnotationMirror glbAnno;
+        if (receiverTree == null) {
+          glbAnno = returnAnno;
+        } else {
+          AnnotatedTypeMirror receiverType = getAnnotatedType(receiverTree);
+          // The current type of the receiver, or top if none exists.
+          AnnotationMirror receiverAnno = receiverType.getPrimaryAnnotationInHierarchy(top);
+          glbAnno =
+              qualHierarchy.greatestLowerBoundShallow(
+                  returnAnno,
+                  type.getUnderlyingType(),
+                  receiverAnno,
+                  receiverType.getUnderlyingType());
+        }
 
-        AnnotationMirror returnAnno = type.getAnnotationInHierarchy(top);
-        type.replaceAnnotation(qualHierarchy.greatestLowerBound(returnAnno, receiverAnno));
+        type.replaceAnnotation(glbAnno);
       }
       return super.visitMethodInvocation(tree, type);
     }
@@ -326,6 +339,37 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
     }
   }
 
+  /**
+   * Returns the accumulated values on the given (expression, usually) tree. This differs from
+   * calling {@link #getAnnotatedType(Tree)}, because this version takes into account accumulated
+   * methods that are stored on the value. This is useful when dealing with accumulated facts on
+   * variables whose types are type variables (because type variable types cannot be refined
+   * directly, due to the quirks of subtyping between type variables and its interactions with the
+   * qualified type system).
+   *
+   * <p>The returned collection may be either a list or a set.
+   *
+   * @param tree a tree
+   * @return the accumulated values for the given tree, including those stored on the value
+   */
+  public Collection<String> getAccumulatedValues(Tree tree) {
+    AnnotatedTypeMirror type = getAnnotatedType(tree);
+    AnnotationMirror anno = type.getPrimaryAnnotationInHierarchy(top);
+    if (anno != null && isAccumulatorAnnotation(anno)) {
+      return getAccumulatedValues(anno);
+    } else if (anno == null) {
+      // Handle type variables and wildcards.
+      AccumulationValue inferredValue = getInferredValueFor(tree);
+      if (inferredValue != null) {
+        Set<String> accumulatedValues = inferredValue.getAccumulatedValues();
+        if (accumulatedValues != null) {
+          return accumulatedValues;
+        }
+      }
+    }
+    return Collections.emptyList();
+  }
+
   /**
    * All accumulation analyses share a similar type hierarchy. This class implements the subtyping,
    * LUB, and GLB for that hierarchy. The lattice looks like:
@@ -369,7 +413,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
      */
     protected AccumulationQualifierHierarchy(
         Collection<Class<? extends Annotation>> qualifierClasses, Elements elements) {
-      super(qualifierClasses, elements);
+      super(qualifierClasses, elements, AccumulationAnnotatedTypeFactory.this);
     }
 
     /**
@@ -377,8 +421,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
      * them is bottom, in which case the result is also bottom.
      */
     @Override
-    public AnnotationMirror greatestLowerBound(
-        final AnnotationMirror a1, final AnnotationMirror a2) {
+    public AnnotationMirror greatestLowerBoundQualifiers(AnnotationMirror a1, AnnotationMirror a2) {
       if (AnnotationUtils.areSame(a1, bottom) || AnnotationUtils.areSame(a2, bottom)) {
         return bottom;
       }
@@ -421,7 +464,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
      * one of them is bottom, in which case the result is the other annotation.
      */
     @Override
-    public AnnotationMirror leastUpperBound(final AnnotationMirror a1, final AnnotationMirror a2) {
+    public AnnotationMirror leastUpperBoundQualifiers(AnnotationMirror a1, AnnotationMirror a2) {
       if (AnnotationUtils.areSame(a1, bottom)) {
         return a2;
       } else if (AnnotationUtils.areSame(a2, bottom)) {
@@ -461,9 +504,13 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
       return createAccumulatorAnnotation(a1Val);
     }
 
-    /** isSubtype in this type system is subset. */
+    /**
+     * {@inheritDoc}
+     *
+     * <p>isSubtype in this type system is subset.
+     */
     @Override
-    public boolean isSubtype(final AnnotationMirror subAnno, final AnnotationMirror superAnno) {
+    public boolean isSubtypeQualifiers(AnnotationMirror subAnno, AnnotationMirror superAnno) {
       if (AnnotationUtils.areSame(subAnno, bottom)) {
         return true;
       } else if (AnnotationUtils.areSame(superAnno, bottom)) {
@@ -474,8 +521,9 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
         if (isPolymorphicQualifier(superAnno)) {
           return true;
         } else {
-          // Use this slightly more expensive conversion here because this is a rare code path and
-          // it's simpler to read than checking for both predicate and non-predicate forms of top.
+          // Use this slightly more expensive conversion here because this is a rare code
+          // path and it's simpler to read than checking for both predicate and
+          // non-predicate forms of top.
           return "".equals(convertToPredicate(superAnno));
         }
       } else if (isPolymorphicQualifier(superAnno)) {
@@ -541,8 +589,7 @@ public abstract class AccumulationAnnotatedTypeFactory extends BaseAnnotatedType
    * @return null if there is nothing wrong with the annotation, or an error message indicating the
    *     problem if it has an invalid predicate
    */
-  /* package-private */
-  @Nullable String isValidPredicate(AnnotationMirror anm) {
+  /*package-private*/ @Nullable String isValidPredicate(AnnotationMirror anm) {
     String pred = convertToPredicate(anm);
     try {
       evaluatePredicate(Collections.emptyList(), pred);
