@@ -1,7 +1,6 @@
 package org.checkerframework.common.value;
 
 import com.sun.source.tree.ExpressionTree;
-import com.sun.source.tree.Tree;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -11,6 +10,7 @@ import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.type.TypeMirror;
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.common.value.qual.ArrayLen;
 import org.checkerframework.common.value.qual.ArrayLenRange;
 import org.checkerframework.common.value.qual.StringVal;
@@ -49,7 +49,6 @@ import org.checkerframework.dataflow.cfg.node.NumericalMultiplicationNode;
 import org.checkerframework.dataflow.cfg.node.NumericalPlusNode;
 import org.checkerframework.dataflow.cfg.node.NumericalSubtractionNode;
 import org.checkerframework.dataflow.cfg.node.SignedRightShiftNode;
-import org.checkerframework.dataflow.cfg.node.StringConcatenateAssignmentNode;
 import org.checkerframework.dataflow.cfg.node.StringConcatenateNode;
 import org.checkerframework.dataflow.cfg.node.StringConversionNode;
 import org.checkerframework.dataflow.cfg.node.StringLiteralNode;
@@ -72,10 +71,15 @@ import org.plumelib.util.CollectionsPlume;
 
 /** The transfer class for the Value Checker. */
 public class ValueTransfer extends CFTransfer {
+
   /** The Value type factory. */
   protected final ValueAnnotatedTypeFactory atypeFactory;
+
   /** The Value qualifier hierarchy. */
-  protected final QualifierHierarchy hierarchy;
+  protected final QualifierHierarchy qualHierarchy;
+
+  /** True if -AnonNullStringsConcatenation was passed on the command line. */
+  private final boolean nonNullStringsConcatenation;
 
   /**
    * Create a new ValueTransfer.
@@ -85,7 +89,9 @@ public class ValueTransfer extends CFTransfer {
   public ValueTransfer(CFAbstractAnalysis<CFValue, CFStore, CFTransfer> analysis) {
     super(analysis);
     atypeFactory = (ValueAnnotatedTypeFactory) analysis.getTypeFactory();
-    hierarchy = atypeFactory.getQualifierHierarchy();
+    qualHierarchy = atypeFactory.getQualifierHierarchy();
+    nonNullStringsConcatenation =
+        atypeFactory.getChecker().hasOption("nonNullStringsConcatenation");
   }
 
   /** Returns a range of possible lengths for an integer from a range, as casted to a String. */
@@ -111,10 +117,10 @@ public class ValueTransfer extends CFTransfer {
    * Returns a range of possible lengths for {@code subNode}, as casted to a String.
    *
    * @param subNode some subnode of {@code p}
-   * @param p TransferInput
+   * @param p a TransferInput
    * @return a range of possible lengths for {@code subNode}, as casted to a String
    */
-  private Range getStringLengthRange(Node subNode, TransferInput<CFValue, CFStore> p) {
+  private @Nullable Range getStringLengthRange(Node subNode, TransferInput<CFValue, CFStore> p) {
     CFValue value = p.getValueOfSubNode(subNode);
 
     AnnotationMirror anno = getValueAnnotation(value);
@@ -151,7 +157,8 @@ public class ValueTransfer extends CFTransfer {
    * {@code subNode}'s type is top/unknown. Returns an empty list if {@code subNode}'s type is
    * bottom.
    */
-  private List<Integer> getStringLengths(Node subNode, TransferInput<CFValue, CFStore> p) {
+  private @Nullable List<Integer> getStringLengths(
+      Node subNode, TransferInput<CFValue, CFStore> p) {
 
     CFValue value = p.getValueOfSubNode(subNode);
     AnnotationMirror anno = getValueAnnotation(value);
@@ -174,8 +181,9 @@ public class ValueTransfer extends CFTransfer {
       // characters always have length 1
       return Collections.singletonList(1);
     } else if (isIntRange(subNode, p)) {
-      // Try to get a list of lengths from a range of integer values converted to string @IntVal is
-      // not checked for, because if it is present, we would already have the actual string values
+      // Try to get a list of lengths from a range of integer values converted to string.
+      // @IntVal is not checked for, because if it is present, we would already have the
+      // actual string values.
       Range lengthRange = getIntRangeStringLengthRange(subNode, p);
       return ValueCheckerUtils.getValuesFromRange(lengthRange, Integer.class);
     } else if (subNodeTypeKind == TypeKind.BYTE) {
@@ -195,10 +203,10 @@ public class ValueTransfer extends CFTransfer {
    * bottom.
    *
    * @param subNode a subNode of p
-   * @param p TransferInput
+   * @param p a TransferInput
    * @return a list of possible values for {@code subNode} or null
    */
-  private List<String> getStringValues(Node subNode, TransferInput<CFValue, CFStore> p) {
+  private @Nullable List<String> getStringValues(Node subNode, TransferInput<CFValue, CFStore> p) {
     CFValue value = p.getValueOfSubNode(subNode);
     AnnotationMirror anno = getValueAnnotation(value);
     if (anno == null) {
@@ -258,7 +266,7 @@ public class ValueTransfer extends CFTransfer {
    * @param value a CFValue
    * @return theboolean if {@code value} represents a single boolean value; otherwise null
    */
-  private Boolean getBooleanValue(CFValue value) {
+  private @Nullable Boolean getBooleanValue(CFValue value) {
     AnnotationMirror boolAnno =
         AnnotationUtils.getAnnotationByName(
             value.getAnnotations(), ValueAnnotatedTypeFactory.BOOLVAL_NAME);
@@ -273,7 +281,8 @@ public class ValueTransfer extends CFTransfer {
    * @param p the transfer input in which to look up values
    * @return the possible boolean values for the node
    */
-  private List<Boolean> getBooleanValues(Node subNode, TransferInput<CFValue, CFStore> p) {
+  private @Nullable List<Boolean> getBooleanValues(
+      Node subNode, TransferInput<CFValue, CFStore> p) {
     CFValue value = p.getValueOfSubNode(subNode);
     AnnotationMirror intAnno =
         AnnotationUtils.getAnnotationByName(
@@ -295,7 +304,7 @@ public class ValueTransfer extends CFTransfer {
 
     if (atypeFactory.isIntRange(value.getAnnotations())) {
       intAnno =
-          hierarchy.findAnnotationInHierarchy(value.getAnnotations(), atypeFactory.UNKNOWNVAL);
+          qualHierarchy.findAnnotationInHierarchy(value.getAnnotations(), atypeFactory.UNKNOWNVAL);
       Range range = atypeFactory.getRange(intAnno);
       return ValueCheckerUtils.getValuesFromRange(range, Character.class);
     }
@@ -315,14 +324,15 @@ public class ValueTransfer extends CFTransfer {
    * @return the Value Checker annotation within cfValue
    */
   private AnnotationMirror getValueAnnotation(CFValue cfValue) {
-    return hierarchy.findAnnotationInHierarchy(cfValue.getAnnotations(), atypeFactory.UNKNOWNVAL);
+    return qualHierarchy.findAnnotationInHierarchy(
+        cfValue.getAnnotations(), atypeFactory.UNKNOWNVAL);
   }
 
   /**
    * Returns a list of possible values, or null if no estimate is available and any value is
    * possible.
    */
-  private List<? extends Number> getNumericalValues(
+  private @Nullable List<? extends Number> getNumericalValues(
       Node subNode, TransferInput<CFValue, CFStore> p) {
     AnnotationMirror valueAnno = getValueAnnotation(subNode, p);
     return getNumericalValues(subNode, valueAnno);
@@ -335,7 +345,8 @@ public class ValueTransfer extends CFTransfer {
    * @param valueAnno annotation mirror
    * @return the numerical values in valueAnno casted to the type of subNode
    */
-  private List<? extends Number> getNumericalValues(Node subNode, AnnotationMirror valueAnno) {
+  private @Nullable List<? extends Number> getNumericalValues(
+      Node subNode, AnnotationMirror valueAnno) {
 
     if (valueAnno == null
         || AnnotationUtils.areSameByName(valueAnno, ValueAnnotatedTypeFactory.UNKNOWN_NAME)) {
@@ -393,7 +404,7 @@ public class ValueTransfer extends CFTransfer {
    * Returns true if subNode is annotated with {@code @IntRange}.
    *
    * @param subNode subNode of {@code p}
-   * @param p TransferInput
+   * @param p a TransferInput
    * @return true if this subNode is annotated with {@code @IntRange}
    */
   private boolean isIntRange(Node subNode, TransferInput<CFValue, CFStore> p) {
@@ -428,24 +439,21 @@ public class ValueTransfer extends CFTransfer {
   }
 
   /**
-   * Create a new transfer result based on the original result and the new annotation.
+   * Create a new boolean transfer result based on the original result and the new annotation.
    *
-   * @param result the original result
-   * @param resultAnno the new annotation
-   * @return the new transfer result
+   * @param thenStore the then store for the result
+   * @param elseStore the else store for the result
+   * @param booleanValues the possible values that the result might evaluate to
+   * @param underlyingType the (boolean) result type. This is always boolean; it is an argument
+   *     because there is no easy way to get a TypeMirror for a specific type.
+   * @return a transfer result like {@code result}, but permitting only the given boolean values
    */
-  private TransferResult<CFValue, CFStore> createNewResult(
-      TransferResult<CFValue, CFStore> result, AnnotationMirror resultAnno) {
-    CFValue newResultValue =
-        analysis.createSingleAnnotationValue(
-            resultAnno, result.getResultValue().getUnderlyingType());
-    return new RegularTransferResult<>(newResultValue, result.getRegularStore());
-  }
-
-  /** Create a boolean transfer result. */
   private TransferResult<CFValue, CFStore> createNewResultBoolean(
-      CFStore thenStore, CFStore elseStore, List<Boolean> resultValues, TypeMirror underlyingType) {
-    AnnotationMirror boolVal = atypeFactory.createBooleanAnnotation(resultValues);
+      CFStore thenStore,
+      CFStore elseStore,
+      List<Boolean> booleanValues,
+      TypeMirror underlyingType) {
+    AnnotationMirror boolVal = atypeFactory.createBooleanAnnotation(booleanValues);
     CFValue newResultValue = analysis.createSingleAnnotationValue(boolVal, underlyingType);
     if (elseStore != null) {
       return new ConditionalTransferResult<>(newResultValue, thenStore, elseStore);
@@ -526,7 +534,7 @@ public class ValueTransfer extends CFTransfer {
   }
 
   /**
-   * Transform @IntVal or @IntRange annotations of a array or string length into an @ArrayLen
+   * Transform @IntVal or @IntRange annotations of an array or string length into an @ArrayLen
    * or @ArrayLenRange annotation for the array or string.
    *
    * @param lengthNode an invocation of method {@code length} or an access of the {@code length}
@@ -552,11 +560,11 @@ public class ValueTransfer extends CFTransfer {
     if (lengthAnno == null) {
       return;
     }
+    JavaExpression receiverJE = JavaExpression.fromNode(receiverNode);
     if (AnnotationUtils.areSameByName(lengthAnno, ValueAnnotatedTypeFactory.BOTTOMVAL_NAME)) {
       // If the length is bottom, then this is dead code, so the receiver type
       // should also be bottom.
-      JavaExpression receiver = JavaExpression.fromNode(receiverNode);
-      store.insertValue(receiver, lengthAnno);
+      store.insertValue(receiverJE, lengthAnno);
       return;
     }
 
@@ -579,17 +587,11 @@ public class ValueTransfer extends CFTransfer {
     if (oldRecAnno == null) {
       combinedRecAnno = newRecAnno;
     } else {
-      combinedRecAnno = hierarchy.greatestLowerBound(oldRecAnno, newRecAnno);
+      TypeMirror receiverTM = receiverJE.getType();
+      combinedRecAnno =
+          qualHierarchy.greatestLowerBoundShallow(oldRecAnno, receiverTM, newRecAnno, receiverTM);
     }
-    JavaExpression receiver = JavaExpression.fromNode(receiverNode);
-    store.insertValue(receiver, combinedRecAnno);
-  }
-
-  @Override
-  public TransferResult<CFValue, CFStore> visitStringConcatenateAssignment(
-      StringConcatenateAssignmentNode n, TransferInput<CFValue, CFStore> p) {
-    TransferResult<CFValue, CFStore> result = super.visitStringConcatenateAssignment(n, p);
-    return stringConcatenation(n.getLeftOperand(), n.getRightOperand(), p, result);
+    store.insertValue(receiverJE, combinedRecAnno);
   }
 
   @Override
@@ -645,7 +647,7 @@ public class ValueTransfer extends CFTransfer {
       return false;
     }
 
-    Element element = TreeUtils.elementFromUse((ExpressionTree) node.getTree());
+    Element element = TreeUtils.elementFromTree((ExpressionTree) node.getTree());
     return !ElementUtils.isCompileTimeConstant(element);
   }
 
@@ -657,12 +659,9 @@ public class ValueTransfer extends CFTransfer {
     List<String> leftValues = getStringValues(leftOperand, p);
     List<String> rightValues = getStringValues(rightOperand, p);
 
-    boolean nonNullStringConcat =
-        atypeFactory.getChecker().hasOption("nonNullStringsConcatenation");
-
     if (leftValues != null && rightValues != null) {
       // Both operands have known string values, compute set of results
-      if (!nonNullStringConcat) {
+      if (!nonNullStringsConcatenation) {
         if (isNullable(leftOperand)) {
           leftValues = CollectionsPlume.append(leftValues, "null");
         }
@@ -705,7 +704,7 @@ public class ValueTransfer extends CFTransfer {
 
     if (leftLengths != null && rightLengths != null) {
       // Both operands have known lengths, compute set of result lengths
-      if (!nonNullStringConcat) {
+      if (!nonNullStringsConcatenation) {
         if (isNullable(leftOperand)) {
           leftLengths = new ArrayList<>(leftLengths);
           leftLengths.add(4); // "null"
@@ -731,7 +730,7 @@ public class ValueTransfer extends CFTransfer {
 
     if (leftLengthRange != null && rightLengthRange != null) {
       // Both operands have a length from a known range, compute a range of result lengths
-      if (!nonNullStringConcat) {
+      if (!nonNullStringsConcatenation) {
         if (isNullable(leftOperand)) {
           leftLengthRange = leftLengthRange.union(Range.create(4, 4)); // "null"
         }
@@ -751,13 +750,9 @@ public class ValueTransfer extends CFTransfer {
       Node rightOperand,
       TransferInput<CFValue, CFStore> p,
       TransferResult<CFValue, CFStore> result) {
-
     AnnotationMirror resultAnno =
         createAnnotationForStringConcatenation(leftOperand, rightOperand, p);
-
-    TypeMirror underlyingType = result.getResultValue().getUnderlyingType();
-    CFValue newResultValue = analysis.createSingleAnnotationValue(resultAnno, underlyingType);
-    return new RegularTransferResult<>(newResultValue, result.getRegularStore());
+    return recreateTransferResult(resultAnno, result);
   }
 
   /** Binary operations that are analyzed by the value checker. */
@@ -853,14 +848,14 @@ public class ValueTransfer extends CFTransfer {
   }
 
   /** Calculate the possible values after a binary operation between two numerical type nodes. */
-  private List<Number> calculateValuesBinaryOp(
+  private @Nullable List<Number> calculateValuesBinaryOp(
       Node leftNode, Node rightNode, NumericalBinaryOps op, TransferInput<CFValue, CFStore> p) {
     List<? extends Number> lefts = getNumericalValues(leftNode, p);
     List<? extends Number> rights = getNumericalValues(rightNode, p);
     if (lefts == null || rights == null) {
       return null;
     }
-    List<Number> resultValues = new ArrayList<>();
+    List<Number> resultValues = new ArrayList<>(lefts.size() * rights.size());
     for (Number left : lefts) {
       NumberMath<?> nmLeft = NumberMath.getNumberMath(left);
       for (Number right : rights) {
@@ -919,7 +914,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.ADDITION, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -929,7 +924,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.SUBTRACTION, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -939,7 +934,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.MULTIPLICATION, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -949,7 +944,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.DIVISION, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -959,7 +954,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.DIVISION, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -969,7 +964,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.REMAINDER, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -979,7 +974,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.REMAINDER, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -989,7 +984,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.SHIFT_LEFT, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -999,7 +994,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.SIGNED_SHIFT_RIGHT, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -1009,7 +1004,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.UNSIGNED_SHIFT_RIGHT, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -1019,7 +1014,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.BITWISE_AND, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -1029,7 +1024,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.BITWISE_OR, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -1039,7 +1034,7 @@ public class ValueTransfer extends CFTransfer {
     AnnotationMirror resultAnno =
         calculateNumericalBinaryOp(
             n.getLeftOperand(), n.getRightOperand(), NumericalBinaryOps.BITWISE_XOR, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   /** Unary operations that are analyzed by the value checker. */
@@ -1103,7 +1098,7 @@ public class ValueTransfer extends CFTransfer {
   }
 
   /** Calculate the possible values after a unary operation of a numerical type node. */
-  private List<Number> calculateValuesUnaryOp(
+  private @Nullable List<Number> calculateValuesUnaryOp(
       Node operand, NumericalUnaryOps op, TransferInput<CFValue, CFStore> p) {
     List<? extends Number> lefts = getNumericalValues(operand, p);
     if (lefts == null) {
@@ -1135,7 +1130,7 @@ public class ValueTransfer extends CFTransfer {
     TransferResult<CFValue, CFStore> transferResult = super.visitNumericalMinus(n, p);
     AnnotationMirror resultAnno =
         calculateNumericalUnaryOp(n.getOperand(), NumericalUnaryOps.MINUS, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -1144,7 +1139,7 @@ public class ValueTransfer extends CFTransfer {
     TransferResult<CFValue, CFStore> transferResult = super.visitNumericalPlus(n, p);
     AnnotationMirror resultAnno =
         calculateNumericalUnaryOp(n.getOperand(), NumericalUnaryOps.PLUS, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   @Override
@@ -1153,7 +1148,7 @@ public class ValueTransfer extends CFTransfer {
     TransferResult<CFValue, CFStore> transferResult = super.visitBitwiseComplement(n, p);
     AnnotationMirror resultAnno =
         calculateNumericalUnaryOp(n.getOperand(), NumericalUnaryOps.BITWISE_COMPLEMENT, p);
-    return createNewResult(transferResult, resultAnno);
+    return recreateTransferResult(resultAnno, transferResult);
   }
 
   enum ComparisonOperators {
@@ -1165,7 +1160,7 @@ public class ValueTransfer extends CFTransfer {
     LESS_THAN_EQ;
   }
 
-  private List<Boolean> calculateBinaryComparison(
+  private @Nullable List<Boolean> calculateBinaryComparison(
       Node leftNode,
       CFValue leftValue,
       Node rightNode,
@@ -1180,10 +1175,10 @@ public class ValueTransfer extends CFTransfer {
         || atypeFactory.isIntRange(rightAnno)
         || isIntegralUnknownVal(rightNode, rightAnno)
         || isIntegralUnknownVal(leftNode, leftAnno)) {
-      // If either is @UnknownVal, then refineIntRanges will treat it as the max range and thus
-      // refine it if possible.  Also, if either is an @IntVal, then it will be converted to a
-      // range.  This is less precise in some cases, but avoids the complexity of comparing a list
-      // of values to a range. (This could be implemented in the future.)
+      // If either is @UnknownVal, then refineIntRanges will treat it as the max range and
+      // thus refine it if possible.  Also, if either is an @IntVal, then it will be converted
+      // to a range.  This is less precise in some cases, but avoids the complexity of
+      // comparing a list of values to a range. (This could be implemented in the future.)
       return refineIntRanges(leftNode, leftAnno, rightNode, rightAnno, op, thenStore, elseStore);
     }
 
@@ -1200,14 +1195,15 @@ public class ValueTransfer extends CFTransfer {
     }
 
     // This is a list of all the values that the expression can evaluate to.
-    List<Boolean> resultValues = new ArrayList<>();
+    int numResultValues = lefts.size() * rights.size();
+    List<Boolean> resultValues = new ArrayList<>(numResultValues);
 
     // These lists are used to refine the values in the store based on the results of the
     // comparison.
-    List<Number> thenLeftVals = new ArrayList<>();
-    List<Number> elseLeftVals = new ArrayList<>();
-    List<Number> thenRightVals = new ArrayList<>();
-    List<Number> elseRightVals = new ArrayList<>();
+    List<Number> thenLeftVals = new ArrayList<>(numResultValues);
+    List<Number> elseLeftVals = new ArrayList<>(numResultValues);
+    List<Number> thenRightVals = new ArrayList<>(numResultValues);
+    List<Number> elseRightVals = new ArrayList<>(numResultValues);
 
     for (Number left : lefts) {
       NumberMath<?> nmLeft = NumberMath.getNumberMath(left);
@@ -1258,7 +1254,7 @@ public class ValueTransfer extends CFTransfer {
    * Calculates the result of a binary comparison on a pair of intRange annotations, and refines
    * annotations appropriately.
    */
-  private List<Boolean> refineIntRanges(
+  private @Nullable List<Boolean> refineIntRanges(
       Node leftNode,
       AnnotationMirror leftAnno,
       Node rightNode,
@@ -1367,8 +1363,10 @@ public class ValueTransfer extends CFTransfer {
           (currentValueFromStore == null
               ? atypeFactory.UNKNOWNVAL
               : getValueAnnotation(currentValueFromStore));
-      // Combine the new annotations based on the results of the comparison with the existing type.
-      AnnotationMirror newAnno = hierarchy.greatestLowerBound(anno, currentAnno);
+      // Combine the new annotations based on the results of the comparison with the existing
+      // type.
+      AnnotationMirror newAnno =
+          qualHierarchy.greatestLowerBoundShallow(anno, je.getType(), currentAnno, je.getType());
       store.insertValue(je, newAnno);
 
       if (node instanceof FieldAccessNode) {
@@ -1495,12 +1493,11 @@ public class ValueTransfer extends CFTransfer {
   protected void processConditionalPostconditions(
       MethodInvocationNode n,
       ExecutableElement methodElement,
-      Tree tree,
+      ExpressionTree tree,
       CFStore thenStore,
       CFStore elseStore) {
     // For String.startsWith(String) and String.endsWith(String), refine the minimum length
     // of the receiver to the minimum length of the argument.
-
     ValueMethodIdentifier methodIdentifier = atypeFactory.getMethodIdentifier();
     if (methodIdentifier.isStartsWithMethod(methodElement)
         || methodIdentifier.isEndsWithMethod(methodElement)) {
