@@ -14,10 +14,22 @@ import java.util.Map;
 import java.util.Set;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.Element;
+import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.ExecutableElement;
+import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.type.TypeMirror;
+import org.checkerframework.afu.scenelib.Annotation;
+import org.checkerframework.afu.scenelib.el.AClass;
+import org.checkerframework.afu.scenelib.el.AField;
+import org.checkerframework.afu.scenelib.el.AMethod;
+import org.checkerframework.afu.scenelib.el.AScene;
+import org.checkerframework.afu.scenelib.el.ATypeElement;
+import org.checkerframework.afu.scenelib.el.TypePathEntry;
+import org.checkerframework.afu.scenelib.io.IndexFileParser;
+import org.checkerframework.afu.scenelib.util.JVMNames;
+import org.checkerframework.checker.index.qual.Positive;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.signature.qual.BinaryName;
 import org.checkerframework.common.basetype.BaseTypeChecker;
@@ -36,28 +48,22 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedArrayTyp
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedNullType;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedTypeVariable;
 import org.checkerframework.framework.type.GenericAnnotatedTypeFactory;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.ElementUtils;
-import org.checkerframework.javacutil.Pair;
+import org.checkerframework.javacutil.TypeSystemError;
 import org.checkerframework.javacutil.UserError;
-import scenelib.annotations.Annotation;
-import scenelib.annotations.el.AClass;
-import scenelib.annotations.el.AField;
-import scenelib.annotations.el.AMethod;
-import scenelib.annotations.el.AScene;
-import scenelib.annotations.el.ATypeElement;
-import scenelib.annotations.el.TypePathEntry;
-import scenelib.annotations.io.IndexFileParser;
-import scenelib.annotations.util.JVMNames;
+import org.plumelib.util.CollectionsPlume;
+import org.plumelib.util.IPair;
 
 /**
  * This class stores annotations using scenelib objects.
  *
- * <p>The set of annotations inferred for a certain class is stored in an {@link
- * scenelib.annotations.el.AScene}, which {@code writeScenes()} can write into a file. For example,
- * a class {@code my.pakkage.MyClass} will have its members' inferred types stored in a Scene, and
- * later written into a file named {@code my.pakkage.MyClass.jaif} if using {@link
- * OutputFormat#JAIF}, or {@code my.pakkage.MyClass.astub} if using {@link OutputFormat#STUB}.
+ * <p>The set of annotations inferred for a certain class is stored in an {@link AScene}, which
+ * {@code writeScenes()} can write into a file. For example, a class {@code my.pakkage.MyClass} will
+ * have its members' inferred types stored in a Scene, and later written into a file named {@code
+ * my.pakkage.MyClass.jaif} if using {@link OutputFormat#JAIF}, or {@code my.pakkage.MyClass.astub}
+ * if using {@link OutputFormat#STUB}.
  *
  * <p>This class populates the initial Scenes by reading existing .jaif files on the {@link
  * #JAIF_FILES_PATH} directory (regardless of output format). Having more information in those
@@ -156,6 +162,12 @@ public class WholeProgramInferenceScenesStorage
         ClassSymbol enclosingClass = ((VarSymbol) elt).enclClass();
         className = enclosingClass.flatname.toString();
         break;
+      case CLASS:
+        className = ElementUtils.getBinaryName((TypeElement) elt);
+        break;
+      case PARAMETER:
+        className = ElementUtils.getEnclosingClassName((VariableElement) elt);
+        break;
       default:
         throw new BugInCF("What element? %s %s", elt.getKind(), elt);
     }
@@ -201,8 +213,8 @@ public class WholeProgramInferenceScenesStorage
    * @param fieldElt the field
    * @return the annotations for a field
    */
-  private AField getFieldAnnos(Element fieldElt) {
-    String className = ElementUtils.getEnclosingClassName((VariableElement) fieldElt);
+  private AField getFieldAnnos(VariableElement fieldElt) {
+    String className = ElementUtils.getEnclosingClassName(fieldElt);
     String file = getFileForElement(fieldElt);
     AClass classAnnos = getClassAnnos(className, file, ((VarSymbol) fieldElt).enclClass());
     AField fieldAnnos = classAnnos.fields.getVivify(fieldElt.getSimpleName().toString());
@@ -211,21 +223,28 @@ public class WholeProgramInferenceScenesStorage
 
   @Override
   public boolean hasStorageLocationForMethod(ExecutableElement methodElt) {
-    // The scenes implementation can always add annotations to a method.
-    return true;
+    // The only location that the scenes implementation cannot store an annotation is on
+    // a member of an annotation type, which it cannot distinguish from a normal interface's
+    // method. Without this, the scenes implementation will attempt to annotate annotation
+    // elements, which is an error.
+    Element enclosingType = ElementUtils.enclosingTypeElement(methodElt);
+    return enclosingType == null || enclosingType.getKind() != ElementKind.ANNOTATION_TYPE;
   }
 
   @Override
   public ATypeElement getParameterAnnotations(
       ExecutableElement methodElt,
-      int i,
+      @Positive int index_1based,
       AnnotatedTypeMirror paramATM,
       VariableElement ve,
       AnnotatedTypeFactory atypeFactory) {
+    if (index_1based == 0) {
+      throw new TypeSystemError("0 is illegal as index argument to getParameterAnnotations");
+    }
     AMethod methodAnnos = getMethodAnnos(methodElt);
     AField param =
         methodAnnos.vivifyAndAddTypeMirrorToParameter(
-            i, paramATM.getUnderlyingType(), ve.getSimpleName());
+            index_1based - 1, paramATM.getUnderlyingType(), ve.getSimpleName());
     return param.type;
   }
 
@@ -238,6 +257,26 @@ public class WholeProgramInferenceScenesStorage
     return methodAnnos.receiver.type;
   }
 
+  @Override
+  public AnnotationMirrorSet getMethodDeclarationAnnotations(ExecutableElement elt) {
+    AMethod methodAnnos = getMethodAnnos(elt);
+    Set<Annotation> annos = methodAnnos.tlAnnotationsHere;
+    AnnotationMirrorSet result = new AnnotationMirrorSet();
+    for (Annotation anno : annos) {
+      result.add(
+          AnnotationConverter.annotationToAnnotationMirror(anno, atypeFactory.getProcessingEnv()));
+    }
+    return result;
+  }
+
+  @Override
+  public boolean removeMethodDeclarationAnnotation(
+      ExecutableElement methodElt, AnnotationMirror anno) {
+    AMethod methodAnnos = getMethodAnnos(methodElt);
+    return methodAnnos.tlAnnotationsHere.remove(
+        AnnotationConverter.annotationMirrorToAnnotation(anno));
+  }
+
   @Override
   public ATypeElement getReturnAnnotations(
       ExecutableElement methodElt, AnnotatedTypeMirror atm, AnnotatedTypeFactory atypeFactory) {
@@ -263,6 +302,7 @@ public class WholeProgramInferenceScenesStorage
 
   @Override
   public ATypeElement getPreOrPostconditions(
+      String className,
       Analysis.BeforeOrAfter preOrPost,
       ExecutableElement methodElement,
       String expression,
@@ -270,9 +310,9 @@ public class WholeProgramInferenceScenesStorage
       AnnotatedTypeFactory atypeFactory) {
     switch (preOrPost) {
       case BEFORE:
-        return getPreconditionsForExpression(methodElement, expression, declaredType);
+        return getPreconditionsForExpression(className, methodElement, expression, declaredType);
       case AFTER:
-        return getPostconditionsForExpression(methodElement, expression, declaredType);
+        return getPostconditionsForExpression(className, methodElement, expression, declaredType);
       default:
         throw new BugInCF("Unexpected " + preOrPost);
     }
@@ -281,13 +321,18 @@ public class WholeProgramInferenceScenesStorage
   /**
    * Returns the precondition annotations for a Java expression.
    *
+   * @param className the class that contains the method, for diagnostics only
    * @param methodElement the method
    * @param expression the expression
    * @param declaredType the declared type of the expression
    * @return the precondition annotations for a Java expression
    */
+  @SuppressWarnings("UnusedVariable")
   private ATypeElement getPreconditionsForExpression(
-      ExecutableElement methodElement, String expression, AnnotatedTypeMirror declaredType) {
+      String className,
+      ExecutableElement methodElement,
+      String expression,
+      AnnotatedTypeMirror declaredType) {
     AMethod methodAnnos = getMethodAnnos(methodElement);
     preconditionsToDeclaredTypes.put(methodAnnos.methodSignature + expression, declaredType);
     return methodAnnos.vivifyAndAddTypeMirrorToPrecondition(
@@ -298,13 +343,18 @@ public class WholeProgramInferenceScenesStorage
   /**
    * Returns the postcondition annotations for a Java expression.
    *
+   * @param className the class that contains the method, for diagnostics only
    * @param methodElement the method
    * @param expression the expression
    * @param declaredType the declared type of the expression
    * @return the postcondition annotations for a Java expression
    */
+  @SuppressWarnings("UnusedVariable")
   private ATypeElement getPostconditionsForExpression(
-      ExecutableElement methodElement, String expression, AnnotatedTypeMirror declaredType) {
+      String className,
+      ExecutableElement methodElement,
+      String expression,
+      AnnotatedTypeMirror declaredType) {
     AMethod methodAnnos = getMethodAnnos(methodElement);
     postconditionsToDeclaredTypes.put(methodAnnos.methodSignature + expression, declaredType);
     return methodAnnos.vivifyAndAddTypeMirrorToPostcondition(
@@ -361,27 +411,64 @@ public class WholeProgramInferenceScenesStorage
 
     AMethod methodAnnos = getMethodAnnos(methodElt);
 
-    scenelib.annotations.Annotation sceneAnno =
-        AnnotationConverter.annotationMirrorToAnnotation(anno);
+    Annotation sceneAnno = AnnotationConverter.annotationMirrorToAnnotation(anno);
     boolean isNewAnnotation = methodAnnos.tlAnnotationsHere.add(sceneAnno);
     return isNewAnnotation;
   }
 
   @Override
-  public boolean addFieldDeclarationAnnotation(Element field, AnnotationMirror anno) {
+  public boolean addFieldDeclarationAnnotation(VariableElement field, AnnotationMirror anno) {
     if (!ElementUtils.isElementFromSourceCode(field)) {
       return false;
     }
 
     AField fieldAnnos = getFieldAnnos(field);
 
-    scenelib.annotations.Annotation sceneAnno =
-        AnnotationConverter.annotationMirrorToAnnotation(anno);
+    Annotation sceneAnno = AnnotationConverter.annotationMirrorToAnnotation(anno);
 
     boolean isNewAnnotation = fieldAnnos.tlAnnotationsHere.add(sceneAnno);
     return isNewAnnotation;
   }
 
+  @Override
+  public boolean addDeclarationAnnotationToFormalParameter(
+      ExecutableElement methodElt, @Positive int index_1based, AnnotationMirror anno) {
+    if (index_1based == 0) {
+      throw new TypeSystemError(
+          "0 is illegal as index argument to addDeclarationAnnotationToFormalParameter");
+    }
+    if (!ElementUtils.isElementFromSourceCode(methodElt)) {
+      return false;
+    }
+
+    VariableElement paramElt = methodElt.getParameters().get(index_1based - 1);
+    AnnotatedTypeMirror paramAType = atypeFactory.getAnnotatedType(paramElt);
+    ATypeElement paramAnnos =
+        getParameterAnnotations(methodElt, index_1based, paramAType, paramElt, atypeFactory);
+    Annotation sceneAnno = AnnotationConverter.annotationMirrorToAnnotation(anno);
+
+    boolean isNewAnnotation = paramAnnos.tlAnnotationsHere.add(sceneAnno);
+    return isNewAnnotation;
+  }
+
+  @Override
+  public boolean addClassDeclarationAnnotation(TypeElement classElt, AnnotationMirror anno) {
+    if (!ElementUtils.isElementFromSourceCode(classElt)) {
+      return false;
+    }
+
+    AClass classAnnos =
+        getClassAnnos(
+            ElementUtils.getBinaryName(classElt),
+            getFileForElement(classElt),
+            (ClassSymbol) classElt);
+
+    Annotation sceneAnno = AnnotationConverter.annotationMirrorToAnnotation(anno);
+
+    boolean isNewAnnotation = classAnnos.tlAnnotationsHere.add(sceneAnno);
+    return isNewAnnotation;
+  }
+
   /**
    * Write all modified scenes into files. (Scenes are modified by the method {@link
    * #updateAnnotationSetInScene}.)
@@ -483,7 +570,7 @@ public class WholeProgramInferenceScenesStorage
    *       annotation and rhsATM.
    * </ul>
    *
-   * @param type ATypeElement of the Scene which will be modified
+   * @param type the ATypeElement of the Scene which will be modified
    * @param jaifPath path to a .jaif file for a Scene; used for marking the scene as modified
    *     (needing to be written to disk)
    * @param rhsATM the RHS of the annotated type on the source code
@@ -502,15 +589,19 @@ public class WholeProgramInferenceScenesStorage
     if (rhsATM instanceof AnnotatedNullType && ignoreNullAssignments) {
       return;
     }
-    AnnotatedTypeMirror atmFromScene = atmFromStorageLocation(rhsATM.getUnderlyingType(), type);
+    TypeMirror rhsTM = rhsATM.getUnderlyingType();
+    AnnotatedTypeMirror atmFromScene = atmFromStorageLocation(rhsTM, type);
     updateAtmWithLub(rhsATM, atmFromScene);
     if (lhsATM instanceof AnnotatedTypeVariable) {
-      Set<AnnotationMirror> upperAnnos =
+      AnnotationMirrorSet upperAnnos =
           ((AnnotatedTypeVariable) lhsATM).getUpperBound().getEffectiveAnnotations();
       // If the inferred type is a subtype of the upper bounds of the
       // current type on the source code, halt.
-      if (upperAnnos.size() == rhsATM.getAnnotations().size()
-          && atypeFactory.getQualifierHierarchy().isSubtype(rhsATM.getAnnotations(), upperAnnos)) {
+      if (upperAnnos.size() == rhsATM.getPrimaryAnnotations().size()
+          && atypeFactory
+              .getQualifierHierarchy()
+              .isSubtypeShallow(
+                  rhsATM.getPrimaryAnnotations(), rhsTM, upperAnnos, lhsATM.getUnderlyingType())) {
         return;
       }
     }
@@ -562,13 +653,20 @@ public class WholeProgramInferenceScenesStorage
     }
 
     // LUB primary annotations
-    Set<AnnotationMirror> annosToReplace = new HashSet<>(sourceCodeATM.getAnnotations().size());
-    for (AnnotationMirror amSource : sourceCodeATM.getAnnotations()) {
-      AnnotationMirror amJaif = jaifATM.getAnnotationInHierarchy(amSource);
-      // amJaif only contains  annotations from the jaif, so it might be missing
+    AnnotationMirrorSet annosToReplace = new AnnotationMirrorSet();
+    for (AnnotationMirror amSource : sourceCodeATM.getPrimaryAnnotations()) {
+      AnnotationMirror amJaif = jaifATM.getPrimaryAnnotationInHierarchy(amSource);
+      // amJaif only contains annotations from the jaif, so it might be missing
       // an annotation in the hierarchy
       if (amJaif != null) {
-        amSource = atypeFactory.getQualifierHierarchy().leastUpperBound(amSource, amJaif);
+        amSource =
+            atypeFactory
+                .getQualifierHierarchy()
+                .leastUpperBoundShallow(
+                    amSource,
+                    sourceCodeATM.getUnderlyingType(),
+                    amJaif,
+                    jaifATM.getUnderlyingType());
       }
       annosToReplace.add(amSource);
     }
@@ -689,10 +787,10 @@ public class WholeProgramInferenceScenesStorage
 
   /**
    * Updates an {@link org.checkerframework.framework.type.AnnotatedTypeMirror} to contain the
-   * {@link scenelib.annotations.Annotation}s of an {@link scenelib.annotations.el.ATypeElement}.
+   * {@link Annotation}s of an {@link ATypeElement}.
    *
    * @param result the AnnotatedTypeMirror to be modified
-   * @param storageLocation the {@link scenelib.annotations.el.ATypeElement} used
+   * @param storageLocation the {@link ATypeElement} used
    */
   private void updateAtmFromATypeElement(AnnotatedTypeMirror result, ATypeElement storageLocation) {
     Set<Annotation> annos = getSupportedAnnosInSet(storageLocation.tlAnnotationsHere);
@@ -725,9 +823,9 @@ public class WholeProgramInferenceScenesStorage
     updateTypeElementFromATM(typeToUpdate, defLoc, newATM, curATM, ignoreIfAnnotated);
   }
 
-  ///
-  /// Writing to a file
-  ///
+  //
+  // Writing to a file
+  //
 
   // The prepare*ForWriting hooks are needed in addition to the postProcessClassTree hook because
   // a scene may be modifed and written at any time, including before or after
@@ -741,7 +839,7 @@ public class WholeProgramInferenceScenesStorage
    */
   public void prepareSceneForWriting(AScene compilationUnitAnnos) {
     for (Map.Entry<String, AClass> classEntry : compilationUnitAnnos.classes.entrySet()) {
-      prepareClassForWriting(classEntry.getValue());
+      wpiPrepareClassForWriting(classEntry.getValue());
     }
   }
 
@@ -750,9 +848,9 @@ public class WholeProgramInferenceScenesStorage
    *
    * @param classAnnos the class annotations to modify
    */
-  public void prepareClassForWriting(AClass classAnnos) {
+  public void wpiPrepareClassForWriting(AClass classAnnos) {
     for (Map.Entry<String, AMethod> methodEntry : classAnnos.methods.entrySet()) {
-      prepareMethodForWriting(methodEntry.getValue());
+      wpiPrepareMethodForWriting(classAnnos.className, methodEntry.getValue());
     }
   }
 
@@ -760,10 +858,11 @@ public class WholeProgramInferenceScenesStorage
    * Side-effects the method or constructor annotations to make any desired changes before writing
    * to a file.
    *
+   * @param className the class that contains the method, for diagnostics only
    * @param methodAnnos the method or constructor annotations to modify
    */
-  public void prepareMethodForWriting(AMethod methodAnnos) {
-    atypeFactory.prepareMethodForWriting(methodAnnos);
+  public void wpiPrepareMethodForWriting(String className, AMethod methodAnnos) {
+    atypeFactory.wpiPrepareMethodForWriting(className, methodAnnos);
   }
 
   @Override
@@ -792,7 +891,7 @@ public class WholeProgramInferenceScenesStorage
   }
 
   /**
-   * Updates an {@link scenelib.annotations.el.ATypeElement} to have the annotations of an {@link
+   * Updates an {@link ATypeElement} to have the annotations of an {@link
    * org.checkerframework.framework.type.AnnotatedTypeMirror} passed as argument. Annotations in the
    * original set that should be ignored (see {@link #shouldIgnore}) are not added to the resulting
    * set. This method also checks if the AnnotatedTypeMirror has explicit annotations in source
@@ -821,13 +920,14 @@ public class WholeProgramInferenceScenesStorage
     // The others stay intact.
     Set<Annotation> annosToRemove = getSupportedAnnosInSet(typeToUpdate.tlAnnotationsHere);
     // This method may be called consecutive times for the same ATypeElement.  Each time it is
-    // called, the AnnotatedTypeMirror has a better type estimate for the ATypeElement. Therefore,
-    // it is not a problem to remove all annotations before inserting the new annotations.
+    // called, the AnnotatedTypeMirror has a better type estimate for the ATypeElement.
+    // Therefore, it is not a problem to remove all annotations before inserting the new
+    // annotations.
     typeToUpdate.tlAnnotationsHere.removeAll(annosToRemove);
 
     // Only update the ATypeElement if there are no explicit annotations.
     if (curATM.getExplicitAnnotations().isEmpty() || !ignoreIfAnnotated) {
-      for (AnnotationMirror am : newATM.getAnnotations()) {
+      for (AnnotationMirror am : newATM.getPrimaryAnnotations()) {
         addAnnotationsToATypeElement(
             newATM, typeToUpdate, defLoc, am, curATM.hasEffectiveAnnotation(am));
       }
@@ -836,8 +936,8 @@ public class WholeProgramInferenceScenesStorage
       // annotated.  So instead, only insert the annotation if there is not primary annotation
       // of the same hierarchy.  #shouldIgnore prevent annotations that are subtypes of type
       // vars upper bound from being inserted.
-      for (AnnotationMirror am : newATM.getAnnotations()) {
-        if (curATM.getAnnotationInHierarchy(am) != null) {
+      for (AnnotationMirror am : newATM.getPrimaryAnnotations()) {
+        if (curATM.getPrimaryAnnotationInHierarchy(am) != null) {
           // Don't insert if the type is already has a primary annotation
           // in the same hierarchy.
           break;
@@ -873,10 +973,10 @@ public class WholeProgramInferenceScenesStorage
       // firstKey works as a unique identifier for each annotation
       // that should not be inserted in source code
       String firstKey = aTypeElementToString(typeToUpdate);
-      Pair<String, TypeUseLocation> key = Pair.of(firstKey, defLoc);
+      IPair<String, TypeUseLocation> key = IPair.of(firstKey, defLoc);
       Set<String> annosIgnored = annosToIgnore.get(key);
       if (annosIgnored == null) {
-        annosIgnored = new HashSet<>();
+        annosIgnored = new HashSet<>(CollectionsPlume.mapCapacity(1));
         annosToIgnore.put(key, annosIgnored);
       }
       annosIgnored.add(anno.def().toString());
@@ -900,7 +1000,7 @@ public class WholeProgramInferenceScenesStorage
    * TypeUseLocation to a set of names of annotations.
    */
   public static class AnnotationsInContexts
-      extends HashMap<Pair<String, TypeUseLocation>, Set<String>> {
+      extends HashMap<IPair<String, TypeUseLocation>, Set<String>> {
     private static final long serialVersionUID = 20200321L;
   }
 
