@@ -1,5 +1,6 @@
 package org.checkerframework.framework.type;
 
+import java.io.Closeable;
 import java.io.File;
 import java.io.IOException;
 import java.lang.annotation.Annotation;
@@ -7,6 +8,8 @@ import java.lang.annotation.ElementType;
 import java.lang.annotation.Target;
 import java.net.JarURLConnection;
 import java.net.MalformedURLException;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.util.ArrayList;
@@ -20,13 +23,15 @@ import java.util.List;
 import java.util.Set;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
-import java.util.regex.Pattern;
 import javax.annotation.processing.ProcessingEnvironment;
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
-import javax.tools.Diagnostic.Kind;
+import javax.tools.Diagnostic;
+import org.checkerframework.checker.calledmethods.qual.EnsuresCalledMethods;
+import org.checkerframework.checker.mustcall.qual.InheritableMustCall;
+import org.checkerframework.checker.mustcall.qual.Owning;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.signature.qual.BinaryName;
 import org.checkerframework.checker.signature.qual.DotSeparatedIdentifiers;
@@ -36,6 +41,7 @@ import org.checkerframework.javacutil.AnnotationBuilder;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.InternalUtils;
+import org.checkerframework.javacutil.SystemUtil;
 import org.checkerframework.javacutil.UserError;
 import org.plumelib.reflection.Signatures;
 
@@ -60,29 +66,39 @@ import org.plumelib.reflection.Signatures;
  * #isSupportedAnnotationClass(Class)}. See {@code
  * org.checkerframework.checker.units.UnitsAnnotationClassLoader} for an example.
  */
-public class AnnotationClassLoader {
+@SuppressWarnings(
+    "mustcall:inconsistent.mustcall.subtype" // No need to check that AnnotationClassLoaders are
+// closed. (Just one is created per type factory.)
+)
+@InheritableMustCall({})
+public class AnnotationClassLoader implements Closeable {
   /** For issuing errors to the user. */
   protected final BaseTypeChecker checker;
 
   // For loading from a source package directory
   /** The package name. */
   private final @DotSeparatedIdentifiers String packageName;
+
   /** The package name, with periods replaced by slashes. */
   private final String packageNameWithSlashes;
+
   /** The atomic package names (the package name split at dots). */
   private final List<@Identifier String> fullyQualifiedPackageNameSegments;
+
   /** The name of a Checker's qualifier package. */
   private static final String QUAL_PACKAGE = "qual";
 
   // For loading from a Jar file
   /** The suffix for a .jar file. */
   private static final String JAR_SUFFIX = ".jar";
+
   /** The suffix for a .class file. */
   private static final String CLASS_SUFFIX = ".class";
 
   // Constants
   /** The package separator. */
   private static final char DOT = '.';
+
   /** The path separator, in .jar files, binary names, etc. */
   private static final char SLASH = '/';
 
@@ -96,7 +112,8 @@ public class AnnotationClassLoader {
   private final URL resourceURL;
 
   /** The class loader used to load annotation classes. */
-  protected final URLClassLoader classLoader;
+  @SuppressWarnings("builder:required.method.not.called") // this class is @MustCall({})
+  protected final @Owning URLClassLoader classLoader;
 
   /**
    * The annotation classes bundled with a checker (located in its qual directory) that are deemed
@@ -106,17 +123,13 @@ public class AnnotationClassLoader {
    */
   private final Set<Class<? extends Annotation>> supportedBundledAnnotationClasses;
 
-  /** The package separator: ".". */
-  private static final Pattern DOT_LITERAL_PATTERN =
-      Pattern.compile(Character.toString(DOT), Pattern.LITERAL);
-
   /**
    * Constructor for loading annotations defined for a checker.
    *
    * @param checker a {@link BaseTypeChecker} or its subclass
    */
   @SuppressWarnings("signature") // TODO: reduce use of string manipulation
-  public AnnotationClassLoader(final BaseTypeChecker checker) {
+  public AnnotationClassLoader(BaseTypeChecker checker) {
     this.checker = checker;
     processingEnv = checker.getProcessingEnvironment();
 
@@ -131,19 +144,20 @@ public class AnnotationClassLoader {
     // the package name with dots replaced by slashes will be used to scan file directories
     packageNameWithSlashes = packageName.replace(DOT, SLASH);
 
-    // Each component of the fully qualified package name will be used later to recursively descend
-    // from a root directory to see if the package exists in some particular root directory.
+    // Each component of the fully qualified package name will be used later to recursively
+    // descend from a root directory to see if the package exists in some particular root
+    // directory.
     fullyQualifiedPackageNameSegments = new ArrayList<>();
 
     // from the fully qualified package name, split it at every dot then add to the list
-    fullyQualifiedPackageNameSegments.addAll(Arrays.asList(DOT_LITERAL_PATTERN.split(packageName)));
+    fullyQualifiedPackageNameSegments.addAll(SystemUtil.dotSplitter.splitToList(packageName));
 
     classLoader = getClassLoader();
 
     URL localResourceURL;
     if (classLoader != null) {
-      // if the application classloader is accessible, then directly retrieve the resource URL of
-      // the qual package resource URLs must use slashes
+      // if the application classloader is accessible, then directly retrieve the resource URL
+      // of the qual package resource URLs must use slashes
       localResourceURL = classLoader.getResource(packageNameWithSlashes);
 
       // thread based application classloader, if needed in the future:
@@ -155,11 +169,10 @@ public class AnnotationClassLoader {
     }
 
     if (localResourceURL == null) {
-      // if the application classloader is not accessible (which means the checker class was loaded
-      // using the bootstrap classloader)
-      // or if the classloader didn't find the package,
-      // then scan the classpaths to find a jar or directory which contains the qual package and set
-      // the resource URL to that jar or qual directory
+      // if the application classloader is not accessible (which means the checker class was
+      // loaded using the bootstrap classloader) or if the classloader didn't find the
+      // package, then scan the classpaths to find a jar or directory which contains the qual
+      // package and set the resource URL to that jar or qual directory
       localResourceURL = getURLFromClasspaths();
     }
     resourceURL = localResourceURL;
@@ -169,6 +182,16 @@ public class AnnotationClassLoader {
     loadBundledAnnotationClasses();
   }
 
+  @EnsuresCalledMethods(value = "classLoader", methods = "close")
+  @Override
+  public void close() {
+    try {
+      classLoader.close();
+    } catch (IOException e) {
+      checker.message(Diagnostic.Kind.NOTE, "Failed to close AnnotationClassLoader");
+    }
+  }
+
   /**
    * Scans all classpaths and returns the resource URL to the jar which contains the checker's qual
    * package, or the qual package directory if it exists, or null if no jar or directory contains
@@ -177,7 +200,7 @@ public class AnnotationClassLoader {
    * @return a URL to the jar that contains the qual package, or to the qual package's directory, or
    *     null if no jar or directory contains the qual package
    */
-  private final @Nullable URL getURLFromClasspaths() {
+  private @Nullable URL getURLFromClasspaths() {
     // TODO: This method could probably be replaced with
     // io.github.classgraph.ClassGraph#getClasspathURIs()
 
@@ -191,19 +214,19 @@ public class AnnotationClassLoader {
     Set<String> paths = getClasspaths();
 
     // In checkers, there will be a resource URL for the qual directory. But when called in the
-    // framework (eg GeneralAnnotatedTypeFactory), there won't be a resourceURL since there isn't a
-    // qual directory.
+    // framework (eg GeneralAnnotatedTypeFactory), there won't be a resourceURL since there
+    // isn't a qual directory.
 
-    // Each path from the set of classpaths will be checked to see if it contains the qual directory
-    // of a checker, if so, the first directory or jar that contains the package will be used as the
-    // source for loading classes from the qual package.
+    // Each path from the set of classpaths will be checked to see if it contains the qual
+    // directory of a checker, if so, the first directory or jar that contains the package will
+    // be used as the source for loading classes from the qual package.
 
-    // If either a directory or a jar contains the package, resourceURL will be updated to refer to
-    // that source, otherwise resourceURL remains as null.
+    // If either a directory or a jar contains the package, resourceURL will be updated to refer
+    // to that source, otherwise resourceURL remains as null.
 
     // If both a jar and a directory contain the qual package, then the order of the jar and the
-    // directory in the command line option(s) or environment variables will decide which one gets
-    // examined first.
+    // directory in the command line option(s) or environment variables will decide which one
+    // gets examined first.
     for (String path : paths) {
       // see if the current classpath segment is a jar or a directory
       if (path.endsWith(JAR_SUFFIX)) {
@@ -244,16 +267,16 @@ public class AnnotationClassLoader {
    * @param url a URL referring to either a jar or a directory
    * @return true if the jar or the directory contains the qual package, false otherwise
    */
-  private final boolean containsPackage(final URL url) {
+  private boolean containsPackage(URL url) {
     // see whether the resource URL has a protocol of jar or file
     if (url.getProtocol().equals("jar")) {
       // try to open up the jar file
       try {
         JarURLConnection connection = (JarURLConnection) url.openConnection();
-        JarFile jarFile = connection.getJarFile();
-
-        // check to see if the jar file contains the package
-        return checkJarForPackage(jarFile);
+        try (JarFile jarFile = connection.getJarFile()) {
+          // check to see if the jar file contains the package
+          return checkJarForPackage(jarFile);
+        }
       } catch (IOException e) {
         // do nothing for missing or un-openable Jar files
       }
@@ -275,17 +298,17 @@ public class AnnotationClassLoader {
    * @return true if the jar file contains the qual package, false otherwise
    */
   @SuppressWarnings("JdkObsolete")
-  private final boolean checkJarForPackage(final JarFile jar) {
+  private boolean checkJarForPackage(JarFile jar) {
     Enumeration<JarEntry> jarEntries = jar.entries();
 
     // loop through the entries in the jar
     while (jarEntries.hasMoreElements()) {
       JarEntry je = jarEntries.nextElement();
 
-      // Each entry is the fully qualified path and file name to a particular artifact in the jar
-      // file (eg a class file).
-      // If the jar has the package, one of the entry's name will begin with the package name in
-      // slash notation.
+      // Each entry is the fully qualified path and file name to a particular artifact in the
+      // jar file (eg a class file).
+      // If the jar has the package, one of the entry's name will begin with the package name
+      // in slash notation.
       String entryName = je.getName();
       if (entryName.startsWith(packageNameWithSlashes + SLASH)) {
         return true;
@@ -316,7 +339,7 @@ public class AnnotationClassLoader {
    *     name
    * @return true if the qual package exists within the root directory, false otherwise
    */
-  private final boolean checkDirForPackage(final File currentDir, final Iterator<String> pkgNames) {
+  private boolean checkDirForPackage(File currentDir, Iterator<String> pkgNames) {
     // if the iterator has no more package name segments, then we've found
     // the qual directory of interest
     if (!pkgNames.hasNext()) {
@@ -352,7 +375,7 @@ public class AnnotationClassLoader {
    * @param absolutePathToDirectory an absolute path to a directory
    * @return a URL reference to the directory, or null if the URL is malformed
    */
-  private final @Nullable URL getDirectoryURL(final String absolutePathToDirectory) {
+  private @Nullable URL getDirectoryURL(String absolutePathToDirectory) {
     URL directoryURL = null;
 
     try {
@@ -360,7 +383,8 @@ public class AnnotationClassLoader {
     } catch (MalformedURLException e) {
       processingEnv
           .getMessager()
-          .printMessage(Kind.NOTE, "Directory URL " + absolutePathToDirectory + " is malformed");
+          .printMessage(
+              Diagnostic.Kind.NOTE, "Directory URL " + absolutePathToDirectory + " is malformed");
     }
 
     return directoryURL;
@@ -372,15 +396,15 @@ public class AnnotationClassLoader {
    * @param absolutePathToJarFile an absolute path to a jar file
    * @return a URL reference to the jar file, or null if the URL is malformed
    */
-  private final @Nullable URL getJarURL(final String absolutePathToJarFile) {
+  private @Nullable URL getJarURL(String absolutePathToJarFile) {
     URL jarURL = null;
 
     try {
-      jarURL = new URL("jar:file:" + absolutePathToJarFile + "!/");
-    } catch (MalformedURLException e) {
+      jarURL = new URI("jar:file:" + absolutePathToJarFile + "!/").toURL();
+    } catch (URISyntaxException | MalformedURLException e) {
       processingEnv
           .getMessager()
-          .printMessage(Kind.NOTE, "Jar URL " + absolutePathToJarFile + " is malformed");
+          .printMessage(Diagnostic.Kind.NOTE, "Jar URL " + absolutePathToJarFile + " is malformed");
     }
 
     return jarURL;
@@ -403,17 +427,14 @@ public class AnnotationClassLoader {
    *
    * @return an immutable linked hashset of the classpaths
    */
-  private final Set<String> getClasspaths() {
+  private Set<String> getClasspaths() {
     Set<String> paths = new LinkedHashSet<>();
 
     // add all extension paths
-    String extdirs = System.getProperty("java.ext.dirs");
-    if (extdirs != null && !extdirs.isEmpty()) {
-      paths.addAll(Arrays.asList(extdirs.split(File.pathSeparator)));
-    }
+    paths.addAll(SystemUtil.getPathsProperty("java.ext.dirs"));
 
     // add all paths in CLASSPATH, -cp, and -classpath
-    paths.addAll(Arrays.asList(System.getProperty("java.class.path").split(File.pathSeparator)));
+    paths.addAll(SystemUtil.getPathsProperty("java.class.path"));
 
     // add all paths that are examined by the classloader
     if (classLoader != null) {
@@ -433,7 +454,7 @@ public class AnnotationClassLoader {
    * @return the classloader used to load the checker class, or the system classloader, or null if
    *     both are unavailable
    */
-  private final @Nullable URLClassLoader getClassLoader() {
+  private @Nullable URLClassLoader getClassLoader() {
     ClassLoader result = InternalUtils.getClassLoaderForClass(checker.getClass());
     if (result instanceof URLClassLoader) {
       return (@Nullable URLClassLoader) result;
@@ -447,34 +468,32 @@ public class AnnotationClassLoader {
   @SuppressWarnings("unused") // for debugging
   protected final void printPaths() {
     // all paths in Xbootclasspath
-    String[] bootclassPaths = System.getProperty("sun.boot.class.path").split(File.pathSeparator);
-    processingEnv.getMessager().printMessage(Kind.NOTE, "bootclass path:");
-    for (String path : bootclassPaths) {
-      processingEnv.getMessager().printMessage(Kind.NOTE, "\t" + path);
+    processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "bootclass path:");
+    for (String path : SystemUtil.getPathsProperty("sun.boot.class.path")) {
+      processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "\t" + path);
     }
 
     // all extension paths
-    String[] extensionDirs = System.getProperty("java.ext.dirs").split(File.pathSeparator);
-    processingEnv.getMessager().printMessage(Kind.NOTE, "extension dirs:");
-    for (String path : extensionDirs) {
-      processingEnv.getMessager().printMessage(Kind.NOTE, "\t" + path);
+    processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "extension dirs:");
+    for (String path : SystemUtil.getPathsProperty("java.ext.dirs")) {
+      processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "\t" + path);
     }
 
     // all paths in CLASSPATH, -cp, and -classpath
-    processingEnv.getMessager().printMessage(Kind.NOTE, "java.class.path property:");
-    for (String path : System.getProperty("java.class.path").split(File.pathSeparator)) {
-      processingEnv.getMessager().printMessage(Kind.NOTE, "\t" + path);
+    processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "java.class.path property:");
+    for (String path : SystemUtil.getPathsProperty("java.class.path")) {
+      processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "\t" + path);
     }
 
     // add all paths that are examined by the classloader
-    processingEnv.getMessager().printMessage(Kind.NOTE, "classloader examined paths:");
+    processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "classloader examined paths:");
     if (classLoader != null) {
       URL[] urls = classLoader.getURLs();
       for (int i = 0; i < urls.length; i++) {
-        processingEnv.getMessager().printMessage(Kind.NOTE, "\t" + urls[i].getFile());
+        processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "\t" + urls[i].getFile());
       }
     } else {
-      processingEnv.getMessager().printMessage(Kind.NOTE, "classloader unavailable");
+      processingEnv.getMessager().printMessage(Diagnostic.Kind.NOTE, "classloader unavailable");
     }
   }
 
@@ -487,16 +506,17 @@ public class AnnotationClassLoader {
     Set<@BinaryName String> annotationNames;
     // see whether the resource URL has a protocol of jar or file
     if (resourceURL != null && resourceURL.getProtocol().contentEquals("jar")) {
-      // if the checker class file is contained within a jar, then the resource URL for the qual
-      // directory will have the protocol "jar". This means the whole checker is loaded as a jar
-      // file.
+      // if the checker class file is contained within a jar, then the resource URL for the
+      // qual directory will have the protocol "jar". This means the whole checker is loaded
+      // as a jar file.
 
       JarURLConnection connection;
       // create a connection to the jar file
       try {
         connection = (JarURLConnection) resourceURL.openConnection();
 
-        // disable caching / connection sharing of the low level URLConnection to the Jar file
+        // disable caching / connection sharing of the low level URLConnection to the Jar
+        // file
         connection.setDefaultUseCaches(false);
         connection.setUseCaches(false);
 
@@ -518,9 +538,10 @@ public class AnnotationClassLoader {
       }
 
     } else if (resourceURL != null && resourceURL.getProtocol().contentEquals("file")) {
-      // If the checker class file is found within the file system itself within some directory
-      // (usually development build directories), then process the package as a file directory in
-      // the file system and load the annotations contained in the qual directory.
+      // If the checker class file is found within the file system itself within some
+      // directory (usually development build directories), then process the package as a file
+      // directory in the file system and load the annotations contained in the qual
+      // directory.
 
       // open up the directory
       File packageDir = new File(resourceURL.getFile());
@@ -566,7 +587,7 @@ public class AnnotationClassLoader {
    * @return a set of fully qualified class names of the annotations
    */
   @SuppressWarnings("JdkObsolete")
-  private final Set<@BinaryName String> getBundledAnnotationNamesFromJar(final JarFile jar) {
+  private Set<@BinaryName String> getBundledAnnotationNamesFromJar(JarFile jar) {
     Set<@BinaryName String> annos = new LinkedHashSet<>();
 
     // get an enumeration iterator for all the content entries in the jar file
@@ -604,7 +625,7 @@ public class AnnotationClassLoader {
    *     {@link #isSupportedAnnotationClass(Class)}
    */
   public final @Nullable Class<? extends Annotation> loadExternalAnnotationClass(
-      final @BinaryName String annoName) {
+      @BinaryName String annoName) {
     return loadAnnotationClass(annoName, true);
   }
 
@@ -616,7 +637,7 @@ public class AnnotationClassLoader {
    * @return a set of annotation classes
    */
   public final Set<Class<? extends Annotation>> loadExternalAnnotationClassesFromDirectory(
-      final String dirName) {
+      String dirName) {
     File rootDirectory = new File(dirName);
     Set<@BinaryName String> annoNames =
         getAnnotationNamesFromDirectory(null, rootDirectory, rootDirectory);
@@ -637,10 +658,10 @@ public class AnnotationClassLoader {
    *     its sub-directories
    */
   @SuppressWarnings("signature") // TODO: reduce use of string manipulation
-  private final Set<@BinaryName String> getAnnotationNamesFromDirectory(
-      final @Nullable @DotSeparatedIdentifiers String packageName,
-      final File rootDirectory,
-      final File currentDirectory) {
+  private Set<@BinaryName String> getAnnotationNamesFromDirectory(
+      @Nullable @DotSeparatedIdentifiers String packageName,
+      File rootDirectory,
+      File currentDirectory) {
     Set<@BinaryName String> results = new LinkedHashSet<>();
 
     // Full path to root directory
@@ -648,14 +669,10 @@ public class AnnotationClassLoader {
 
     // check every file and directory within the current directory
     File[] directoryContents = currentDirectory.listFiles();
-    Arrays.sort(
-        directoryContents,
-        new Comparator<File>() {
-          @Override
-          public int compare(File o1, File o2) {
-            return o1.getName().compareTo(o2.getName());
-          }
-        });
+    if (directoryContents == null) {
+      throw new UserError("Directory does not exist: %s", currentDirectory);
+    }
+    Arrays.sort(directoryContents, Comparator.comparing(File::getName));
     for (File file : directoryContents) {
       if (file.isFile()) {
         // TODO: simplify all this string manipulation.
@@ -712,7 +729,7 @@ public class AnnotationClassLoader {
    *     is not supported by a checker, null is returned.
    */
   protected final @Nullable Class<? extends Annotation> loadAnnotationClass(
-      final @BinaryName String className, boolean issueError) {
+      @BinaryName String className, boolean issueError) {
 
     // load the class
     Class<?> cls = null;
@@ -761,7 +778,8 @@ public class AnnotationClassLoader {
               + " is not a type annotation."
               + " Check its @Target meta-annotation.");
     } else {
-      // issueError is set to false for loading the qual directory or any external directories.
+      // issueError is set to false for loading the qual directory or any external
+      // directories.
       // We don't issue any errors since there may be meta-annotations or non-type annotations
       // in such directories.
       return null;
@@ -776,7 +794,7 @@ public class AnnotationClassLoader {
    * @see #loadAnnotationClass(String, boolean)
    */
   protected final Set<Class<? extends Annotation>> loadAnnotationClasses(
-      final @Nullable Set<@BinaryName String> annoNames) {
+      @Nullable Set<@BinaryName String> annoNames) {
     Set<Class<? extends Annotation>> loadedClasses = new LinkedHashSet<>();
 
     if (annoNames != null && !annoNames.isEmpty()) {
@@ -803,8 +821,7 @@ public class AnnotationClassLoader {
    * @param annoClass an annotation class
    * @return true if the annotation is well defined, false if it isn't
    */
-  protected boolean hasWellDefinedTargetMetaAnnotation(
-      final Class<? extends Annotation> annoClass) {
+  protected boolean hasWellDefinedTargetMetaAnnotation(Class<? extends Annotation> annoClass) {
     return annoClass.getAnnotation(Target.class) != null
         && AnnotationUtils.hasTypeQualifierElementTypes(
             annoClass.getAnnotation(Target.class).value(), annoClass);
@@ -822,7 +839,7 @@ public class AnnotationClassLoader {
    * @param annoClass an annotation class
    * @return true if the annotation is supported, false if it isn't
    */
-  protected boolean isSupportedAnnotationClass(final Class<? extends Annotation> annoClass) {
+  protected boolean isSupportedAnnotationClass(Class<? extends Annotation> annoClass) {
     return true;
   }
 }
