@@ -1,14 +1,17 @@
 package org.checkerframework.framework.type;
 
+import java.util.Collection;
+import java.util.Collections;
 import java.util.List;
-import java.util.Set;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.type.TypeKind;
+import javax.lang.model.type.TypeMirror;
 import javax.lang.model.util.Types;
 import org.checkerframework.common.basetype.BaseTypeChecker;
+import org.checkerframework.framework.qual.AnnotatedFor;
 import org.checkerframework.framework.qual.Covariant;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedArrayType;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedDeclaredType;
@@ -21,6 +24,7 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedWildcard
 import org.checkerframework.framework.type.visitor.AbstractAtmComboVisitor;
 import org.checkerframework.framework.util.AnnotatedTypes;
 import org.checkerframework.framework.util.AtmCombo;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.TreeUtils;
@@ -33,7 +37,7 @@ import org.checkerframework.javacutil.TypesUtils;
  * options passed to DefaultTypeHierarchy.
  *
  * <p>Subtyping rules of the JLS can be found in <a
- * href="https://docs.oracle.com/javase/specs/jls/se11/html/jls-4.html#jls-4.10">section 4.10,
+ * href="https://docs.oracle.com/javase/specs/jls/se17/html/jls-4.html#jls-4.10">section 4.10,
  * "Subtyping"</a>.
  *
  * <p>Note: The visit methods of this class must be public but it is intended to be used through a
@@ -52,12 +56,14 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   protected final BaseTypeChecker checker;
 
   /** The qualifier hierarchy that is associated with this. */
-  protected final QualifierHierarchy qualifierHierarchy;
+  protected final QualifierHierarchy qualHierarchy;
+
   /** The equality comparer. */
   protected final StructuralEqualityComparer equalityComparer;
 
   /** Whether to ignore raw types. */
   protected final boolean ignoreRawTypes;
+
   /** Whether to make array subtyping invariant with respect to array component types. */
   protected final boolean invariantArrayComponents;
 
@@ -81,18 +87,18 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * Creates a DefaultTypeHierarchy.
    *
    * @param checker the type-checker that is associated with this
-   * @param qualifierHierarchy the qualifier hierarchy that is associated with this
+   * @param qualHierarchy the qualifier hierarchy that is associated with this
    * @param ignoreRawTypes whether to ignore raw types
    * @param invariantArrayComponents whether to make array subtyping invariant with respect to array
    *     component types
    */
   public DefaultTypeHierarchy(
-      final BaseTypeChecker checker,
-      final QualifierHierarchy qualifierHierarchy,
+      BaseTypeChecker checker,
+      QualifierHierarchy qualHierarchy,
       boolean ignoreRawTypes,
       boolean invariantArrayComponents) {
     this.checker = checker;
-    this.qualifierHierarchy = qualifierHierarchy;
+    this.qualHierarchy = qualHierarchy;
     this.isSubtypeVisitHistory = new SubtypeVisitHistory();
     this.areEqualVisitHistory = new StructuralEqualityVisitHistory();
     this.equalityComparer = createEqualityComparer();
@@ -126,8 +132,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * @return true if subtype is a subtype of supertype or equal to it
    */
   @Override
-  public boolean isSubtype(final AnnotatedTypeMirror subtype, final AnnotatedTypeMirror supertype) {
-    for (final AnnotationMirror top : qualifierHierarchy.getTopAnnotations()) {
+  public boolean isSubtype(AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype) {
+    for (AnnotationMirror top : qualHierarchy.getTopAnnotations()) {
       if (!isSubtype(subtype, supertype, top)) {
         return false;
       }
@@ -136,6 +142,106 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
     return true;
   }
 
+  /** A set of annotations and a {@link TypeMirror}. */
+  @AnnotatedFor("nullness")
+  private static class ShallowType {
+
+    /** A set of annotations. */
+    AnnotationMirrorSet annos;
+
+    /** A TypeMirror. */
+    TypeMirror typeMirror;
+
+    /**
+     * Creates a {@code ShallowType}.
+     *
+     * @param annos a set of annotations
+     * @param typeMirror a type mirror
+     */
+    private ShallowType(AnnotationMirrorSet annos, TypeMirror typeMirror) {
+      this.annos = annos;
+      this.typeMirror = typeMirror;
+    }
+
+    /**
+     * Creates a {@code ShallowType} from {@code type}: the annotations are the effective
+     * annotations on {@code type} and the type mirror is the underlying type of {@code type}.
+     *
+     * @param type an annotated type to convert to a {@code ShallowType}
+     * @return a shallow type created from {@code type}
+     */
+    @SuppressWarnings("nullness") // AnnotatedTypeMirror isn't annotated for nullness.
+    public static ShallowType create(AnnotatedTypeMirror type) {
+      AnnotatedTypeMirror erasedType = type.getErased();
+      TypeMirror typeMirror =
+          erasedType.getKind() == type.getKind()
+              ? type.getUnderlyingType()
+              : erasedType.getUnderlyingType();
+      // The effective annotations are the primary annotations on the erased type.
+      return new ShallowType(erasedType.getPrimaryAnnotations(), typeMirror);
+    }
+  }
+
+  @Override
+  public boolean isSubtypeShallowEffective(
+      AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype) {
+    ShallowType subShallowType = ShallowType.create(subtype);
+    ShallowType superShallowType = ShallowType.create(supertype);
+    return qualHierarchy.isSubtypeShallow(
+        subShallowType.annos,
+        subShallowType.typeMirror,
+        superShallowType.annos,
+        superShallowType.typeMirror);
+  }
+
+  @Override
+  public boolean isSubtypeShallowEffective(
+      AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype, AnnotationMirror hierarchy) {
+    ShallowType subShallowType = ShallowType.create(subtype);
+    ShallowType superShallowType = ShallowType.create(supertype);
+    return qualHierarchy.isSubtypeShallow(
+        qualHierarchy.findAnnotationInSameHierarchy(subShallowType.annos, hierarchy),
+        subShallowType.typeMirror,
+        qualHierarchy.findAnnotationInSameHierarchy(superShallowType.annos, hierarchy),
+        superShallowType.typeMirror);
+  }
+
+  @Override
+  public boolean isSubtypeShallowEffective(
+      AnnotatedTypeMirror subtype, Collection<? extends AnnotationMirror> superQualifiers) {
+    ShallowType subShallowType = ShallowType.create(subtype);
+    return qualHierarchy.isSubtypeShallow(
+        subShallowType.annos, superQualifiers, subShallowType.typeMirror);
+  }
+
+  @Override
+  public boolean isSubtypeShallowEffective(
+      Collection<? extends AnnotationMirror> subQualifiers, AnnotatedTypeMirror supertype) {
+    ShallowType superShallowType = ShallowType.create(supertype);
+    return qualHierarchy.isSubtypeShallow(
+        subQualifiers, superShallowType.annos, superShallowType.typeMirror);
+  }
+
+  @Override
+  public boolean isSubtypeShallowEffective(
+      AnnotatedTypeMirror subtype, AnnotationMirror superQualifier) {
+    ShallowType subShallowType = ShallowType.create(subtype);
+    return qualHierarchy.isSubtypeShallow(
+        qualHierarchy.findAnnotationInSameHierarchy(subShallowType.annos, superQualifier),
+        superQualifier,
+        subShallowType.typeMirror);
+  }
+
+  @Override
+  public boolean isSubtypeShallowEffective(
+      AnnotationMirror subQualifier, AnnotatedTypeMirror supertype) {
+    ShallowType superShallowType = ShallowType.create(supertype);
+    return qualHierarchy.isSubtypeShallow(
+        subQualifier,
+        qualHierarchy.findAnnotationInSameHierarchy(superShallowType.annos, subQualifier),
+        superShallowType.typeMirror);
+  }
+
   /**
    * Returns true if {@code subtype <: supertype}, but only for the hierarchy of which {@code top}
    * is the top.
@@ -143,13 +249,11 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * @param subtype expected subtype
    * @param supertype expected supertype
    * @param top the top of the hierarchy for which we want to make a comparison
-   * @return true if {@code subtype} is a subtype of, or equal to, {@code supertype} in the
-   *     qualifier hierarchy whose top is {@code top}
+   * @return true if {@code subtype} is a subtype of or equal to {@code supertype}, in the qualifier
+   *     hierarchy whose top is {@code top}
    */
   protected boolean isSubtype(
-      final AnnotatedTypeMirror subtype,
-      final AnnotatedTypeMirror supertype,
-      final AnnotationMirror top) {
+      AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype, AnnotationMirror top) {
     assert top != null;
     currentTop = top;
     return AtmCombo.accept(subtype, supertype, null, this);
@@ -161,13 +265,11 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * @return error message for the case when two types shouldn't be compared
    */
   @Override
-  protected String defaultErrorMessage(
-      final AnnotatedTypeMirror subtype, final AnnotatedTypeMirror supertype, final Void p) {
-    return "Incomparable types ("
-        + subtype
-        + ", "
-        + supertype
-        + ") visitHistory = "
+  public String defaultErrorMessage(
+      AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype, Void p) {
+    return super.defaultErrorMessage(subtype, supertype, p)
+        + System.lineSeparator()
+        + "  visitHistory = "
         + isSubtypeVisitHistory;
   }
 
@@ -181,16 +283,19 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    *     for the current top.
    */
   protected boolean isPrimarySubtype(AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype) {
-    final AnnotationMirror subtypeAnno = subtype.getAnnotationInHierarchy(currentTop);
-    final AnnotationMirror supertypeAnno = supertype.getAnnotationInHierarchy(currentTop);
+    TypeMirror subTM = subtype.getUnderlyingType();
+    TypeMirror superTM = supertype.getUnderlyingType();
+
+    AnnotationMirror subtypeAnno = subtype.getPrimaryAnnotationInHierarchy(currentTop);
+    AnnotationMirror supertypeAnno = supertype.getPrimaryAnnotationInHierarchy(currentTop);
     if (checker.getTypeFactory().hasQualifierParameterInHierarchy(supertype, currentTop)
         && checker.getTypeFactory().hasQualifierParameterInHierarchy(subtype, currentTop)) {
       // If the types have a class qualifier parameter, the qualifiers must be equivalent.
-      return qualifierHierarchy.isSubtype(subtypeAnno, supertypeAnno)
-          && qualifierHierarchy.isSubtype(supertypeAnno, subtypeAnno);
+      return qualHierarchy.isSubtypeShallow(subtypeAnno, subTM, supertypeAnno, superTM)
+          && qualHierarchy.isSubtypeShallow(supertypeAnno, superTM, subtypeAnno, subTM);
     }
 
-    return qualifierHierarchy.isSubtype(subtypeAnno, supertypeAnno);
+    return qualHierarchy.isSubtypeShallow(subtypeAnno, subTM, supertypeAnno, superTM);
   }
 
   /**
@@ -201,8 +306,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * @param supertype a type that may be a supertype
    * @return true if subtype {@literal <:} supertype
    */
-  protected boolean isSubtypeCaching(
-      final AnnotatedTypeMirror subtype, final AnnotatedTypeMirror supertype) {
+  protected boolean isSubtypeCaching(AnnotatedTypeMirror subtype, AnnotatedTypeMirror supertype) {
     if (isSubtypeVisitHistory.contains(subtype, supertype, currentTop)) {
       // visitHistory only contains pairs in a subtype relationship.
       return true;
@@ -221,8 +325,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * mirror of {@code supertype}.
    */
   protected boolean areAllSubtypes(
-      final Iterable<? extends AnnotatedTypeMirror> subtypes, final AnnotatedTypeMirror supertype) {
-    for (final AnnotatedTypeMirror subtype : subtypes) {
+      Iterable<? extends AnnotatedTypeMirror> subtypes, AnnotatedTypeMirror supertype) {
+    for (AnnotatedTypeMirror subtype : subtypes) {
       if (!isSubtype(subtype, supertype, currentTop)) {
         return false;
       }
@@ -231,8 +335,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
     return true;
   }
 
-  protected boolean areEqualInHierarchy(
-      final AnnotatedTypeMirror type1, final AnnotatedTypeMirror type2) {
+  protected boolean areEqualInHierarchy(AnnotatedTypeMirror type1, AnnotatedTypeMirror type2) {
     return equalityComparer.areEqualInHierarchy(type1, type2, currentTop);
   }
 
@@ -241,11 +344,11 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    * by {@code outside} is a superset of, or equal to, the set of types denoted by {@code inside}.
    *
    * <p>Containment is described in <a
-   * href="https://docs.oracle.com/javase/specs/jls/se11/html/jls-4.html#jls-4.5.1">JLS section
+   * href="https://docs.oracle.com/javase/specs/jls/se17/html/jls-4.html#jls-4.5.1">JLS section
    * 4.5.1 "Type Arguments of Parameterized Types"</a>.
    *
    * <p>As described in <a
-   * href=https://docs.oracle.com/javase/specs/jls/se11/html/jls-4.html#jls-4.10.2>JLS section
+   * href=https://docs.oracle.com/javase/specs/jls/se17/html/jls-4.html#jls-4.10.2>JLS section
    * 4.10.2 Subtyping among Class and Interface Types</a>, a declared type S is considered a
    * supertype of another declared type T only if all of S's type arguments "contain" the
    * corresponding type arguments of the subtype T.
@@ -265,7 +368,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       return previousResult;
     }
 
-    if (shouldIgnoreUninferredTypeArgs(inside) || shouldIgnoreUninferredTypeArgs(outside)) {
+    if (shouldIgnoreRawTypeArgs(inside) || shouldIgnoreRawTypeArgs(outside)) {
       areEqualVisitHistory.put(inside, outside, currentTop, true);
       return true;
     }
@@ -284,12 +387,10 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
               canBeCovariant);
       areEqualVisitHistory.put(inside, outside, currentTop, result);
       return result;
-    }
-    if ((TypesUtils.isCapturedTypeVariable(outside.getUnderlyingType())
-        && !TypesUtils.isCapturedTypeVariable(inside.getUnderlyingType()))) {
-      // TODO: This branch should be removed after #979 is fixed.
-      // This workaround is only needed when outside is a captured type variable,
-      // but inside is not.
+    } else if (TypesUtils.isCapturedTypeVariable(outside.getUnderlyingType())) {
+      // Sometimes the wildcard has been captured too early, so treat the captured type
+      // variable as wildcard.
+      // This is all cases except bullet 6, "T <= T".
       AnnotatedTypeVariable outsideTypeVar = (AnnotatedTypeVariable) outside;
 
       // Add a placeholder in case of recursion, to prevent infinite regress.
@@ -300,15 +401,31 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
               outsideTypeVar.getLowerBound(),
               outsideTypeVar.getUpperBound(),
               canBeCovariant);
-
       areEqualVisitHistory.put(inside, outside, currentTop, result);
-      return result;
+      if (result) {
+        return true;
+      }
+      areEqualVisitHistory.remove(inside, outsideTypeVar, currentTop);
     }
+
     // The remainder of the method is bullet 6, "T <= T".
     if (canBeCovariant) {
       return isSubtype(inside, outside, currentTop);
     }
-    return areEqualInHierarchy(inside, outside);
+
+    try {
+      return areEqualInHierarchy(inside, outside);
+    } catch (Exception e) {
+      // Ignore exception and try capturing.
+      // See https://github.com/typetools/checker-framework/issues/6867.
+      // https://bugs.openjdk.org/browse/JDK-8054309
+    }
+    AnnotatedTypeMirror capturedOutside = outside.atypeFactory.applyCaptureConversion(outside);
+    previousResult = areEqualVisitHistory.get(inside, capturedOutside, currentTop);
+    if (previousResult != null) {
+      return previousResult;
+    }
+    return areEqualInHierarchy(inside, capturedOutside);
   }
 
   /**
@@ -339,11 +456,11 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
           return isSubtype(outsideLower, inside);
         }
       }
-      // If inside is a wildcard, then isSubtype(outsideLower, inside) calls isSubtype(outsideLower,
-      // inside.getLowerBound()) and isSubtype(inside, outsideUpper) calls
-      // isSubtype(inside.getUpperBound(), outsideUpper). This is slightly different from the
-      // algorithm in the JLS.  Only one of the Java type bounds can be specified, but there can be
-      // annotations on both the upper and lower bound of a wildcard.
+      // If inside is a wildcard, then isSubtype(outsideLower, inside) calls
+      // isSubtype(outsideLower, inside.getLowerBound()) and isSubtype(inside, outsideUpper)
+      // calls isSubtype(inside.getUpperBound(), outsideUpper). This is slightly different
+      // from the algorithm in the JLS.  Only one of the Java type bounds can be specified,
+      // but there can be annotations on both the upper and lower bound of a wildcard.
       return isSubtype(outsideLower, inside) && isSubtype(inside, outsideUpper);
     } catch (Throwable ex) {
       // Work around:
@@ -356,17 +473,15 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   }
 
   /**
-   * Returns true if {@code type} is an uninferred type argument and if the checker should not issue
-   * warnings about uninferred type arguments.
+   * Returns true if {@code type} is a type argument from a raw type and if the checker should not
+   * issue warnings about such type arguments.
    *
    * @param type type to check
-   * @return true if {@code type} is an uninferred type argument and if the checker should not issue
-   *     warnings about uninferred type arguments
+   * @return true if {@code type} is a type argument from a raw type and if the checker should not
+   *     issue warnings about such type arguments.
    */
-  private boolean shouldIgnoreUninferredTypeArgs(AnnotatedTypeMirror type) {
-    return type.atypeFactory.ignoreUninferredTypeArguments
-        && type.getKind() == TypeKind.WILDCARD
-        && ((AnnotatedWildcardType) type).isUninferredTypeArgument();
+  private boolean shouldIgnoreRawTypeArgs(AnnotatedTypeMirror type) {
+    return this.ignoreRawTypes && AnnotatedTypes.isTypeArgOfRawType(type);
   }
 
   // ------------------------------------------------------------------------
@@ -431,20 +546,12 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
     if (!isPrimarySubtype(subtype, supertype)) {
       return false;
     }
-    AnnotatedTypeFactory factory = subtype.atypeFactory;
-    if (factory.ignoreUninferredTypeArguments
-        && (factory.containsUninferredTypeArguments(subtype)
-            || factory.containsUninferredTypeArguments(supertype))) {
-      // Calling castedAsSuper may cause the uninferredTypeArguments to be lost. So, just
-      // return true here.
-      return true;
-    }
 
     if (isSubtypeVisitHistory.contains(subtype, supertype, currentTop)) {
       return true;
     }
 
-    final boolean result =
+    boolean result =
         visitTypeArgs(
             subtype, supertype, subtype.isUnderlyingTypeRaw(), supertype.isUnderlyingTypeRaw());
     isSubtypeVisitHistory.put(subtype, supertype, currentTop, result);
@@ -466,8 +573,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   protected boolean visitTypeArgs(
       AnnotatedDeclaredType subtype,
       AnnotatedDeclaredType supertype,
-      final boolean subtypeRaw,
-      final boolean supertypeRaw) {
+      boolean subtypeRaw,
+      boolean supertypeRaw) {
     AnnotatedTypeFactory typeFactory = subtype.atypeFactory;
 
     // JLS 11: 4.10.2. Subtyping among Class and Interface Types
@@ -479,8 +586,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       return true;
     }
 
-    final List<? extends AnnotatedTypeMirror> subtypeTypeArgs = subtypeAsSuper.getTypeArguments();
-    final List<? extends AnnotatedTypeMirror> supertypeTypeArgs = supertype.getTypeArguments();
+    List<? extends AnnotatedTypeMirror> subtypeTypeArgs = subtypeAsSuper.getTypeArguments();
+    List<? extends AnnotatedTypeMirror> supertypeTypeArgs = supertype.getTypeArguments();
 
     if (subtypeTypeArgs.size() != supertypeTypeArgs.size()) {
       throw new BugInCF("Type arguments are not the same size: %s %s", subtypeAsSuper, supertype);
@@ -490,14 +597,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       return true;
     }
 
-    final TypeElement supertypeElem = (TypeElement) supertype.getUnderlyingType().asElement();
-    AnnotationMirror covariantAnno = typeFactory.getDeclAnnotation(supertypeElem, Covariant.class);
-
-    List<Integer> covariantArgIndexes =
-        (covariantAnno == null)
-            ? null
-            : AnnotationUtils.getElementValueArray(
-                covariantAnno, covariantValueElement, Integer.class);
+    List<Integer> covariantArgIndexes = getCovariantArgIndexes(supertype);
 
     // JLS 11: 4.10.2. Subtyping among Class and Interface Types
     // 4th paragraph, bullet 2
@@ -508,6 +608,10 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       }
     } catch (Exception e) {
       // Some types need to be captured first, so ignore crashes.
+      for (int i = 0; i < supertypeTypeArgs.size(); i++) {
+        areEqualVisitHistory.remove(
+            subtypeAsSuper.getTypeArguments().get(i), supertypeTypeArgs.get(i), currentTop);
+      }
     }
     // 5th paragraph:
     // Instead of calling isSubtype with the captured type, just check for containment.
@@ -519,6 +623,19 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
         capturedSubtypeAsSuper.getTypeArguments(), supertypeTypeArgs, covariantArgIndexes);
   }
 
+  @Override
+  public List<Integer> getCovariantArgIndexes(AnnotatedDeclaredType type) {
+    TypeElement supertypeElem = (TypeElement) type.getUnderlyingType().asElement();
+    AnnotationMirror covariantAnno =
+        type.atypeFactory.getDeclAnnotation(supertypeElem, Covariant.class);
+    if (covariantAnno == null) {
+      return Collections.emptyList();
+    }
+
+    return AnnotationUtils.getElementValueArray(
+        covariantAnno, covariantValueElement, Integer.class);
+  }
+
   /**
    * Calls {@link #isContainedBy(AnnotatedTypeMirror, AnnotatedTypeMirror, boolean)} on the two
    * lists of type arguments. Returns true if every type argument in {@code supertypeTypeArgs}
@@ -537,7 +654,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
     for (int i = 0; i < supertypeTypeArgs.size(); i++) {
       AnnotatedTypeMirror superTypeArg = supertypeTypeArgs.get(i);
       AnnotatedTypeMirror subTypeArg = subtypeTypeArgs.get(i);
-      boolean covariant = covariantArgIndexes != null && covariantArgIndexes.contains(i);
+      boolean covariant = covariantArgIndexes.contains(i);
       if (!isContainedBy(subTypeArg, superTypeArg, covariant)) {
         return false;
       }
@@ -560,14 +677,15 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   @Override
   public Boolean visitDeclared_Primitive(
       AnnotatedDeclaredType subtype, AnnotatedPrimitiveType supertype, Void p) {
-    // We do an asSuper first because in some cases unboxing implies a more specific annotation
-    // e.g. @UnknownInterned Integer => @Interned int  because primitives are always interned
-    final AnnotatedPrimitiveType subAsSuper =
-        AnnotatedTypes.castedAsSuper(subtype.atypeFactory, subtype, supertype);
-    if (subAsSuper == null) {
-      return isPrimarySubtype(subtype, supertype);
+    AnnotatedTypeMirror unboxedType;
+    try {
+      unboxedType = subtype.atypeFactory.getUnboxedType(subtype);
+    } catch (IllegalArgumentException ex) {
+      throw new BugInCF(
+          "DefaultTypeHierarchy: subtype isn't a boxed type: subtype: %s superType: %s",
+          subtype, supertype);
     }
-    return isPrimarySubtype(subAsSuper, supertype);
+    return isPrimarySubtype(unboxedType, supertype);
   }
 
   @Override
@@ -714,13 +832,18 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   @Override
   public Boolean visitPrimitive_Declared(
       AnnotatedPrimitiveType subtype, AnnotatedDeclaredType supertype, Void p) {
-    // see comment in visitDeclared_Primitive
-    final AnnotatedDeclaredType subAsSuper =
-        AnnotatedTypes.castedAsSuper(subtype.atypeFactory, subtype, supertype);
-    if (subAsSuper == null) {
-      return isPrimarySubtype(subtype, supertype);
+    AnnotatedTypeFactory atypeFactory = subtype.atypeFactory;
+    Types types = atypeFactory.types;
+    AnnotatedPrimitiveType narrowedType = subtype;
+    if (TypesUtils.isBoxedPrimitive(supertype.getUnderlyingType())) {
+      TypeMirror unboxedSuper = types.unboxedType(supertype.getUnderlyingType());
+      if (unboxedSuper.getKind() != subtype.getKind()
+          && TypesUtils.canBeNarrowingPrimitiveConversion(unboxedSuper, types)) {
+        narrowedType = atypeFactory.getNarrowedPrimitive(subtype, unboxedSuper);
+      }
     }
-    return isPrimarySubtype(subAsSuper, supertype);
+    AnnotatedTypeMirror boxedSubtype = atypeFactory.getBoxedType(narrowedType);
+    return isPrimarySubtype(boxedSubtype, supertype);
   }
 
   @Override
@@ -744,8 +867,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   @Override
   public Boolean visitPrimitive_Wildcard(
       AnnotatedPrimitiveType subtype, AnnotatedWildcardType supertype, Void p) {
-    if (supertype.atypeFactory.ignoreUninferredTypeArguments
-        && supertype.isUninferredTypeArgument()) {
+    if (shouldIgnoreRawTypeArgs(supertype)) {
       return true;
     }
     // this can occur when passing a primitive to a method on a raw type (see test
@@ -843,11 +965,15 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   public Boolean visitTypevar_Typevar(
       AnnotatedTypeVariable subtype, AnnotatedTypeVariable supertype, Void p) {
 
+    TypeMirror subTM = subtype.getUnderlyingType();
+    TypeMirror superTM = supertype.getUnderlyingType();
+
     if (AnnotatedTypes.haveSameDeclaration(checker.getTypeUtils(), subtype, supertype)) {
-      // The underlying types of subtype and supertype are uses of the same type parameter, but they
+      // The underlying types of subtype and supertype are uses of the same type parameter,
+      // but they
       // may have different primary annotations.
-      boolean subtypeHasAnno = subtype.getAnnotationInHierarchy(currentTop) != null;
-      boolean supertypeHasAnno = supertype.getAnnotationInHierarchy(currentTop) != null;
+      boolean subtypeHasAnno = subtype.getPrimaryAnnotationInHierarchy(currentTop) != null;
+      boolean supertypeHasAnno = supertype.getPrimaryAnnotationInHierarchy(currentTop) != null;
 
       if (subtypeHasAnno && supertypeHasAnno) {
         // If both have primary annotations then just check the primary annotations
@@ -859,16 +985,18 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
         return areEqualInHierarchy(subtype, supertype);
       } else if (subtypeHasAnno && !supertypeHasAnno) {
         // This is the case "@A T <: T" where T is a type variable.
-        Set<AnnotationMirror> superLBs =
-            AnnotatedTypes.findEffectiveLowerBoundAnnotations(qualifierHierarchy, supertype);
-        AnnotationMirror superLB =
-            qualifierHierarchy.findAnnotationInHierarchy(superLBs, currentTop);
-        return qualifierHierarchy.isSubtype(subtype.getAnnotationInHierarchy(currentTop), superLB);
+        AnnotationMirrorSet superLBs =
+            AnnotatedTypes.findEffectiveLowerBoundAnnotations(qualHierarchy, supertype);
+        AnnotationMirror superLB = qualHierarchy.findAnnotationInHierarchy(superLBs, currentTop);
+        return qualHierarchy.isSubtypeShallow(
+            subtype.getPrimaryAnnotationInHierarchy(currentTop), subTM, superLB, superTM);
       } else if (!subtypeHasAnno && supertypeHasAnno) {
         // This is the case "T <: @A T" where T is a type variable.
-        return qualifierHierarchy.isSubtype(
+        return qualHierarchy.isSubtypeShallow(
             subtype.getEffectiveAnnotationInHierarchy(currentTop),
-            supertype.getAnnotationInHierarchy(currentTop));
+            subTM,
+            supertype.getPrimaryAnnotationInHierarchy(currentTop),
+            superTM);
       }
     }
 
@@ -879,13 +1007,13 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       }
     }
 
-    if (TypesUtils.isCapturedTypeVariable(subtype.getUnderlyingType())
-        && TypesUtils.isCapturedTypeVariable(supertype.getUnderlyingType())) {
-      // This should be removed when 979 is fixed.
+    if (TypesUtils.isCapturedTypeVariable(subTM) && TypesUtils.isCapturedTypeVariable(superTM)) {
       // This case happens when the captured type variables should be the same type, but
       // aren't because type argument inference isn't implemented correctly.
-      return isContainedWithinBounds(
-          subtype, supertype.getLowerBound(), supertype.getUpperBound(), false);
+      if (isContainedWithinBounds(
+          subtype, supertype.getLowerBound(), supertype.getUpperBound(), false)) {
+        return true;
+      }
     }
 
     if (supertype.getLowerBound().getKind() != TypeKind.NULL) {
@@ -919,11 +1047,11 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   @Override
   public Boolean visitWildcard_Declared(
       AnnotatedWildcardType subtype, AnnotatedDeclaredType supertype, Void p) {
-    if (subtype.isUninferredTypeArgument()) {
-      if (subtype.atypeFactory.ignoreUninferredTypeArguments) {
+    if (subtype.isTypeArgOfRawType()) {
+      if (ignoreRawTypes) {
         return true;
       } else if (supertype.getTypeArguments().isEmpty()) {
-        // visitWildcard_Type doesn't check uninferred type arguments, because the
+        // visitWildcard_Type doesn't check type arguments from raw types, because the
         // underlying Java types may not be in the correct relationship.  But, if the
         // declared type does not have type arguments, then checking primary annotations is
         // sufficient.
@@ -931,9 +1059,7 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
         // @Nullable String, then it is safe to return true. However if the supertype is
         // @NullableList<@NonNull String> then it's not possible to decide if it is a
         // subtype of the wildcard.
-        AnnotationMirror subtypeAnno = subtype.getEffectiveAnnotationInHierarchy(currentTop);
-        AnnotationMirror supertypeAnno = supertype.getAnnotationInHierarchy(currentTop);
-        return qualifierHierarchy.isSubtype(subtypeAnno, supertypeAnno);
+        return isSubtypeShallowEffective(subtype, supertype, currentTop);
       }
     }
     return visitWildcard_Type(subtype, supertype);
@@ -948,10 +1074,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
   @Override
   public Boolean visitWildcard_Primitive(
       AnnotatedWildcardType subtype, AnnotatedPrimitiveType supertype, Void p) {
-    if (subtype.isUninferredTypeArgument()) {
-      AnnotationMirror subtypeAnno = subtype.getEffectiveAnnotationInHierarchy(currentTop);
-      AnnotationMirror supertypeAnno = supertype.getAnnotationInHierarchy(currentTop);
-      return qualifierHierarchy.isSubtype(subtypeAnno, supertypeAnno);
+    if (subtype.isTypeArgOfRawType()) {
+      return isSubtypeShallowEffective(subtype, supertype, currentTop);
     }
     return visitWildcard_Type(subtype, supertype);
   }
@@ -1073,12 +1197,14 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       return isSubtypeCaching(subtypeUpperBound, supertype);
     } catch (BugInCF e) {
       if (TypesUtils.isCapturedTypeVariable(subtype.underlyingType)) {
-        // The upper bound of captured type variable may be computed incorrectly by javac. javac
-        // computes the upper bound as a declared type, when it should be an intersection type.
+        // The upper bound of captured type variable may be computed incorrectly by javac.
+        // javac computes the upper bound as a declared type, when it should be an
+        // intersection type.
         // (This is a bug in the GLB algorithm; see
-        // https://bugs.openjdk.java.net/browse/JDK-8039222)
-        // In this case, the upperbound is not a subtype of `supertype` and the Checker Framework
-        // crashes. So catch that crash and just return false.
+        // https://bugs.openjdk.org/browse/JDK-8039222)
+        // In this case, the upperbound is not a subtype of `supertype` and the Checker
+        // Framework crashes. So catch that crash and just return false.
+        // TODO: catch the problem more locally.
         return false;
       }
       throw e;
@@ -1126,9 +1252,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    */
   protected boolean visitType_Wildcard(
       AnnotatedTypeMirror subtype, AnnotatedWildcardType supertype) {
-    if (supertype.isUninferredTypeArgument()) { // TODO: REMOVE WHEN WE FIX TYPE ARG INFERENCE
-      // Can't call isSubtype because underlying Java types won't be subtypes.
-      return supertype.atypeFactory.ignoreUninferredTypeArguments;
+    if (supertype.isTypeArgOfRawType()) {
+      return ignoreRawTypes;
     }
     return isSubtype(subtype, supertype.getSuperBound(), currentTop);
   }
@@ -1142,8 +1267,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
    */
   protected boolean visitWildcard_Type(
       AnnotatedWildcardType subtype, AnnotatedTypeMirror supertype) {
-    if (subtype.isUninferredTypeArgument()) {
-      return subtype.atypeFactory.ignoreUninferredTypeArguments;
+    if (subtype.isTypeArgOfRawType()) {
+      return ignoreRawTypes;
     }
 
     if (supertype.getKind() == TypeKind.WILDCARD) {
@@ -1156,8 +1281,8 @@ public class DefaultTypeHierarchy extends AbstractAtmComboVisitor<Boolean, Void>
       // visitWildcard_Type is called when checking the method call `method(x)`,
       // and also when checking lambdas.
 
-      boolean subtypeHasAnno = subtype.getAnnotationInHierarchy(currentTop) != null;
-      boolean supertypeHasAnno = supertype.getAnnotationInHierarchy(currentTop) != null;
+      boolean subtypeHasAnno = subtype.getPrimaryAnnotationInHierarchy(currentTop) != null;
+      boolean supertypeHasAnno = supertype.getPrimaryAnnotationInHierarchy(currentTop) != null;
 
       if (subtypeHasAnno && supertypeHasAnno) {
         // If both have primary annotations then just check the primary annotations
