@@ -28,8 +28,8 @@ import org.checkerframework.framework.type.QualifierHierarchy;
 import org.checkerframework.framework.type.visitor.EquivalentAtmComboScanner;
 import org.checkerframework.framework.type.visitor.SimpleAnnotatedTypeScanner;
 import org.checkerframework.framework.util.AnnotatedTypes;
-import org.checkerframework.framework.util.AnnotationMirrorMap;
-import org.checkerframework.framework.util.AnnotationMirrorSet;
+import org.checkerframework.javacutil.AnnotationMirrorMap;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypesUtils;
@@ -50,7 +50,7 @@ import org.plumelib.util.CollectionsPlume;
  *   <li>the PolyCollector creates an instantiation
  *   <li>if the instantiation is non-empty: the Replacer does resolution -- that is, it replaces
  *       each occurrence of {@code @Poly*} by the concrete qualifier it maps to in the instantiation
- *   <li>if the instantiation is empty, the Completer replaces each {@code @Poly*} by the top
+ *   <li>if the instantiation is empty, the Completer replaces each {@code @Poly*} by the bottom
  *       qualifier
  * </ul>
  */
@@ -74,7 +74,7 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
   protected final AnnotationMirrorSet topQuals;
 
   /** Determines the instantiations for each polymorphic qualifier. */
-  private PolyCollector collector = new PolyCollector();
+  private final PolyCollector collector = new PolyCollector();
 
   /** Resolves each polymorphic qualifier by replacing it with its instantiation. */
   private final SimpleAnnotatedTypeScanner<Void, AnnotationMirrorMap<AnnotationMirror>> replacer;
@@ -82,6 +82,9 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
   /**
    * Completes a type by removing any unresolved polymorphic qualifiers, replacing them with the
    * bottom qualifiers.
+   *
+   * <p>This is only called when {@code instantiationMapping} is empty. (And that implies that there
+   * are no polymorphic qualifiers on formal parameters??)
    */
   private final SimpleAnnotatedTypeScanner<Void, Void> completer;
 
@@ -90,7 +93,7 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
       new AnnotationMirrorMap<>();
 
   /** The visit method returns true if the passed type has any polymorphic qualifiers. */
-  protected final SimpleAnnotatedTypeScanner<Boolean, Void> polyScanner;
+  protected final SimpleAnnotatedTypeScanner<Boolean, Void> hasPolyScanner;
 
   /**
    * Creates an {@link AbstractQualifierPolymorphism} instance that uses the given checker for
@@ -111,8 +114,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
               for (Map.Entry<AnnotationMirror, AnnotationMirror> entry : polyQuals.entrySet()) {
                 AnnotationMirror poly = entry.getKey();
                 AnnotationMirror top = entry.getValue();
-                if (type.hasAnnotation(poly)) {
-                  type.removeAnnotation(poly);
+                if (type.hasPrimaryAnnotation(poly)) {
+                  type.removePrimaryAnnotation(poly);
                   if (type.getKind() != TypeKind.TYPEVAR && type.getKind() != TypeKind.WILDCARD) {
                     // Do not add qualifiers to type variables and
                     // wildcards
@@ -130,10 +133,10 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
               return null;
             });
 
-    this.polyScanner =
+    this.hasPolyScanner =
         new SimpleAnnotatedTypeScanner<>(
             (type, notused) -> {
-              for (AnnotationMirror a : type.getAnnotations()) {
+              for (AnnotationMirror a : type.getPrimaryAnnotations()) {
                 if (qualHierarchy.isPolymorphicQualifier(a)) {
                   return true;
                 }
@@ -156,15 +159,11 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
     polyInstantiationForQualifierParameter.clear();
   }
 
-  /**
-   * Returns true if {@code type} has any polymorphic qualifiers
-   *
-   * @param type a type that might have polymorphic qualifiers
-   * @return true if {@code type} has any polymorphic qualifiers
-   */
-  protected boolean hasPolymorphicQualifiers(AnnotatedTypeMirror type) {
-    return polyScanner.visit(type);
+  @Override
+  public boolean hasPolymorphicQualifiers(AnnotatedTypeMirror type) {
+    return hasPolyScanner.visit(type);
   }
+
   /**
    * Resolves polymorphism annotations for the given type.
    *
@@ -181,11 +180,11 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
     // method element requires two.
     // See also BaseTypeVisitor.visitMethodInvocation and
     // CFGBuilder.CFGTranslationPhaseOne.visitMethodInvocation.
-    if (TreeUtils.isEnumSuper(tree)) {
+    if (TreeUtils.isEnumSuperCall(tree)) {
       return;
     }
     List<AnnotatedTypeMirror> parameters =
-        AnnotatedTypes.adaptParameters(atypeFactory, type, tree.getArguments());
+        AnnotatedTypes.adaptParameters(atypeFactory, type, tree.getArguments(), tree);
     List<AnnotatedTypeMirror> arguments =
         CollectionsPlume.mapList(atypeFactory::getAnnotatedType, tree.getArguments());
 
@@ -193,7 +192,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
         collector.visit(arguments, parameters);
 
     // For super() and this() method calls, getReceiverType(tree) does not return the correct
-    // type. So, just skip those.  This is consistent with skipping receivers of constructors below.
+    // type. So, just skip those.  This is consistent with skipping receivers of constructors
+    // below.
     if (type.getReceiverType() != null
         && !TreeUtils.isSuperConstructorCall(tree)
         && !TreeUtils.isThisConstructorCall(tree)) {
@@ -203,7 +203,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
               collector.visit(atypeFactory.getReceiverType(tree), type.getReceiverType()));
     }
 
-    if (instantiationMapping != null && !instantiationMapping.isEmpty()) {
+    if ((instantiationMapping != null && !instantiationMapping.isEmpty())
+        || TreeUtils.isCallToVarargsMethodWithZeroVarargsActuals(tree)) {
       replacer.visit(type, instantiationMapping);
     } else {
       completer.visit(type);
@@ -216,8 +217,9 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
     if (polyQuals.isEmpty() || !hasPolymorphicQualifiers(type)) {
       return;
     }
+
     List<AnnotatedTypeMirror> parameters =
-        AnnotatedTypes.adaptParameters(atypeFactory, type, tree.getArguments());
+        AnnotatedTypes.adaptParameters(atypeFactory, type, tree.getArguments(), tree);
     List<AnnotatedTypeMirror> arguments =
         CollectionsPlume.mapList(atypeFactory::getAnnotatedType, tree.getArguments());
 
@@ -251,7 +253,7 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
     AnnotationMirrorMap<AnnotationMirror> matchingMapping = new AnnotationMirrorMap<>();
     polyQuals.forEach(
         (polyAnnotation, topAnno) -> {
-          AnnotationMirror annoOnOwner = owner.getAnnotationInHierarchy(topAnno);
+          AnnotationMirror annoOnOwner = owner.getPrimaryAnnotationInHierarchy(topAnno);
           if (annoOnOwner != null) {
             matchingMapping.put(polyAnnotation, annoOnOwner);
           }
@@ -299,8 +301,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
       }
     }
     // Deal with varargs
-    if (memberReference.isVarArgs() && !functionalInterface.isVarArgs()) {
-      parameters = AnnotatedTypes.expandVarArgsParametersFromTypes(memberReference, args);
+    if (memberReference.isVarargs() && !functionalInterface.isVarargs()) {
+      parameters = AnnotatedTypes.expandVarargsParametersFromTypes(memberReference, args);
     }
 
     instantiationMapping =
@@ -332,8 +334,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
     for (Map.Entry<AnnotationMirror, AnnotationMirror> kv : polyQuals.entrySet()) {
       AnnotationMirror top = kv.getValue();
       AnnotationMirror poly = kv.getKey();
-      if (polyType.hasAnnotation(poly)) {
-        AnnotationMirror typeQual = type.getAnnotationInHierarchy(top);
+      if (polyType.hasPrimaryAnnotation(poly)) {
+        AnnotationMirror typeQual = type.getPrimaryAnnotationInHierarchy(top);
         if (typeQual != null) {
           if (atypeFactory.hasQualifierParameterInHierarchy(type, top)) {
             polyInstantiationForQualifierParameter.put(poly, typeQual);
@@ -366,9 +368,9 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
    *
    * <p>This method is called on all parts of a type.
    *
-   * @param type AnnotatedTypeMirror whose poly annotations are replaced; it is side-effected by
+   * @param type the AnnotatedTypeMirror whose poly annotations are replaced; it is side-effected by
    *     this method
-   * @param replacements mapping from polymorphic annotation to instantiation
+   * @param replacements a mapping from polymorphic annotation to instantiation
    */
   protected abstract void replace(
       AnnotatedTypeMirror type, AnnotationMirrorMap<AnnotationMirror> replacements);
@@ -386,7 +388,7 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
      *
      * <p>Uses reference equality rather than equals because the visitor may visit two types that
      * are structurally equal, but not actually the same. For example, the wildcards in {@code
-     * Pair<?,?>} may be equal, but they both should be visited.
+     * IPair<?,?>} may be equal, but they both should be visited.
      */
     private final Set<AnnotatedTypeMirror> visitedTypes =
         Collections.newSetFromMap(new IdentityHashMap<AnnotatedTypeMirror, Boolean>());
@@ -441,8 +443,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
      * Calls {@link #visit(AnnotatedTypeMirror, AnnotatedTypeMirror)} for each type in {@code
      * types}.
      *
-     * @param types AnnotateTypeMirrors used to find instantiations
-     * @param polyTypes AnnotatedTypeMirrors that may have polymorphic qualifiers
+     * @param types the AnnotateTypeMirrors used to find instantiations
+     * @param polyTypes the AnnotatedTypeMirrors that may have polymorphic qualifiers
      * @return a mapping of polymorphic qualifiers to their instantiations
      */
     private AnnotationMirrorMap<AnnotationMirror> visit(
@@ -477,8 +479,8 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
      * Creates a mapping of polymorphic qualifiers to their instantiations by visiting each
      * composite type in {@code type}.
      *
-     * @param type AnnotateTypeMirror used to find instantiations
-     * @param polyType AnnotatedTypeMirror that may have polymorphic qualifiers
+     * @param type the AnnotatedTypeMirror used to find instantiations
+     * @param polyType the AnnotatedTypeMirror that may have polymorphic qualifiers
      * @return a mapping of polymorphic qualifiers to their instantiations
      */
     private AnnotationMirrorMap<AnnotationMirror> visit(
@@ -492,7 +494,7 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
         if (wildcardType.getExtendsBound().getKind() == TypeKind.WILDCARD) {
           wildcardType = (AnnotatedWildcardType) wildcardType.getExtendsBound();
         }
-        if (wildcardType.isUninferredTypeArgument()) {
+        if (wildcardType.isTypeArgOfRawType()) {
           return mapQualifierToPoly(wildcardType.getExtendsBound(), polyType);
         }
 
@@ -513,14 +515,6 @@ public abstract class AbstractQualifierPolymorphism implements QualifierPolymorp
       return visit(asSuper, polyType, null);
     }
 
-    @Override
-    protected String defaultErrorMessage(
-        AnnotatedTypeMirror type1, AnnotatedTypeMirror type2, Void aVoid) {
-      return String.format(
-          "AbstractQualifierPolymorphism: Unexpected combination: type1: %s (%s) type2: %s (%s).",
-          type1, type1.getKind(), type2, type2.getKind());
-    }
-
     @Override
     public AnnotationMirrorMap<AnnotationMirror> visitArray_Array(
         AnnotatedArrayType type1, AnnotatedArrayType type2, Void aVoid) {
