@@ -47,10 +47,13 @@ import javax.lang.model.type.DeclaredType;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.type.TypeMirror;
 import javax.lang.model.util.Types;
-import javax.tools.Diagnostic.Kind;
+import javax.tools.Diagnostic;
 import org.checkerframework.checker.compilermsgs.qual.CompilerMessageKey;
+import org.checkerframework.checker.nullness.qual.EnsuresNonNull;
 import org.checkerframework.checker.nullness.qual.MonotonicNonNull;
+import org.checkerframework.checker.nullness.qual.NonNull;
 import org.checkerframework.checker.nullness.qual.Nullable;
+import org.checkerframework.checker.regex.qual.Regex;
 import org.checkerframework.dataflow.expression.ArrayAccess;
 import org.checkerframework.dataflow.expression.ArrayCreation;
 import org.checkerframework.dataflow.expression.BinaryOperation;
@@ -60,6 +63,7 @@ import org.checkerframework.dataflow.expression.FormalParameter;
 import org.checkerframework.dataflow.expression.JavaExpression;
 import org.checkerframework.dataflow.expression.LocalVariable;
 import org.checkerframework.dataflow.expression.MethodCall;
+import org.checkerframework.dataflow.expression.SuperReference;
 import org.checkerframework.dataflow.expression.ThisReference;
 import org.checkerframework.dataflow.expression.UnaryOperation;
 import org.checkerframework.dataflow.expression.ValueLiteral;
@@ -90,13 +94,14 @@ public class JavaExpressionParseUtil {
    * Anchored pattern for a formal parameter use; matches a string that is exactly a formal
    * parameter use.
    */
-  protected static final Pattern ANCHORED_PARAMETER_PATTERN =
+  protected static final @Regex(1) Pattern ANCHORED_PARAMETER_PATTERN =
       Pattern.compile("^" + PARAMETER_REGEX + "$");
 
   /**
    * Unanchored pattern for a formal parameter use; can be used to find all formal parameter uses.
    */
-  protected static final Pattern UNANCHORED_PARAMETER_PATTERN = Pattern.compile(PARAMETER_REGEX);
+  protected static final @Regex(1) Pattern UNANCHORED_PARAMETER_PATTERN =
+      Pattern.compile(PARAMETER_REGEX);
 
   /**
    * Parsable replacement for formal parameter references. It is parsable because it is a Java
@@ -108,7 +113,7 @@ public class JavaExpressionParseUtil {
   private static final int PARAMETER_PREFIX_LENGTH = PARAMETER_PREFIX.length();
 
   /** A pattern that matches the start of a formal parameter in "#2" syntax. */
-  private static Pattern FORMAL_PARAMETER = Pattern.compile("#(\\d)");
+  private static final Pattern FORMAL_PARAMETER = Pattern.compile("#(\\d)");
 
   /** The replacement for a formal parameter in "#2" syntax. */
   private static final String PARAMETER_REPLACEMENT = PARAMETER_PREFIX + "$1";
@@ -220,6 +225,7 @@ public class JavaExpressionParseUtil {
      * appears in the expression.
      */
     private final @Nullable ThisReference thisReference;
+
     /**
      * For each formal parameter, the expression to which to parse it. For example, the second
      * (index 1) element of the list is what "#2" parses to. If this field is {@code null}, a parse
@@ -231,7 +237,7 @@ public class JavaExpressionParseUtil {
      * Create a new ExpressionToJavaExpressionVisitor.
      *
      * @param enclosingType type of the class that encloses the JavaExpression
-     * @param thisReference JavaExpression to which to parse "this", or null if "this" should not
+     * @param thisReference a JavaExpression to which to parse "this", or null if "this" should not
      *     appear in the expression
      * @param parameters list of JavaExpressions to which to parse a formal parameter reference such
      *     as "#2", or null if parameters should not appear in the expression
@@ -262,7 +268,7 @@ public class JavaExpressionParseUtil {
      *
      * @param expr the JavaParser {@link Expression} to convert
      * @param enclosingType type of the class that encloses the JavaExpression
-     * @param thisReference JavaExpression to which to parse "this", or null if "this" should not
+     * @param thisReference a JavaExpression to which to parse "this", or null if "this" should not
      *     appear in the expression
      * @param parameters list of JavaExpressions to which to parse parameters, or null if parameters
      *     should not appear in the expression
@@ -300,6 +306,7 @@ public class JavaExpressionParseUtil {
      * Initializes the {@code resolver} field if necessary. Does nothing on invocations after the
      * first.
      */
+    @EnsuresNonNull("resolver")
     private void setResolverField() {
       if (resolver == null) {
         resolver = new Resolver(env);
@@ -360,13 +367,17 @@ public class JavaExpressionParseUtil {
 
     @Override
     public JavaExpression visit(SuperExpr n, Void aVoid) {
+      if (thisReference == null) {
+        throw new ParseRuntimeException(
+            constructJavaExpressionParseError("super", "\"super\" isn't allowed here"));
+      }
       // super literal
       TypeMirror superclass = TypesUtils.getSuperclass(enclosingType, types);
       if (superclass == null) {
         throw new ParseRuntimeException(
             constructJavaExpressionParseError("super", enclosingType + " has no superclass"));
       }
-      return new ThisReference(superclass);
+      return new SuperReference(superclass);
     }
 
     // expr is an expression in parentheses.
@@ -536,7 +547,8 @@ public class JavaExpressionParseUtil {
      * @return the {@code ClassName} for {@code identifier}, or null if it is not a class name
      */
     protected @Nullable ClassName getIdentifierAsUnqualifiedClassName(String identifier) {
-      // Is identifier an inner class of enclosingType or of any enclosing class of enclosingType?
+      // Is identifier an inner class of enclosingType or of any enclosing class of
+      // enclosingType?
       TypeMirror searchType = enclosingType;
       while (searchType.getKind() == TypeKind.DECLARED) {
         DeclaredType searchDeclaredType = (DeclaredType) searchType;
@@ -550,6 +562,8 @@ public class JavaExpressionParseUtil {
         searchType = getTypeOfEnclosingClass(searchDeclaredType);
       }
 
+      setResolverField();
+
       if (enclosingType.getKind() == TypeKind.DECLARED) {
         // Is identifier in the same package as this?
         PackageSymbol packageSymbol =
@@ -598,6 +612,7 @@ public class JavaExpressionParseUtil {
      */
     protected @Nullable FieldAccess getIdentifierAsFieldAccess(
         JavaExpression receiverExpr, String identifier) {
+      setResolverField();
       // Find the field element.
       TypeMirror enclosingTypeOfField = receiverExpr.getType();
       VariableElement fieldElem;
@@ -620,9 +635,18 @@ public class JavaExpressionParseUtil {
           // field not found.
           return null;
         }
+        if (receiverExpr instanceof SuperReference
+            && thisReference.getType().getKind() == TypeKind.DECLARED) {
+          Element thisFieldElem =
+              resolver.findField(identifier, thisReference.getType(), pathToCompilationUnit);
+          if (thisFieldElem == null) {
+            receiverExpr = thisReference;
+          }
+        }
       }
 
-      // Construct a FieldAccess expression.
+      // `fieldElem` is now set.  Construct a FieldAccess expression.
+
       if (ElementUtils.isStatic(fieldElem)) {
         Element classElem = fieldElem.getEnclosingElement();
         JavaExpression staticClassReceiver = new ClassName(ElementUtils.getType(classElem));
@@ -780,8 +804,11 @@ public class JavaExpressionParseUtil {
       throw constructJavaExpressionParseError(methodName, "no such method");
     }
 
-    // expr is a field access, a fully qualified class name, or a class name qualified with
-    // another class name (e.g. {@code OuterClass.InnerClass})
+    // `expr` should be a field access, a fully qualified class name, or a class name qualified
+    // with another class name (e.g. {@code OuterClass.InnerClass}).  If the expression refers
+    // to a class that is not available to the resolver (the class wasn't passed to javac on
+    // the command line), then the argument can be "outerpackage.innerpackage", which will lead
+    // to a confusing error message.
     @Override
     public JavaExpression visit(FieldAccessExpr expr, Void aVoid) {
       setResolverField();
@@ -840,12 +867,10 @@ public class JavaExpressionParseUtil {
 
     @Override
     public JavaExpression visit(ArrayCreationExpr expr, Void aVoid) {
-      List<JavaExpression> dimensions =
+      List<@Nullable JavaExpression> dimensions =
           CollectionsPlume.mapList(
               (ArrayCreationLevel dimension) ->
-                  dimension.getDimension().isPresent()
-                      ? dimension.getDimension().get().accept(this, aVoid)
-                      : null,
+                  dimension.getDimension().map(dim -> dim.accept(this, aVoid)).orElse(null),
               expr.getLevels());
 
       List<JavaExpression> initializers;
@@ -927,13 +952,15 @@ public class JavaExpressionParseUtil {
       TypeMirror rightType = rightJe.getType();
       TypeMirror type;
       // isSubtype() first does the cheaper test isSameType(), so no need to do it here.
-      if (types.isSubtype(leftType, rightType)) {
+      if (expr.getOperator() == BinaryExpr.Operator.PLUS
+          && (TypesUtils.isString(leftType) || TypesUtils.isString(rightType))) {
+        // JLS 15.18.1 says, "If only one operand expression is of type String, then string
+        // conversion is performed on the other operand to produce a string at run time."
+        type = stringTypeMirror;
+      } else if (types.isSubtype(leftType, rightType)) {
         type = rightType;
       } else if (types.isSubtype(rightType, leftType)) {
         type = leftType;
-      } else if (expr.getOperator() == BinaryExpr.Operator.PLUS
-          && (TypesUtils.isString(leftType) || TypesUtils.isString(rightType))) {
-        type = stringTypeMirror;
       } else {
         throw new ParseRuntimeException(
             constructJavaExpressionParseError(
@@ -1053,14 +1080,17 @@ public class JavaExpressionParseUtil {
   public static int parameterIndex(String s) {
     Matcher matcher = ANCHORED_PARAMETER_PATTERN.matcher(s);
     if (matcher.find()) {
-      return Integer.parseInt(matcher.group(1));
+      @SuppressWarnings(
+          "nullness:assignment") // group 1 is non-null due to the structure of the regex
+      @NonNull String group1 = matcher.group(1);
+      return Integer.parseInt(group1);
     }
     return -1;
   }
 
-  ///////////////////////////////////////////////////////////////////////////
-  /// Contexts
-  ///
+  // ///////////////////////////////////////////////////////////////////////////
+  // Contexts
+  //
 
   /**
    * Returns the type of the innermost enclosing class. Returns Type.noType if the type is a
@@ -1092,19 +1122,23 @@ public class JavaExpressionParseUtil {
     }
   }
 
-  ///////////////////////////////////////////////////////////////////////////
-  /// Exceptions
-  ///
+  // ///////////////////////////////////////////////////////////////////////////
+  // Exceptions
+  //
 
   /**
    * An exception that indicates a parse error. Call {@link #getDiagMessage} to obtain a {@link
    * DiagMessage} that can be used for error reporting.
    */
   public static class JavaExpressionParseException extends Exception {
+    /** The serial version identifier. */
     private static final long serialVersionUID = 2L;
+
     /** The error message key. */
-    private @CompilerMessageKey String errorKey;
+    private final @CompilerMessageKey String errorKey;
+
     /** The arguments to the error message key. */
+    @SuppressWarnings("serial") // I do not intend to serialize JavaExpressionParseException objects
     public final Object[] args;
 
     /**
@@ -1142,7 +1176,7 @@ public class JavaExpressionParseUtil {
      * @return a DiagMessage that can be used for error reporting
      */
     public DiagMessage getDiagMessage() {
-      return new DiagMessage(Kind.ERROR, errorKey, args);
+      return new DiagMessage(Diagnostic.Kind.ERROR, errorKey, args);
     }
 
     public boolean isFlowParseError() {
