@@ -3,7 +3,6 @@ package org.checkerframework.javacutil;
 import com.sun.source.tree.AnnotationTree;
 import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.ModifiersTree;
-import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.model.JavacElements;
 import java.lang.annotation.Annotation;
@@ -13,18 +12,14 @@ import java.lang.annotation.Target;
 import java.lang.reflect.Array;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.Comparator;
 import java.util.EnumSet;
-import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.NavigableSet;
 import java.util.Objects;
 import java.util.Set;
 import java.util.StringJoiner;
-import java.util.TreeMap;
 import java.util.TreeSet;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.AnnotationValue;
@@ -37,17 +32,22 @@ import javax.lang.model.type.DeclaredType;
 import javax.lang.model.util.ElementFilter;
 import org.checkerframework.checker.interning.qual.CompareToMethod;
 import org.checkerframework.checker.interning.qual.EqualsMethod;
+import org.checkerframework.checker.interning.qual.Interned;
 import org.checkerframework.checker.nullness.qual.NonNull;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.signature.qual.BinaryName;
 import org.checkerframework.checker.signature.qual.CanonicalName;
 import org.checkerframework.dataflow.qual.Pure;
 import org.checkerframework.dataflow.qual.SideEffectFree;
-import org.checkerframework.framework.util.DefaultAnnotationFormatter;
 import org.checkerframework.javacutil.AnnotationBuilder.CheckerFrameworkAnnotationMirror;
+import org.plumelib.util.ArrayMap;
 import org.plumelib.util.CollectionsPlume;
 
-/** A utility class for working with annotations. */
+/**
+ * A utility class for working with annotations.
+ *
+ * <p>Note: {@code AnnotationMirror}s are immutable.
+ */
 public class AnnotationUtils {
 
   // Class cannot be instantiated.
@@ -64,6 +64,10 @@ public class AnnotationUtils {
   /**
    * Returns the fully-qualified name of an annotation as a String.
    *
+   * <p>This method is efficient for {@code AnnotationBuilder.CheckerFrameworkAnnotationMirror}, for
+   * which it looks up the name. This method may be inefficient for other subclasses of {@code
+   * AnnotationMirror}, because it may compute a new string.
+   *
    * @param annotation the annotation whose name to return
    * @return the fully-qualified name of an annotation as a String
    */
@@ -71,13 +75,34 @@ public class AnnotationUtils {
     if (annotation instanceof AnnotationBuilder.CheckerFrameworkAnnotationMirror) {
       return ((AnnotationBuilder.CheckerFrameworkAnnotationMirror) annotation).annotationName;
     }
-    final DeclaredType annoType = annotation.getAnnotationType();
-    final TypeElement elm = (TypeElement) annoType.asElement();
+    DeclaredType annoType = annotation.getAnnotationType();
+    TypeElement elm = (TypeElement) annoType.asElement();
     @SuppressWarnings("signature:assignment") // JDK needs annotations
     @CanonicalName String name = elm.getQualifiedName().toString();
     return name;
   }
 
+  /**
+   * Returns the fully-qualified name of an annotation as a String.
+   *
+   * <p>This is more efficient than calling {@link annotationName} and {@link
+   * java.lang.String#intern}.
+   *
+   * @param annotation the annotation whose name to return
+   * @return the fully-qualified name of an annotation as a String
+   */
+  public static final @CanonicalName @Interned String annotationNameInterned(
+      AnnotationMirror annotation) {
+    if (annotation instanceof AnnotationBuilder.CheckerFrameworkAnnotationMirror) {
+      return ((AnnotationBuilder.CheckerFrameworkAnnotationMirror) annotation).annotationName;
+    }
+    DeclaredType annoType = annotation.getAnnotationType();
+    TypeElement elm = (TypeElement) annoType.asElement();
+    @SuppressWarnings("signature:assignment") // JDK needs annotations
+    @CanonicalName String name = elm.getQualifiedName().toString();
+    return name.intern();
+  }
+
   /**
    * Returns the binary name of an annotation as a String.
    *
@@ -85,8 +110,8 @@ public class AnnotationUtils {
    * @return the binary name of an annotation as a String
    */
   public static final @BinaryName String annotationBinaryName(AnnotationMirror annotation) {
-    final DeclaredType annoType = annotation.getAnnotationType();
-    final TypeElement elm = (TypeElement) annoType.asElement();
+    DeclaredType annoType = annotation.getAnnotationType();
+    TypeElement elm = (TypeElement) annoType.asElement();
     return ElementUtils.getBinaryName(elm);
   }
 
@@ -114,6 +139,40 @@ public class AnnotationUtils {
     return sameElementValues(a1, a2);
   }
 
+  /**
+   * Return -1, 0, or 1 depending on whether the name of a1 is less, equal to, or greater than that
+   * of a2 (lexicographically).
+   *
+   * @param a1 the first AnnotationMirror to compare
+   * @param a2 the second AnnotationMirror to compare
+   * @return true iff a1 and a2 have the same annotation name
+   * @see #areSame(AnnotationMirror, AnnotationMirror)
+   */
+  @EqualsMethod
+  public static int compareByName(AnnotationMirror a1, AnnotationMirror a2) {
+    if (a1 == a2) {
+      return 0;
+    }
+    if (a1 == null || a2 == null) {
+      throw new BugInCF("Unexpected null argument:  compareByName(%s, %s)", a1, a2);
+    }
+
+    // This is largely duplicated code.  The point of this block is that
+    // the `if (name1 == name2)` test is very fast.
+    if (a1 instanceof CheckerFrameworkAnnotationMirror
+        && a2 instanceof CheckerFrameworkAnnotationMirror) {
+      @Interned @CanonicalName String name1 = ((CheckerFrameworkAnnotationMirror) a1).annotationName;
+      @Interned @CanonicalName String name2 = ((CheckerFrameworkAnnotationMirror) a2).annotationName;
+      if (name1 == name2) {
+        return 0;
+      } else {
+        return name1.compareTo(name2);
+      }
+    }
+
+    return annotationName(a1).compareTo(annotationName(a2));
+  }
+
   /**
    * Return true iff a1 and a2 have the same annotation type.
    *
@@ -124,20 +183,7 @@ public class AnnotationUtils {
    */
   @EqualsMethod
   public static boolean areSameByName(AnnotationMirror a1, AnnotationMirror a2) {
-    if (a1 == a2) {
-      return true;
-    }
-    if (a1 == null || a2 == null) {
-      throw new BugInCF("Unexpected null argument:  areSameByName(%s, %s)", a1, a2);
-    }
-
-    if (a1 instanceof CheckerFrameworkAnnotationMirror
-        && a2 instanceof CheckerFrameworkAnnotationMirror) {
-      return ((CheckerFrameworkAnnotationMirror) a1).annotationName
-          == ((CheckerFrameworkAnnotationMirror) a2).annotationName;
-    }
-
-    return annotationName(a1).equals(annotationName(a2));
+    return compareByName(a1, a2) == 0;
   }
 
   /**
@@ -153,7 +199,7 @@ public class AnnotationUtils {
   }
 
   /**
-   * Checks that the annotation {@code am} has the name of {@code annoClass}. Values are ignored.
+   * Checks that the annotation {@code am} has class {@code annoClass}. Values are ignored.
    *
    * <p>This method is not very efficient. It is more efficient to use {@code
    * AnnotatedTypeFactory#areSameByClass} or {@link #areSameByName}.
@@ -187,9 +233,9 @@ public class AnnotationUtils {
       return areSame(c1.iterator().next(), c2.iterator().next());
     }
 
-    // while loop depends on SortedSet implementation.
-    NavigableSet<AnnotationMirror> s1 = createAnnotationSet();
-    NavigableSet<AnnotationMirror> s2 = createAnnotationSet();
+    // while loop depends on NavigableSet implementation.
+    AnnotationMirrorSet s1 = new AnnotationMirrorSet();
+    AnnotationMirrorSet s2 = new AnnotationMirrorSet();
     s1.addAll(c1);
     s2.addAll(c2);
     Iterator<AnnotationMirror> iter1 = s1.iterator();
@@ -336,19 +382,20 @@ public class AnnotationUtils {
   }
 
   /**
-   * Provide ordering for {@link AnnotationMirror}s. AnnotationMirrors are first compared by their
-   * fully-qualified names, then by their element values in order of the name of the element.
+   * Provide an ordering for {@link AnnotationMirror}s. AnnotationMirrors are first compared by
+   * their fully-qualified names, then by their element values in order of the name of the element.
    *
    * @param a1 the first annotation
    * @param a2 the second annotation
    * @return an ordering over AnnotationMirrors based on their name and values
    */
   public static int compareAnnotationMirrors(AnnotationMirror a1, AnnotationMirror a2) {
-    if (!AnnotationUtils.areSameByName(a1, a2)) {
-      return annotationName(a1).compareTo(annotationName(a2));
+    int nameComparison = compareByName(a1, a2);
+    if (nameComparison != 0) {
+      return nameComparison;
     }
 
-    // The annotations have the same name, but different values, so compare values.
+    // The annotations have the same name, but possibly different values, so compare values.
     Map<? extends ExecutableElement, ? extends AnnotationValue> vals1 = a1.getElementValues();
     Map<? extends ExecutableElement, ? extends AnnotationValue> vals2 = a2.getElementValues();
     Set<ExecutableElement> sortedElements =
@@ -356,6 +403,7 @@ public class AnnotationUtils {
     sortedElements.addAll(
         ElementFilter.methodsIn(a1.getAnnotationType().asElement().getEnclosedElements()));
 
+    // getDefaultValue() returns null if the method is not an annotation interface element.
     for (ExecutableElement meth : sortedElements) {
       AnnotationValue aval1 = vals1.get(meth);
       if (aval1 == null) {
@@ -374,11 +422,11 @@ public class AnnotationUtils {
   }
 
   /**
-   * Return 0 iff the two AnnotationValue objects are the same.
+   * Compare the two AnnotationValue objects for order.
    *
    * @param av1 the first AnnotationValue to compare
    * @param av2 the second AnnotationValue to compare
-   * @return 0 if the two annotation values are the same
+   * @return -1 if the first is lesser, 0 if they are the same, or 1 if the first is greater
    */
   @CompareToMethod
   private static int compareAnnotationValue(AnnotationValue av1, AnnotationValue av2) {
@@ -433,7 +481,8 @@ public class AnnotationUtils {
     } else if ((val1 instanceof AnnotationMirror) && (val2 instanceof AnnotationMirror)) {
       return compareAnnotationMirrors((AnnotationMirror) val1, (AnnotationMirror) val2);
     } else if ((val1 instanceof AnnotationValue) && (val2 instanceof AnnotationValue)) {
-      // This case occurs because of the recursive call when comparing arrays of annotation values.
+      // This case occurs because of the recursive call when comparing arrays of annotation
+      // values.
       return compareAnnotationValue((AnnotationValue) val1, (AnnotationValue) val2);
     }
 
@@ -453,65 +502,6 @@ public class AnnotationUtils {
     return result;
   }
 
-  /**
-   * Create a map suitable for storing {@link AnnotationMirror} as keys.
-   *
-   * <p>It can store one instance of {@link AnnotationMirror} of a given declared type, regardless
-   * of the annotation element values.
-   *
-   * @param <V> the value of the map
-   * @return a new map with {@link AnnotationMirror} as key
-   */
-  public static <V> Map<AnnotationMirror, V> createAnnotationMap() {
-    return new TreeMap<>(AnnotationUtils::compareAnnotationMirrors);
-  }
-
-  /**
-   * Constructs a {@link Set} for storing {@link AnnotationMirror}s.
-   *
-   * <p>It stores at most once instance of {@link AnnotationMirror} of a given type, regardless of
-   * the annotation element values.
-   *
-   * @return a sorted new set to store {@link AnnotationMirror} as element
-   */
-  public static NavigableSet<AnnotationMirror> createAnnotationSet() {
-    return new TreeSet<>(AnnotationUtils::compareAnnotationMirrors);
-  }
-
-  /**
-   * Constructs a {@link Set} for storing {@link AnnotationMirror}s contain all the annotations in
-   * {@code annos}.
-   *
-   * <p>It stores at most once instance of {@link AnnotationMirror} of a given type, regardless of
-   * the annotation element values.
-   *
-   * @param annos a Collection of AnnotationMirrors to put in the created set
-   * @return a sorted new set to store {@link AnnotationMirror} as element
-   */
-  public static NavigableSet<AnnotationMirror> createAnnotationSet(
-      Collection<AnnotationMirror> annos) {
-    TreeSet<AnnotationMirror> set = new TreeSet<>(AnnotationUtils::compareAnnotationMirrors);
-    set.addAll(annos);
-    return set;
-  }
-
-  /**
-   * Constructs an unmodifiable {@link Set} for storing {@link AnnotationMirror}s contain all the
-   * annotations in {@code annos}.
-   *
-   * <p>It stores at most once instance of {@link AnnotationMirror} of a given type, regardless of
-   * the annotation element values.
-   *
-   * @param annos a Collection of AnnotationMirrors to put in the created set
-   * @return a sorted, unmodifiable, new set to store {@link AnnotationMirror} as element
-   */
-  public static NavigableSet<AnnotationMirror> createUnmodifiableAnnotationSet(
-      Collection<AnnotationMirror> annos) {
-    TreeSet<AnnotationMirror> set = new TreeSet<>(AnnotationUtils::compareAnnotationMirrors);
-    set.addAll(annos);
-    return Collections.unmodifiableNavigableSet(set);
-  }
-
   /**
    * Returns true if the given annotation has a @Inherited meta-annotation.
    *
@@ -574,7 +564,8 @@ public class AnnotationUtils {
       case TYPE_USE:
         return EnumSet.noneOf(ElementKind.class);
       default:
-        // TODO: Use MODULE enum constants directly instead of looking them up by name.  (Java 11)
+        // TODO: Use MODULE enum constants directly instead of looking them up by name.
+        // (Java 11)
         if (elementType.name().equals("MODULE")) {
           return EnumSet.of(ElementKind.valueOf("MODULE"));
         }
@@ -589,6 +580,45 @@ public class AnnotationUtils {
   // Annotation values: inefficient extractors that take an element name
   // **********************************************************************
 
+  /**
+   * Get the element with the name {@code elementName} of the annotation {@code anno}. The result
+   * has type {@code expectedType}. If there is no value for {@code elementName}, {@code
+   * defaultValue} is returned
+   *
+   * <p>This method is intended only for use when the class of the annotation is not on the user's
+   * classpath. This is for users of the Dataflow Framework that do not use the rest of the Checker
+   * Framework. Type-checkers can assume that checker-qual.jar is on the classpath and should use
+   * {@link #getElementValue(AnnotationMirror, ExecutableElement, Class)} or {@link
+   * #getElementValue(AnnotationMirror, ExecutableElement, Class, Object)}.
+   *
+   * @param anno the annotation whose element to access
+   * @param elementName the name of the element to access
+   * @param expectedType the type of the element and the return value
+   * @param defaultValue the value to return if the element is not present
+   * @param <T> the class of the type
+   * @return the value of the element with the given name
+   */
+  public static <T> T getElementValueNotOnClasspath(
+      AnnotationMirror anno, CharSequence elementName, Class<T> expectedType, T defaultValue) {
+    Map<? extends ExecutableElement, ? extends AnnotationValue> valmap = anno.getElementValues();
+
+    for (Map.Entry<? extends ExecutableElement, ? extends AnnotationValue> entry :
+        valmap.entrySet()) {
+      ExecutableElement elem = entry.getKey();
+      if (elem.getSimpleName().contentEquals(elementName)) {
+        AnnotationValue val = entry.getValue();
+        try {
+          return expectedType.cast(val.getValue());
+        } catch (ClassCastException e) {
+          throw new BugInCF(
+              "getElementValueNotOnClasspath(%s, %s, %s): val=%s, val.getValue()=%s [%s]",
+              anno, elementName, expectedType, val, val.getValue(), val.getValue().getClass());
+        }
+      }
+    }
+    return defaultValue;
+  }
+
   /**
    * Returns the values of an annotation's elements, including defaults. The method with the same
    * name in JavacElements cannot be used directly, because it includes a cast to
@@ -601,12 +631,11 @@ public class AnnotationUtils {
    * @see JavacElements#getElementValuesWithDefaults(AnnotationMirror)
    * @param ad annotation to examine
    * @return the values of the annotation's elements, including defaults
-   * @deprecated use a method that takes an {@link ExecutableElement}
    */
-  @Deprecated // 2021-03-29; do not remove, just make private
-  public static Map<? extends ExecutableElement, ? extends AnnotationValue>
+  private static Map<? extends ExecutableElement, ? extends AnnotationValue>
       getElementValuesWithDefaults(AnnotationMirror ad) {
-    Map<ExecutableElement, AnnotationValue> valMap = new HashMap<>();
+    // Most annotations have no elements.
+    Map<ExecutableElement, AnnotationValue> valMap = new ArrayMap<>(0);
     if (ad.getElementValues() != null) {
       valMap.putAll(ad.getElementValues());
     }
@@ -620,29 +649,6 @@ public class AnnotationUtils {
     return valMap;
   }
 
-  /**
-   * Verify whether the element with the name {@code elementName} exists in the annotation {@code
-   * anno}.
-   *
-   * <p>This method is intended for use only by the framework. Clients should use a method that
-   * takes an {@link ExecutableElement}.
-   *
-   * @param anno the annotation to examine
-   * @param elementName the name of the element
-   * @return whether the element exists in anno
-   * @deprecated use a method that takes an {@link ExecutableElement}
-   */
-  @Deprecated // 2021-03-30
-  public static boolean hasElementValue(AnnotationMirror anno, CharSequence elementName) {
-    Map<? extends ExecutableElement, ? extends AnnotationValue> valmap = anno.getElementValues();
-    for (ExecutableElement elem : valmap.keySet()) {
-      if (elem.getSimpleName().contentEquals(elementName)) {
-        return true;
-      }
-    }
-    return false;
-  }
-
   /**
    * Get the element with the name {@code elementName} of the annotation {@code anno}. The result
    * has type {@code expectedType}.
@@ -664,24 +670,22 @@ public class AnnotationUtils {
    * @deprecated use {@link #getElementValue(AnnotationMirror, ExecutableElement, Class)} or {@link
    *     #getElementValue(AnnotationMirror, ExecutableElement, Class, Object)}
    */
-  @Deprecated // for use only by the framework
+  @Deprecated // for use only by the framework, not by clients
   public static <T> T getElementValue(
       AnnotationMirror anno, CharSequence elementName, Class<T> expectedType, boolean useDefaults) {
     Map<? extends ExecutableElement, ? extends AnnotationValue> valmap;
     if (useDefaults) {
-      @SuppressWarnings(
-          "deprecation" // remove when getElementValuesWithDefaults is made private and
-      // non-deprecated
-      )
       Map<? extends ExecutableElement, ? extends AnnotationValue> valmapTmp =
           getElementValuesWithDefaults(anno);
       valmap = valmapTmp;
     } else {
       valmap = anno.getElementValues();
     }
-    for (ExecutableElement elem : valmap.keySet()) {
+    for (Map.Entry<? extends ExecutableElement, ? extends AnnotationValue> entry :
+        valmap.entrySet()) {
+      ExecutableElement elem = entry.getKey();
       if (elem.getSimpleName().contentEquals(elementName)) {
-        AnnotationValue val = valmap.get(elem);
+        AnnotationValue val = entry.getValue();
         try {
           return expectedType.cast(val.getValue());
         } catch (ClassCastException e) {
@@ -733,7 +737,7 @@ public class AnnotationUtils {
    */
   public static <T> @Nullable T getElementValueOrNull(
       AnnotationMirror anno, CharSequence elementName, Class<T> expectedType, boolean useDefaults) {
-    // This implementation permits getElementValue a more detailed error message than if
+    // This implementation permits getElementValue to give a more detailed error message than if
     // getElementValue called getElementValueOrNull and threw an error if the result was null.
     try {
       return getElementValue(anno, elementName, expectedType, useDefaults);
@@ -758,7 +762,7 @@ public class AnnotationUtils {
    */
   public static <T> @Nullable List<T> getElementValueArrayOrNull(
       AnnotationMirror anno, CharSequence elementName, Class<T> expectedType, boolean useDefaults) {
-    // This implementation permits getElementValue a more detailed error message than if
+    // This implementation permits getElementValue to give a more detailed error message than if
     // getElementValue called getElementValueOrNull and threw an error if the result was null.
     try {
       return getElementValueArray(anno, elementName, expectedType, useDefaults);
@@ -767,31 +771,6 @@ public class AnnotationUtils {
     }
   }
 
-  /**
-   * Get the element with the name {@code name} of the annotation {@code anno}. The result is an
-   * enum of type {@code T}.
-   *
-   * <p>This method is intended only for use by the framework. A checker implementation should use
-   * {@link #getElementValueEnum(AnnotationMirror, ExecutableElement, Class)} or {@link
-   * #getElementValueEnum(AnnotationMirror, ExecutableElement, Class, Enum)}.
-   *
-   * @param anno the annotation to disassemble
-   * @param elementName the name of the element to access
-   * @param expectedType the type of the element and the return value, an enum
-   * @param <T> the class of the type
-   * @param useDefaults whether to apply default values to the element
-   * @return the value of the element with the given name
-   * @deprecated use {@link #getElementValueEnum(AnnotationMirror, ExecutableElement, Class)} or
-   *     {@link #getElementValueEnum(AnnotationMirror, ExecutableElement, Class, Enum)}
-   */
-  @Deprecated // 2021-03-29
-  public static <T extends Enum<T>> T getElementValueEnum(
-      AnnotationMirror anno, CharSequence elementName, Class<T> expectedType, boolean useDefaults) {
-    VarSymbol vs = getElementValue(anno, elementName, VarSymbol.class, useDefaults);
-    T value = Enum.valueOf(expectedType, vs.getSimpleName().toString());
-    return value;
-  }
-
   /**
    * Get the element with the name {@code elementName} of the annotation {@code anno}, where the
    * element has an array type. One element of the result has type {@code expectedType}.
@@ -842,32 +821,6 @@ public class AnnotationUtils {
     return result;
   }
 
-  /**
-   * Get the element with the name {@code elementName} of the annotation {@code anno}, or the
-   * default value if no element is present explicitly, where the element has an array type and the
-   * elements are {@code Enum}s. One element of the result has type {@code expectedType}.
-   *
-   * <p>This method is intended only for use by the framework. A checker implementation should use
-   * {@link #getElementValueEnumArray(AnnotationMirror, ExecutableElement, Class)} or {@link
-   * #getElementValueEnumArray(AnnotationMirror, ExecutableElement, Class, Enum[])}.
-   *
-   * @param anno the annotation to disassemble
-   * @param elementName the name of the element to access
-   * @param expectedType the component type of the element and of the return type, an enum
-   * @param <T> the class of the type
-   * @param useDefaults whether to apply default values to the element
-   * @return the value of the element with the given name
-   * @deprecated use {@link #getElementValueEnumArray(AnnotationMirror, ExecutableElement, Class)}
-   *     or {@link #getElementValueEnumArray(AnnotationMirror, ExecutableElement, Class, Enum[])}
-   */
-  @Deprecated // 2021-03-29
-  public static <T extends Enum<T>> T[] getElementValueEnumArray(
-      AnnotationMirror anno, CharSequence elementName, Class<T> expectedType, boolean useDefaults) {
-    @SuppressWarnings("unchecked")
-    List<AnnotationValue> la = getElementValue(anno, elementName, List.class, useDefaults);
-    return annotationValueListToEnumArray(la, expectedType);
-  }
-
   /**
    * Get the Name of the class that is referenced by element {@code elementName}.
    *
@@ -894,28 +847,6 @@ public class AnnotationUtils {
     return result;
   }
 
-  /**
-   * Get the list of Names of the classes that are referenced by element {@code elementName}. It
-   * fails if the class wasn't found.
-   *
-   * <p>This method is intended only for use by the framework. A checker implementation should use
-   * {@link #getElementValueClassNames(AnnotationMirror, ExecutableElement)}.
-   *
-   * @param anno the annotation whose field to access; it must be present in the annotation
-   * @param annoElement the element/field of {@code anno} whose content is a list of classes
-   * @param useDefaults whether to apply default values to the element
-   * @return the names of classes in {@code anno.annoElement}
-   * @deprecated use {@link #getElementValueClassNames(AnnotationMirror,ExecutableElement)}
-   */
-  @Deprecated // 2021-03-29
-  public static List<@CanonicalName Name> getElementValueClassNames(
-      AnnotationMirror anno, CharSequence annoElement, boolean useDefaults) {
-    List<Type.ClassType> la =
-        getElementValueArray(anno, annoElement, Type.ClassType.class, useDefaults);
-    return CollectionsPlume.<Type.ClassType, @CanonicalName Name>mapList(
-        (Type.ClassType classType) -> classType.asElement().getQualifiedName(), la);
-  }
-
   // **********************************************************************
   // Annotation values: efficient extractors that take an ExecutableElement
   // **********************************************************************
@@ -936,7 +867,7 @@ public class AnnotationUtils {
    * @param <T> the class of the type
    * @return the value of the element with the given name
    */
-  public static <T> @Nullable T getElementValue(
+  public static <T> T getElementValue(
       AnnotationMirror anno, ExecutableElement element, Class<T> expectedType) {
     AnnotationValue av = anno.getElementValues().get(element);
     if (av == null) {
@@ -962,7 +893,7 @@ public class AnnotationUtils {
    * @param defaultValue the value to return if the element is not present
    * @return the value of the element with the given name
    */
-  public static <T> @Nullable T getElementValue(
+  public static <T> T getElementValue(
       AnnotationMirror anno, ExecutableElement element, Class<T> expectedType, T defaultValue) {
     AnnotationValue av = anno.getElementValues().get(element);
     if (av == null) {
@@ -1141,7 +1072,7 @@ public class AnnotationUtils {
     if (av == null) {
       throw new BugInCF("getElementValueArray(%s, %s, ...)", anno, element);
     }
-    return AnnotationUtils.annotationValueToList(av, expectedType);
+    return annotationValueToList(av, expectedType);
   }
 
   /**
@@ -1165,7 +1096,7 @@ public class AnnotationUtils {
     if (av == null) {
       return defaultValue;
     } else {
-      return AnnotationUtils.annotationValueToList(av, expectedType);
+      return annotationValueToList(av, expectedType);
     }
   }
 
@@ -1388,7 +1319,7 @@ public class AnnotationUtils {
    * <p>To test containment, use {@link #annotationValueContains(AnnotationValue, String)} or {@link
    * #annotationValueContainsToString(AnnotationValue, String)}.
    *
-   * @param avList an AnnotationValue that is null or a list of Strings
+   * @param avList an AnnotationValue that is a list of Strings
    * @param expectedType the component type of the argument and of the return type, an enum
    * @param <T> the class of the type
    * @return the annotation value, converted to a list
@@ -1448,9 +1379,9 @@ public class AnnotationUtils {
    * @param <T> the key type
    */
   public static <T extends @NonNull Object> void updateMappingToImmutableSet(
-      Map<T, Set<AnnotationMirror>> map, T key, Set<AnnotationMirror> newQual) {
+      Map<T, AnnotationMirrorSet> map, T key, AnnotationMirrorSet newQual) {
 
-    Set<AnnotationMirror> result = AnnotationUtils.createAnnotationSet();
+    AnnotationMirrorSet result = new AnnotationMirrorSet();
     // TODO: if T is also an AnnotationMirror, should we use areSame?
     if (!map.containsKey(key)) {
       result.addAll(newQual);
@@ -1458,7 +1389,8 @@ public class AnnotationUtils {
       result.addAll(map.get(key));
       result.addAll(newQual);
     }
-    map.put(key, Collections.unmodifiableSet(result));
+    result.makeUnmodifiable();
+    map.put(key, result);
   }
 
   /**
@@ -1468,9 +1400,9 @@ public class AnnotationUtils {
    * @param constructorDeclaration declaration tree of constructor
    * @return set of annotations explicit on the resulting type of the constructor
    */
-  public static Set<AnnotationMirror> getExplicitAnnotationsOnConstructorResult(
+  public static AnnotationMirrorSet getExplicitAnnotationsOnConstructorResult(
       MethodTree constructorDeclaration) {
-    Set<AnnotationMirror> annotationSet = AnnotationUtils.createAnnotationSet();
+    AnnotationMirrorSet annotationSet = new AnnotationMirrorSet();
     ModifiersTree modifiersTree = constructorDeclaration.getModifiers();
     if (modifiersTree != null) {
       List<? extends AnnotationTree> annotationTrees = modifiersTree.getAnnotations();
@@ -1542,7 +1474,7 @@ public class AnnotationUtils {
    * @return the string representation, using simple (not fully-qualified) names
    */
   @SideEffectFree
-  public static String toStringSimple(Set<AnnotationMirror> annos) {
+  public static String toStringSimple(AnnotationMirrorSet annos) {
     DefaultAnnotationFormatter defaultAnnotationFormatter = new DefaultAnnotationFormatter();
     StringJoiner result = new StringJoiner(" ");
     for (AnnotationMirror am : annos) {
